      SUBROUTINE BIDIAG(U,C,VH,M,MMAX)
C
C The subroutine BIDIAG converts the complex M x M matrix C into upper
C bidiagonal form by a series of Householder unitary transformations,
C applied alternately, to columns by pre-multiplication and to rows by
C post-multiplication. Finally, rows are scaled by pre-multiplication
C with a unitary matrix and, alternately, columns are scaled by
C post-multiplication with a unitary matrix, to reduce the phase of diagonal
C elements to zero and, alternately, to reduce the phase of above diagonal
C elements to zero, producing an upper bidiagonal matrix with real and
C positive elements. Thus, the final, real and positive upper bidiagonal
C matrix B is found as B=U.C.VH, a unitary similarity transformation of C.
C B is returned in C. The unitary matrices U and VH are also returned by
C the subroutine.
C 
      DOUBLE COMPLEX C(MMAX,MMAX),V(MMAX),HU(MMAX),
     1U(MMAX,MMAX),VH(MMAX,MMAX),H0(MMAX),H1(MMAX),H2(MMAX),CONST
      DOUBLE PRECISION EMAG 
C Initialize unitary matrices U and VH.
      DO 10 I=1,M
        DO 11 J=1,M
          U(I,J)=(0.D0,0.D0)
          VH(I,J)=(0.D0,0.D0)
 11     CONTINUE
        U(I,I)=(1.D0,0.D0)
        VH(I,I)=(1.D0,0.D0)  
 10   CONTINUE  
C Apply Householder annihilation.
      MM1=M-1
      DO 12 N=1,MM1
C Put Nth column in V.
        DO 13 I=1,M
          V(I)=C(I,N)
 13     CONTINUE
C Annihilate Nth column.  
        CALL HHOLDER(V,HU,N,M,MMAX)
C Multiply Hermitian transpose of Householder vector HU
C into unitary matrix U and into transformed matrix B.
        DO 14 I=1,M
          H1(I)=(0.D0,0.D0)
          H2(I)=(0.D0,0.D0)
          DO 14 J=N,M
            H0(J)=DCONJG(HU(J))
            H1(I)=H1(I)+H0(J)*U(J,I)
            H2(I)=H2(I)+H0(J)*C(J,I)
 14     CONTINUE
C Update unitary matrix U and transformed matrix B.
        DO 15 I=N,M
          DO 15 J=1,M
            U(I,J)=U(I,J)-2.D0*HU(I)*H1(J)
            C(I,J)=C(I,J)-2.D0*HU(I)*H2(J)
 15     CONTINUE
C Overwrite annihilated Nth column.
        DO 16 I=1,M
          C(I,N)=V(I)
 16     CONTINUE  
        IF(N.EQ.MM1) GO TO 12       
C Put Nth row in V.
        DO 17 J=1,M
          V(J)=C(N,J)
 17     CONTINUE
        NP1=N+1
C Annihilate Nth row.
        CALL HHOLDER(V,HU,NP1,M,MMAX)
C Multiply unitary matrix VH and transformed matrix C
C into Householder vector HU.
        DO 18 I=1,M
          H1(I)=(0.D0,0.D0)
          H2(I)=(0.D0,0.D0)
          DO 18 J=NP1,M
            H0(J)=DCONJG(HU(J))
            H1(I)=H1(I)+VH(I,J)*H0(J)
            H2(I)=H2(I)+C(I,J)*H0(J)
 18     CONTINUE
C Update unitary matrix VH and transformed matrix B.
        DO 19 I=1,M
          DO 19 J=NP1,M
            VH(I,J)=VH(I,J)-2.D0*H1(I)*HU(J)
            C(I,J)=C(I,J)-2.D0*H2(I)*HU(J)
 19     CONTINUE    
C Overwrite annihlated Nth row.
        DO 20 J=1,M
          C(N,J)=V(J)
 20     CONTINUE     
 12   CONTINUE
C Convert upper diagonal matrix B to a real, positive matrix.
      N=1
      CONST=(1.D0,0.D0)
 21   CONTINUE
      NP1=N+1
C Find magnitudes of current row elements.
      EMAG=DREAL(C(N,N)*DCONJG(C(N,N)))
      EMAG=DSQRT(EMAG)
C Update unitary matrix U.
      CONST=DCONJG(CONST)*DCONJG(C(N,N))/EMAG
C Scale current row.
      DO 22 I=1,M
        U(N,I)=CONST*U(N,I)
 22   CONTINUE
C Replace diagonal element.  
      C(N,N)=DCMPLX(EMAG,0.D0)
      IF(N.EQ.M) GO TO 23
      EMAG=DREAL(C(N,NP1)*DCONJG(C(N,NP1)))
      EMAG=DSQRT(EMAG)
C Update unitary matrix VH.
      CONST=DCONJG(CONST)*DCONJG(C(N,NP1))/EMAG
C Scale next column.
      DO 24 I=1,M
         VH(I,NP1)=CONST*VH(I,NP1)
 24   CONTINUE
C Replace above diagonal element.   
      C(N,NP1)=DCMPLX(EMAG,0.D0)
      N=N+1
      GO TO 21
 23   CONTINUE
      RETURN
      END
