      SUBROUTINE BPEC(M,NP1,F,G,PEF,PER)
C
C Subroutine BPEC finds the prediction error filter coefficients G
C of length NP1 from those of length N for the time sequence F of length M.
C It also returns the forward and backward prediction error sequences.
C
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DIMENSION F(M),G(M),H(M),O(M),PEF(M),PER(M)
C Find previous recursion number.
      N=NP1-1
      IF(N.NE.0) GO TO 10
C If previous recursion number is zero, set forward and backward
C prediction error sequences to given data sequence.
      DO 11 J=1,M
        PEF(J)=F(J)
        PER(J)=F(J)
 11   CONTINUE
C Find next member of prediction error filter G(NP1).
 10   SN=(0.D0,0.D0)
      SD=(0.D0,0.D0)
C Find upper limit of sum.
      JJ=M-N-1
      DO 12 J=1,JJ
        SN=SN-2.D0*PEF(J+N+1)*DCONJG(PER(J))
        SD=SD+PEF(J+N+1)*DCONJG(PEF(J+N+1))+PER(J)*DCONJG(PER(J))
 12   CONTINUE
      G(NP1)=SN/SD
C If previous recursion number is zero, do not update prediction error
C filter coefficients.
      IF(N.EQ.0) GO TO 13
C Update prediction error filter coefficients and store them in vector H(J).
      DO 14 J=1,N
        K=N-J+1
        H(J)=G(J)+G(NP1)*DCONJG(G(K))
 14   CONTINUE
C Put updated prediction error coefficients in G(J).
      DO 15 J=1,N
        G(J)=H(J)
 15   CONTINUE
C Update forward prediction error sequence and store in H(K).
 13   DO 16 J=1,JJ
        K=J+NP1
        H(K)=PEF(K)+G(NP1)*PER(J)
 16   CONTINUE
C Update backward prediction error sequence and store in O(J).
      DO 17 J=1,JJ
        O(J)=PER(J)+DCONJG(G(NP1))*PEF(J+NP1)
 17   CONTINUE
C Put updated forward and backward prediction error sequences
C in PEF(K) AND PER(J), respectively.
      DO 18 J=1,JJ
        K=J+NP1
        PEF(K)=H(K)
        PER(J)=O(J)
 18   CONTINUE
      RETURN
      END
