C                  PROGRAMME EIGENS.FOR
C
C EIGENS.FOR calculates the eigenvalues and eigenvectors of the
C subseismic equation for small constant stratification. The
C eleven matrices produced by MAT.FOR are read in from the file
C MATRES.DAT as well as the decompression factor from the file
C DECOMP.DAT produced by DECOMP.FOR.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BM(34980),BMP(34980),BML(11,34980),BMFS(264,264),
     1CMI(24,44,4),CMM(24,44,4),CI(24,44),CM(24,44),
     2EINVI(24,24),EINVM(24,24),ENAME(10),XX(12),W(12),
     3B(264),BI(264),DET(35772),RI(6),ZJ(11),
     4UD(11,6),PPD(11,6),QD(11,6),RRD(11,6),XD(11),YD(6),
     5RAD(100),FDC(100),BB(98,198),C(264,529),BMF(264,264)
      REAL*4 U(11,6),PP(11,6),Q(11,6),RR(11,6),X(11),Y(6),
     1 SUMFS,SUMDS,ALPHA,BETA,VINC,SCHT,ZETA2S,SIG4,SF
      OPEN(UNIT=1,FILE='matres.dat',STATUS='OLD')
      OPEN(UNIT=4,FILE='decomp.dat',STATUS='OLD')
      PI=3.141592653589793D0
C Set angular frequency of Earth's rotation (WGS84).
      WE=7.292115D-5
C Calculate number of hours in a semi-sidereal day. 
      HSSD=PI/(WE*3600.D0)
C Set maximum relative error of eigenvalue.
      RERRM=1.D-12
C Read in and write out Earth model name.
      READ(1,10)(ENAME(I),I=1,10)
 10   FORMAT(10A8)
      WRITE(6,11)
 11   FORMAT(10X,'Eigenvalue-eigenvector calculation for')      
      WRITE(6,12)(ENAME(I),I=1,10)
 12   FORMAT(10X,10A8)
C Write out headers.
      WRITE(6,13)
 13   FORMAT(1X,'1. rigid, non-gravitating boundaries,',
     1 ' neutral stratification (Poincare).')
      WRITE(6,14)
 14   FORMAT(1X,'2. elastic boundaries, neutral stratification.')
      WRITE(6,15)
 15   FORMAT(1X,'3. elastic boundaries, small stratification.')
      READ(5,*)ICASE
C Read in physical parameter values.
      READ(1,16)ICORE,ICOMP,IPAR,M,RIOB,RCMB
 16   FORMAT(4I4,2D23.15)
C Convert azimuthal number to a real variable.
      AZM=DFLOAT(M)
C Read in gravity at the inner core boundary, core-mantle boundary.
      READ(1,17)GZEROC,GZEROM
C Write out azimuthal number.
      WRITE(6,18)M
 18   FORMAT(1X,'azimuthal number =',I2)
C Write out inner core presence or absence.
      IF(ICORE.EQ.1) GO TO 19
      WRITE(6,20)
 20   FORMAT(1X,'inner core present.')
      GO TO 21
 19   WRITE(6,22)
 22   FORMAT(1X,'no inner core.')
C Write out compressibility.
 21   IF(ICOMP.EQ.1) GO TO 23
      WRITE(6,24)
 24   FORMAT(1X,'real Earth compressibility used.')
      GO TO 25
 23   WRITE(6,26)
 26   FORMAT(1X,'incompressible outer core used.')
C Set number of Love numbers to be used to zero.
      INU=0
C Write out parity.
 25   IF(IPAR.EQ.1) GO TO 27
      WRITE(6,28)
 28   FORMAT(1X,'Even across the equator.')
      GO TO 29
 27   WRITE(6,30)
 30   FORMAT(1X,'Odd across the equator.')
 29   IF(ICASE.LT.3)GO TO 31
      WRITE(6,32)
 32   FORMAT(1X,'Enter Vaisala period in hours (negative for',
     1'  unstable stratification)')
      READ(5,*)WVS
C Compute square of dimensionless Vaisala angular frequency.
      WVS=HSSD/WVS
      WVSS=WVS*WVS
      WVS=DSIGN(WVSS,WVS)
      GO TO 33
 31   WVS=0.D0
 33   WRITE(6,34)WVS
 34   FORMAT(1X,'Square of dimensionless Vaisala angular frequency=',
     1 D9.2)
C Read in numerical parameter values.
      READ(1,35) MM1,NM1,INUM
 35   FORMAT(3I4)
C Read in frequency coefficients for degree one inner core
C Love numbers alpha, beta and gamma.  
      READ(1,36)ALPH,BET,GAMM
 36   FORMAT(3D23.15)
C Multiply beta by azimuthal number.
      BET=AZM*BET
C Write out numerical parameters.
      WRITE(6,37)MM1
 37   FORMAT(1X,'Number of elements in radius =',I2)
      WRITE(6,38)NM1
 38   FORMAT(1X,'Number of elements in latitude =',I2)
      IF(ICORE.EQ.0) WRITE(6,39)INUM
 39   FORMAT(1X,'Maximum number of Love numbers that can be used =',I3)
      MM1P1=MM1+1
      NM1P1=NM1+1
      MT4=MM1P1*4
      NT4=NM1P1*4
C Compute order of main matrix BM and lengths of symmetric
C storage vectors, BS.
      INR=MT4*NM1P1
      INRR=INR-2*MM1P1
      IF(ICORE.EQ.0) GO TO 40
      INRR=INRR-NT4+2
      IF(M.EQ.0.AND.IPAR.EQ.0) INRR=INRR+NM1P1
 40   INSV=(INRR*(INRR+1))/2
C Read in eleven matrices to be used in calculation.
      DO 41 J=1,11
        READ(1,42)(BML(J,K),K=1,INSV)
 41   CONTINUE  
 42   FORMAT(D23.15)
C Read in decompression factor f.
      READ(4,10)(ENAME(I),I=1,10)
      READ(4,43) N1
      DO 44 I=1,N1
        J=N1-I+1
        READ(4,17)RAD(J),FDC(J)
 44   CONTINUE
 43   FORMAT(I10)
 17   FORMAT(2D23.15)
C Prepare for interpolation of f.
      M1=100
      M2=198
      M3=98
      N2=2*N1-2
      N3=N1-2
      CALL SPMAT(N1,N2,N3,C,RAD,BB,M1,M2,M3)  
C Enter abscissae for 12-point Gaussian integration.
      XX(1)=-0.981560634246719D0
      XX(2)=-0.904117256370475D0
      XX(3)=-0.769902674194305D0
      XX(4)=-0.587317954286617D0
      XX(5)=-0.367831498998180D0
      XX(6)=-0.125233408511469D0
      XX(7)=-XX(6)
      XX(8)=-XX(5)
      XX(9)=-XX(4)
      XX(10)=-XX(3)
      XX(11)=-XX(2)
      XX(12)=-XX(1)
C Enter weights for 12-point Gaussian integration. 
      W(1)=0.047175336386512D0
      W(2)=0.106939325995318D0
      W(3)=0.160078328543346D0
      W(4)=0.203167426723066D0
      W(5)=0.233492536538355D0
      W(6)=0.249147045813403D0
      W(7)=W(6)
      W(8)=W(5)
      W(9)=W(4)
      W(10)=W(3)
      W(11)=W(2)
      W(12)=W(1)
C Compute number of columns in constraint matrices.
      NC=NT4
      NCR=NC-2
C Read in constraint matrices.
      IF(ICORE.EQ.1) GO TO 45
      READ(1,42)(((CMI(I,J,K),K=1,4),J=1,NCR),I=1,INUM)
 45   READ(1,42)(((CMM(I,J,K),K=1,4),J=1,NCR),I=1,INUM)
C Begin iteration on eigenvalues and eigenfunctions.
C Set row dimension of right-hand side.
      IB=264
C Begin iteration on eigenvalues.
C Set number of Love numbers to be used in applying boundary
C conditions.
      IF(ICORE.EQ.1) GO TO 50
 47   WRITE(6,48)
 48   FORMAT(1X,'Choose INU, the number of Love numbers for applying',
     1 ' boundary conditions.')
 49   READ(5,*)INU
      IF(INU.LE.INUM) GO TO 50
      WRITE(6,51)
 51   FORMAT(1X,'INU unacceptable, choose new value.')
      GO TO 49
 50   INRRC=INRR-INU*(2-ICORE)
      IF(ICORE.EQ.0) GO TO 52
      IF(M.EQ.0.AND.IPAR.EQ.0) INRRC=INRRC-NM1
C Choose search a range or iterate on two trial values.
 52   WRITE(6,53)
 53   FORMAT(1X,'Enter -1 for search a range, 0 for iterate.')
      READ(5,*)ISW
      IF(ISW) 54,55,55
C Enter range and number of trials for search.
 54   WRITE(6,56)
 56   FORMAT(1X,'Enter range and number of trials.')
      READ(5,*)BOT,TOP,NT
      DPER=(TOP-BOT)/DFLOAT(NT)
      NTP1=NT+1
      DO 57 I=1,NTP1
      SIG1=HSSD/(BOT+DFLOAT(I-1)*DPER)
      CALL EIGEN(BML,BM,BMFS,INRR,CMI,CMM,CI,CM,EINVI,EINVM,M,IPAR,      
     1  INU,NCR,ICORE,ICASE,WVS,INSV,B,SIG1,ERR1,IE,ALPH,BET,GAMM)
      TS=12.D0/SIG1
      T=HSSD/SIG1
 57   WRITE(6,61)TS,ERR1,IE,T
C Decide whether to do new search or to iterate.
      GO TO 52
C Enter two trial periods for iteration.
 55   WRITE(6,58)
 58   FORMAT(1X,'Enter two trial periods.')
      READ(5,*)T1,T2
      WRITE(6,59)T1,T2
 59   FORMAT(1X,2D20.8)
      SIG1=HSSD/T1
      SIG2=HSSD/T2
      CALL EIGEN(BML,BM,BMFS,INRR,CMI,CMM,CI,CM,EINVI,EINVM,M,IPAR,
     1 INU,NCR,ICORE,ICASE,WVS,INSV,B,SIG1,ERR1,IE1,ALPH,BET,GAMM)
      CALL EIGEN(BML,BM,BMFS,INRR,CMI,CMM,CI,CM,EINVI,EINVM,M,IPAR,
     1 INU,NCR,ICORE,ICASE,WVS,INSV,B,SIG2,ERR2,IE2,ALPH,BET,GAMM)
 60   AIE=IE2-IE1
      SIG=(SIG1*ERR2*(10.D0**AIE)-SIG2*ERR1)/(ERR2*(10.D0**AIE)-ERR1)
      TS=12.D0/SIG
      T=HSSD/SIG
      CALL EIGEN(BML,BM,BMFS,INRR,CMI,CMM,CI,CM,EINVI,EINVM,M,IPAR,
     1 INU,NCR,ICORE,ICASE,WVS,INSV,B,SIG,ERR,IE,ALPH,BET,GAMM) 
      WRITE(6,61)TS,ERR1,IE,T
 61   FORMAT(1X,'Sidereal period=',F7.4,2X,'Determinant=',F6.3,
     1 2X,'Exp=',I4,1X,'Period=',F8.5,1X,'hours')
C Decide whether to do new search, do another iterate or go to
C eigenfunction calculation.
      WRITE(6,62)
 62   FORMAT(1X,'Enter -1 for search, 0 for another iteration,',
     1 '  1 for  eigenfunction calculation.')
      READ(5,*)IBR
      IF(IBR)54,63,64
 63   SIG1=SIG2
      ERR1=ERR2
      IE1=IE2
      SIG2=SIG
      ERR2=ERR
      IE2=IE
      GO TO 60
C Begin eigenfunction calculation.
C Zero eigenvectors.
 64   DO 46 I=1,IB
        B(I)=0.D0
        BI(I)=0.D0
 46   CONTINUE
C Set all components of eigenvector to unity.
      DO 65 I=1,INRRC
        B(I)=1.D0
 65   CONTINUE
C Construct derivative of of constrained matrix at
C frequency SIG.
      CALL CMATP(BML,BMP,BMFS,INRR,CMI,CMM,CI,CM,EINVI,EINVM,M,IPAR,
     1 INU,NCR,ICORE,INSV,SIG,ICASE,WVS,ALPH,BET,GAMM)
C Save current eigenvector.
 66   DO 67 I=1,INRRC
        BI(I)=B(I)
 67   CONTINUE
C Multiply derivative of main matrix into eigenvctor.
      CALL MULT(BMP,BI,B,INRRC)
C Restore full matrix.
      CALL MATF(BMF,BM,INRRC)
C Solve for new eigenvector.
      CALL LINSOL(BMF,B,INRRC,C,DET,264,529)
C Calculate eigenvector length and its scalar product.
C with previous eigenvector.
      SUMS=0.D0
      SP=0.D0
      DO 68 I=1,INRRC
        SP=SP+B(I)*BI(I)
        SUMS=SUMS+B(I)*B(I)
 68   CONTINUE
      AINRRC=INRRC
      SUM=DSQRT(SUMS/AINRRC)
C Normalize eigenvector magnitude and direction and
C find relative departure from previous eigenvector.
      SQDIFF=0.D0
      DO 69 I=1,INRRC
        B(I)=B(I)/DSIGN(SUM,SP)
        DIFF=B(I)-BI(I)
        SQDIFF=SQDIFF+DIFF*DIFF
 69   CONTINUE
      DIFF=DSQRT(SQDIFF/AINRRC)
      WRITE(6,70)DIFF
 70   FORMAT(1X,'Relative difference from previous eigenvector =',
     1 D22.15)
C Test if eigenvector acceptable.
      IF(DIFF.GT.RERRM) GO TO 66 
C Improve eigenvalue and redo decomposition.
      SIG=SIG-AINRRC/SP
      CALL EIGEN(BML,BM,BMFS,INRR,CMI,CMM,CI,CM,EINVI,EINVM,M,IPAR,
     1  INU,NCR,ICORE,ICASE,WVS,INSV,BI,SIG,ERR,IE,ALPH,BET,GAMM)
      TS=12.D0/SIG
      T=HSSD/SIG
      WRITE(6,61)TS,ERR,IE,T
C Store and re-inflate eigenvector.
C Restore elements determined by constraint equations.
      NCC=NCR-INU
C Set initial read point on deflated eigenvector.
      JL=1
C Set amount by which index of re-inflated vector exceeds
C index of deflated vector.
      IC=0
C Test for no inner core.
      IF(ICORE.EQ.1) GO TO 71
C Restore elements at the inner core boundary.
      IF(INU.EQ.0) GO TO 72
      DO 73 I=1,INU
        BI(I)=0.D0
        DO 73 J=1,NCC
          BI(I)=BI(I)+CI(I,J)*B(J)
 73   CONTINUE
 72      IC=INU
      GO TO 74
C Test for even, axisymmetric case.
 71   IF(M.EQ.0.AND.IPAR.EQ.0) GO TO 75
      GO TO 74
C Replace function values at geocentre for even,
C axisymmetric case.
 75   BI(1)=B(1)
      DO 76 I=1,NM1
        IP1=I+1
        BI(IP1)=B(1)
 76   CONTINUE
      JL=2
      IC=NM1
C Copy unconstrained variables up to beginning of
C core-mantle boundary constrained variables.
 74   JU=INRRC-NCC
      DO 77 II=JL,JU
        I=II+IC
        BI(I)=B(II)
 77   CONTINUE
C Restore elements at the core-mantle boundary.
      IF(INU.EQ.0) GO TO 78
      DO 79 II=1,INU
        I=II+JU+IC
        BI(I)=0.D0
        DO 79 J=1,NCC
          JJ=JU+J
          BI(I)=BI(I)+CM(II,J)*B(JJ)
 79   CONTINUE
 78   IC=IC+INU
      JUP1=JU+1
      DO 80 II=JUP1,INRRC
        I=II+IC
        BI(I)=B(II)
 80   CONTINUE
C Store re-inflated eigenvector.
      DO 81 I=1,INRR
        B(I)=BI(I)
 81   CONTINUE
C Restore struck out zero elements.
C Test for inner core.
      IF(ICORE.EQ.0) GO TO 82
C Test for even, axisymmetric case.
      IF(M.EQ.0.AND.IPAR.EQ.0) GO TO 83
C Restore function and derivatives at geocentre.
      BI(1)=0.D0
      BI(2)=0.D0
      DO 84 I=1,NM1
        II=(I-1)*4+3
        IIP1=II+1
        IIP2=II+2
        IIP3=II+3
        BI(II)=0.D0
        BI(IIP1)=0.D0
        BI(IIP2)=0.D0
        BI(IIP3)=0.D0
 84   CONTINUE
      JL=1
      IC=NT4-2
      GO TO 85
C Restore derivatives.
 83   BI(1)=B(1)
      BI(2)=0.D0 
      DO 86 I=1,NM1
        IP1=I+1
        IT4=4*I
        IT4M1=IT4-1
        IT4P1=IT4+1
        IT4P2=IT4+2
        BI(IT4M1)=B(IP1)
        BI(IT4)=0.D0
        BI(IT4P1)=0.D0
        BI(IT4P2)=0.D0
 86   CONTINUE
      JL=NM1P1+1
      IC=3*NM1+1
C Copy remaining elements.
 85   DO 87 II=JL,INRR
        I=II+IC
        BI(I)=B(II)
 87   CONTINUE
      IU=INRR+IC
C Store re-inflated eigenvector.
      DO 88 I=1,IU
        B(I)=BI(I)
 88   CONTINUE
C Restore elements eliminated in applying condition
C at the equator.
C Set counter for elements of uninflated vector.
 82      LL=1
C Calculate indices of struck out elements of four vectors.
      KT=4-2*IPAR-1
      KTP1=KT+1
C Do loop over radius.
      DO 89 I=1,MM1P1
        II=(I-1)*NT4
C Do loop over z,the cosine of co-latitude.
        DO 89 J=1,NM1P1
          JJ=4*(J-1)+II
C Do loop over four vector indices.
          DO 89 K=1,4
            KK=K+JJ
C Test if not on equator.
            IF(J.NE.1) GO TO 90
            GO TO 91
C Copy element directly if off the equator.
 90         BI(KK)=B(LL)
            LL=LL+1
            GO TO 89
C If on the equator, test to see if element has been
C struck out.
 91         IF(K.EQ.KT.OR.K.EQ.KTP1) GO TO 92
C Copy element on equator but not struck out.
            BI(KK)=B(LL)
            LL=LL+1
            GO TO 89
C Insert zero for struck out element.
 92         BI(KK)=0.D0
 89   CONTINUE
C Find equivalent wavenumber of eigenvector.
      SUMFS=0.E0
      SUMDS=0.E0
C Set node points in z, the cosine of co-latitude.
      ZJ(1)=0.D0
      ANM1=NM1
      DTHETA=PI/(2.D0*ANM1)
      DO 93 I=1,NM1
        CSI=NM1-I
        CSI=CSI*DTHETA
        IP1=I+1
        ZJ(IP1)=DCOS(CSI)
 93   CONTINUE
C Set node points in radius.
      RI(1)=RIOB/RCMB
      IF(ICORE.EQ.1) RI(1)=0.D0
      AMM1=MM1
      DRI=(1.D0-RI(1))/AMM1
      DO 94 I=1,MM1
        IP1=I+1
        AI=I
        RI(IP1)=RI(1)+AI*DRI
 94   CONTINUE
C Put phi and derivatives in 2-D arrays U(I,J),PP(I,J),
C Q(I,J),RR(I,J).
      DO 95 I=1,MM1P1
        Y(I)=RI(I)
        YD(I)=RI(I)
        II=(I-1)*NT4
        DO 95 J=1,NM1P1
          FACT=DACOS(ZJ(J))
          JC=NM1P1-J+1
          X(JC)=FACT
          XD(JC)=FACT
          IJ=4*(J-1)+II
          UD(JC,I)=BI(IJ+1)
          U(JC,I)=BI(IJ+1)
          PPD(JC,I)=BI(IJ+3)
          PP(JC,I)=BI(IJ+3)
          QD(JC,I)=BI(IJ+2)
          Q(JC,I)=BI(IJ+2)
          RRD(JC,I)=BI(IJ+4)
          RR(JC,I)=BI(IJ+4)
 95   CONTINUE    
C Calculate Z2S and SIGS.
      SIGS=SIG*SIG
      Z2S=1.D0+WVS-SIGS
C Decide whether to do new search, proceed to plot
C or end.
      WRITE(6,96)
 96   FORMAT(1X,'Enter -1 for search, 0 for do plots, 1 for end.')
      READ(5,*)IBR
      IF(IBR)47,97,98
C Set up plot.
C Begin perspective plot.
C Specify projection parameters.
 97   ALPHA=ASIN(0.5)
      BETA=0.5
C Define single precision variables.
      ZETA2S=Z2S
      SIG4=SIG
C Specify number of wires in x and y directions.
      NXW=20
      NYW=20
C Specify number of pen increments along projector.
      VINC=5.
C Specify relative vertical scale height.
      SCHT=0.9
      WRITE(6,99)
 99   FORMAT(1X,'Enter -1 for both plots, 0 for perspective, ',
     1 ' 1 for flow vectors.')  
      READ(5,*)IBR
      IF(IBR)100,100,101  
 100  CALL THREED(NM1P1,MM1P1,X,Y,IPAR,ICORE,ICOMP,INU,SIG4,ZETA2S,
     1 ALPHA,BETA,VINC,SCHT,NXW,NYW,M,U,PP,Q,RR)
C Decide whether to invert direction of eigenvector and
C replot perspective.
      WRITE(6,102)
 102  FORMAT(1X,'Enter -1 to reverse direction of eigenvector, 1 for',
     1 ' continue.')
      READ(5,*)IDIR
      IF(IDIR)103,103,104
 103  DO 105 I=1,MM1P1
      DO 105 J=1,NM1P1
      JC=NM1P1-J+1
      U(JC,I)=-U(JC,I)
      PP(JC,I)=-PP(JC,I)
      Q(JC,I)=-Q(JC,I)
 105  RR(JC,I)=-RR(JC,I)      
      GO TO 100
 104  IF(IBR.EQ.0) GO TO 112
C Begin flow vector plot.
C Choose scaled or unscaled plot.
 101  SF=1.E0 
 106  WRITE(6,107)
 107  FORMAT(1X,'Enter 0 for scaled plot, 1 for unscaled plot.')
      READ(5,*)IS
      IF(IS)108,108,109 
C Enter scale factor.
 108  WRITE(6,110)
 110  FORMAT(1X,'Enter scale factor for flow vectors.')
      READ(5,*)SF
 109  CALL FLOWV(NM1P1,MM1P1,X,Y,IPAR,ICOMP,INU,
     1NXW,NYW,ZETA2S,SIG4,M,U,PP,Q,RR,SF,IS,ALPHA)
C Decide whether to choose new scale factor and replot flow field.
      WRITE(6,111)
 111  FORMAT(1X,'Enter -1 for new scale factor, 1 for continue.')
      READ(5,*)ISF
      IF(ISF)106,112,112
C Decide whether to do new search or to end.
 112  WRITE(6,113)
 113  FORMAT(1X,'Enter -1 for new search, 1 for end.')
      READ(5,*)IBR
      IF(IBR)47,98,98
 98   CONTINUE
      END

      SUBROUTINE FLOWV(N,M,X,Y,IPAR,ICOMP,INU,
     1NXW,NYW,ZETA2S,SIG,IAZ,U,P,Q,RR,SF,IS,ALPHA)
C Subroutine to plot displacement vectors for flow field.
      CHARACTER NUMB*10
      DIMENSION X(11),Y(6),DISPV(3,40,40),
     1 U(11,6),P(11,6),Q(11,6),RR(11,6)
      PI=3.1415927
C Calculate parameters.
      WE=7.292115E-5
      HSSD=PI/(WE*3600.0)
      MM1=M-1
      NM1=N-1
      SIGS=SIG*SIG
      WVS=ZETA2S-1.0+SIGS
      T=HSSD/SIG
C Set up plot.
      CALL HARDCOPY_RANGE(0.0,1023.0,0.0,779.0,-249.0,1249.0,-299.0,
     1 841.0,1)
      CALL MONITOR_RANGE(18,6,0.0,1023.0,0.0,779.0,0.0,1.0,0.0,0.75,1)
      CALL DWG_BATCH('flowv',IERR)
      IF(IERR.EQ.1)WRITE(6,10)
 10   FORMAT(1X,'Unable to open drawing file for flowv.')
C Kill time while waiting for plot to be set up.
      CALL PLOT_R(0.0,0.0,3)
      DO 11 I=1,8000
        AI=FLOAT(I)
        AI=AI/10.0
        CALL PLOT_R(AI,0.0,3)
 11    CONTINUE
C Begin plotting.
C Select font.
      CALL PALPHA('ROMAN.3 ',0)
C Select colour.
      CALL PLOT_COLOR(6,1)
C Draw title.
      XTICK=500.0-0.5*PSMLEN('Displacement Vector Field',25,
     1 40.0)
      CALL PSYM(XTICK,775.0,40.0,'Displacement Vector Field',
     1 0.0,25)
C  Draw data box.
      CALL PLOT_R(1000.0,-250.0,3)
      CALL PLOT_R(1200.0,-250.0,2)
      CALL PLOT_R(1200.0,-50.0,2)
      CALL PLOT_R(1000.0,-50.0,2)
      CALL PLOT_R(1000.0,-250.0,2)
C Put data in the box.
      CALL PSYM(1015.0,-75.0,15.0,'m=',0.0,2)
      WRITE(NUMB,12)IAZ
 12   FORMAT(I1)
      XTICK=1015.0+PSMLEN('m=',2,15.0)
      CALL PSYM(XTICK,-75.0,15.0,NUMB,0.0,1)
      XTICK=XTICK+PSMLEN(NUMB,1,15.0)
      CALL PSYM(XTICK,-75.0,15.0,', ',0.0,2)
      XTICK=XTICK+PSMLEN(', ',2,15.0)
      IF(IPAR.EQ.0)GO TO 13
      CALL PSYM(XTICK,-75.0,15.0,'odd',0.0,3)
      GO TO 14
 13   CALL PSYM(1080.0,-75.0,15.0,'even',0.0,4)
 14   WRITE(NUMB,12)MM1
      CALL PSYM(1015.0,-125.0,15.0,NUMB,0.0,1)
      XTICK=1015.0+PSMLEN(NUMB,1,15.0)
      CALL PSYM(XTICK,-125.0,15.0,' X',0.0,2)
      XTICK=XTICK+PSMLEN(' X',2,15.0)
      WRITE(NUMB,15)NM1
      CALL PSYM(XTICK,-125.0,15.0,NUMB,0.0,2)
      XTICK=XTICK+PSMLEN(NUMB,2,15.0)
      CALL PSYM(XTICK,-125.0,15.0,', ',0.0,2)
      XTICK=XTICK+PSMLEN(', ',2,15.0)
      CALL PSYM(XTICK,-125.0,15.0,'INU=',0.0,4)
      XTICK=XTICK+PSMLEN('INU=',4,15.0)
      WRITE(NUMB,15)INU
 15   FORMAT(I2)
      CALL PSYM(XTICK,-125.0,15.0,NUMB,0.0,2)
      CALL PSYM(1015.0,-175.0,15.0,'N',0.0,1)
      XTICK=1015.0+PSMLEN('N',1,15.0)
      CALL PSYM(XTICK,-162.0,10.0,'2',0.0,1)
      XTICK=XTICK+PSMLEN('2',1,10.0)
      CALL PSYM(XTICK,-175.0,15.0,'=',0.0,1)
      XTICK=XTICK+PSMLEN('=',1,15.0)
      WRITE(NUMB,16)WVS
 16   FORMAT(F6.3)
      CALL PSYM(XTICK,-175.0,15.0,NUMB,0.0,6)
      XTICK=XTICK+PSMLEN(NUMB,6,15.0)
      CALL PSYM(XTICK,-175.0,15.0,', ',0.0,2)
      XTICK=XTICK+PSMLEN(', ',2,15.0)
      CALL PSYM(XTICK,-175.0,15.0,'f=',0.0,2)
      XTICK=XTICK+PSMLEN('F=',2,15.0)
      IF(ICOMP.EQ.0) GO TO 17
      CALL PSYM(XTICK,-175.0,15.0,'1',0.0,1)
      GO TO 18
 17   CALL PSYM(XTICK,-175.0,15.0,'R',0.0,1)
 18   CALL PSYM(1015.0,-225.0,15.0,'Period=',0.0,7)
      XTICK=1015.0+PSMLEN('Period',7,15.0)
      WRITE(NUMB,19)T
 19   FORMAT(F7.3)
      CALL PSYM(XTICK,-225.0,15.0,NUMB,0.0,7) 
C Draw domain boundaries.
C Draw boundary for meridional projection.
      CALL PLOT_R(1200.0,0.0,3)
      CALL PLOT_COLOR(6,1)
      DTHETA=PI/2000.0
      DO 20 I=1,1000
        AI=I
        THETA=AI*DTHETA
        XI=600.0*COS(THETA)+600.0
        YI=600.0*SIN(THETA)
        CALL PLOT_R(XI,YI,2)
 20   CONTINUE
      CALL PLOT_R(XI,YI,3)
      ADB=Y(1)*600.0
      YI=ADB
      CALL PLOT_R(XI,YI,2)
      DO 21 I=1,1000
        AI=I
        THETA=PI/2.0-AI*DTHETA
        XI=ADB*COS(THETA)+600.0
        YI=ADB*SIN(THETA)
        CALL PLOT_R(XI,YI,2)
 21   CONTINUE
      CALL PLOT_R(XI,YI,3)
      CALL PLOT_R(1200.0,YI,2)
C Draw boundary for azimuthal component.
      CALL PLOT_R(-200.0,0.0,3)
      DO 22 I=1,1000
        AI=I
        THETA=AI*DTHETA
        XI=-600.0*COS(THETA)+400.0
        YI=600.0*SIN(THETA)
        CALL PLOT_R(XI,YI,2)
 22   CONTINUE
      CALL PLOT_R(XI,YI,3)
      YI=ADB
      CALL PLOT_R(XI,YI,2)
      DO 23 I=1,1000
        AI=I
        THETA=PI/2.0-AI*DTHETA
        XI=-ADB*COS(THETA)+400.0
        YI=ADB*SIN(THETA)
        CALL PLOT_R(XI,YI,2)
 23   CONTINUE
      CALL PLOT_R(XI,YI,3)
      CALL PLOT_R(-200.0,0.0,2)
C Label axes.
      CALL PLOT_COLOR(3,1)
      XTICK=100.0-0.5*PSMLEN('Azimuthal Component',19,25.0)
      CALL PSYM(XTICK,675.0,25.0,'Azimuthal Component',0.0,19)
      XTICK=500.0-0.5*PSMLEN('Equator',7,25.0)
      CALL PSYM(XTICK,-75.0,25.0,'Equator',0.0,7)
      XTICK=900.0-0.5*PSMLEN('Meridional Projection',22,25.0)
      CALL PSYM(XTICK,675.0,25.0,'Meridional Projection',0.0,21)
      XTICK=500.0-0.5*PSMLEN('R',1,25.0)
      CALL PSYM(XTICK,565.0,25.0,'R',0.0,1)
      XTICK=500.0-0.5*PSMLEN('o',1,25.0)
      CALL PSYM(XTICK,535.0,25.0,'o',0.0,1)
      XTICK=500.0-0.5*PSMLEN('t',1,25.0)
      CALL PSYM(XTICK,505.0,25.0,'t',0.0,1)
      XTICK=500.0-0.5*PSMLEN('a',1,25.0)
      CALL PSYM(XTICK,475.0,25.0,'a',0.0,1)
      XTICK=500.0-0.5*PSMLEN('t',1,25.0)
      CALL PSYM(XTICK,445.0,25.0,'t',0.0,1)
      XTICK=500.0-0.5*PSMLEN('i',1,25.0)
      CALL PSYM(XTICK,415.0,25.0,'i',0.0,1)
      XTICK=500.0-0.5*PSMLEN('o',1,25.0)
      CALL PSYM(XTICK,385.0,25.0,'o',0.0,1)
      XTICK=500.0-0.5*PSMLEN('n',1,25.0)
      CALL PSYM(XTICK,355.0,25.0,'n',0.0,1)
      XTICK=500.0-0.5*PSMLEN('A',1,25.0)
      CALL PSYM(XTICK,295.0,25.0,'A',0.0,1)
      XTICK=500.0-0.5*PSMLEN('x',1,25.0)
      CALL PSYM(XTICK,265.0,25.0,'x',0.0,1)
      XTICK=500.0-0.5*PSMLEN('i',1,25.0)
      CALL PSYM(XTICK,235.0,25.0,'i',0.0,1)
      XTICK=500.0-0.5*PSMLEN('s',1,25.0)
      CALL PSYM(XTICK,205.0,25.0,'s',0.0,1)
C Calculate array of displacement vectors.
      XMIN=X(1)
      XMAX=X(N)
      YMIN=Y(1)
      YMAX=Y(M)
      DELTAX=XMAX-XMIN
      DELTAY=YMAX-YMIN
      DR=DELTAY/FLOAT(NYW-1)
      DTHETA=DELTAX/FLOAT(NXW-1)
C Find maximum displacement magnitude.
      UMAX=0.0
      DO 24 I=1,NXW
        AIM1=I-1
        THETA=XMIN+AIM1*DTHETA
        CT=COS(THETA)
        ST=SIN(THETA)
        DO 24 J=1,NYW
          AJM1=J-1
          R=YMIN+AJM1*DR
          CALL UVECT(UR,UT,UP,R,THETA,X,Y,IPAR,N,M,ZETA2S,SIG,
     1    IAZ,U,P,Q,RR)
          UMAG=SQRT(UR*UR+UT*UT+UP*UP)           
          DISPV(1,I,J)=UR*ST+UT*CT
          DISPV(2,I,J)=UR*CT-UT*ST
          DISPV(3,I,J)=UP
          IF(UMAG.GT.UMAX) UMAX=UMAG
 24    CONTINUE
C Scale displacements by largest and plot vectors.
      CALL PLOT_COLOR(4,1)
      CS=COS(PI/12.0)
      SN=SIN(PI/12.0)
      DO 25 I=1,NXW
        AIM1=I-1
        THETA=XMIN+AIM1*DTHETA
        CT=COS(THETA)
        ST=SIN(THETA)
        DO 25 J=1,NYW
          AJM1=J-1
          R=YMIN+AJM1*DR
          DISPV(1,I,J)=DISPV(1,I,J)/UMAX
          DISPV(2,I,J)=DISPV(2,I,J)/UMAX
          DISPV(3,I,J)=DISPV(3,I,J)/UMAX
          XI=600.0*R*ST+600.0
          YI=600.0*R*CT
          XIZ=-600.0*R*ST+400.0
          YIZ=YI
C Plot arrow on meridional projection.
          CALL PLOT_R(XI,YI,3)
          DXI=DISPV(1,I,J)
          DYI=DISPV(2,I,J)
          UMAG=SQRT(DXI*DXI+DYI*DYI)
          IF(UMAG.EQ.0.0) GO TO 26
          DXI=DXI/UMAG
          DYI=DYI/UMAG
C Draw arrow stem.
C Branch for unscaled flow vectors.
          IF(IS)27,27,28
 27       DXI=DXI*UMAG*SF
          DYI=DYI*UMAG*SF
 28       XI=XI+30.0*DXI
          YI=YI+30.0*DYI
          CALL PLOT_R(XI,YI,2)
C Draw arrowhead.
          DXICS=10.0*DXI*CS
          DYISN=10.0*DYI*SN
          DXISN=10.0*DXI*SN
          DYICS=10.0*DYI*CS
          DDXI=-DXICS+DYISN
          DDYI=-DYICS-DXISN
          XI=XI+DDXI
          YI=YI+DDYI
          CALL PLOT_R(XI,YI,2)
          XI=XI-DDXI
          YI=YI-DDYI
          CALL PLOT_R(XI,YI,3)
          DDXI=-DXICS-DYISN
          DDYI=-DYICS+DXISN
          XI=XI+DDXI
          YI=YI+DDYI
          CALL PLOT_R(XI,YI,2)
C Plot arrow for azimuthal component.
 26       CALL PLOT_R(XIZ,YIZ,3)
          DXI=-DISPV(3,I,J)*SIN(ALPHA)
          DYI=-DISPV(3,I,J)*COS(ALPHA)
          UMAG=ABS(DISPV(3,I,J))
          IF(UMAG.EQ.0.0) GO TO 25
          DXI=DXI/UMAG
          DYI=DYI/UMAG
C Draw arrow stem.
C Branch for unscaled flow vectors.
          IF(IS)29,29,30
 29       DXI=DXI*UMAG*SF
          DYI=DYI*UMAG*SF
 30       XIZ=XIZ+30.0*DXI
          YIZ=YIZ+30.0*DYI
          CALL PLOT_R(XIZ,YIZ,2)
C Draw arrowhead.
          DXICS=10.0*DXI*CS
          DYISN=10.0*DYI*SN
          DXISN=10.0*DXI*SN
          DYICS=10.0*DYI*CS
          DDXI=-DXICS+DYISN
          DDYI=-DYICS-DXISN
          XIZ=XIZ+DDXI
          YIZ=YIZ+DDYI
          CALL PLOT_R(XIZ,YIZ,2)
          XIZ=XIZ-DDXI
          YIZ=YIZ-DDYI
          CALL PLOT_R(XIZ,YIZ,3)
          DDXI=-DXICS-DYISN
          DDYI=-DYICS+DXISN
          XIZ=XIZ+DDXI
          YIZ=YIZ+DDYI
          CALL PLOT_R(XIZ,YIZ,2)
 25    CONTINUE
      CALL FLUSH_PLOT
      WRITE(6,31)
 31   FORMAT(1X,'Pause to look at flow vectors, type 0 to go on.')
      READ(5,*)ITRASH
      CALL CLEAR_PLOT
      RETURN
      END
      
      SUBROUTINE THREED(N,M,X,Y,IPAR,ICORE,ICOMP,INU,SIG4,ZETA2S,
     1 ALPHA,BETA,VINC,SCHT,NXW,NYW,IAZ,U,P,Q,R)
C   This is a two-dimensional bicubic spline interpolation and
C   perspective plot programme. A function U is defined on a
C   rectangular grid of N values of X, M values of Y. P, Q, and R 
C   are its first partial and cross derivatives at the grid points.
C   The main calling programme must contain a dimension statement
C   with X(N),Y(M),U(N,M),P(N,M),Q(N,M),R(N,M),A(N,M,4,4), DX(NM1),
C   DY(MM1),ZX(NM1),ZY(MM1),ORD(MNM),UNB(MNM) AND F(MNM),
C   where NM1=N-1, MM1=M-1, MNM=MAX(N,M). As well as the grid points
C   and function values it must also define the angle ALPHA which
C   the projected X-axis makes with the negative Z-axis (the vertical
C   screen co-ordinate), the X-axis foreshortening factor BETA,
C   the number of wires in the X and Y directions NXW and NYW,
C   the number of pen increments in each increment along the
C   projector taken in checking visibility VINC, and fraction of
C   full function height at the back of the perspective to be plotted.
C   The latter parameter is called SCHT.
      LOGICAL*4 ASIS
      CHARACTER NUMB*10
      DIMENSION X(11),Y(6),U(11,6),P(11,6),Q(11,6),R(11,6)
      COMMON XMIN,YMIN,GMIN,SCXY,SCX,SCY,SCFN,DDX,XMAX,YMAX
      COMMON/MODE/ASIS
      PI=3.1415927
      WE=7.292115E-5
      HSSD=PI/(WE*3600.0)
      MM1=M-1
      NM1=N-1
C  Calculate physical parameters.
      SIGS=SIG4*SIG4
      WVS=ZETA2S-1.0+SIGS
      T=HSSD/SIG4
C  Begin layout.
      XMIN=X(1)
      XMAX=X(N)
      YMIN=Y(1)
      YMAX=Y(M)
C   Calculate increments in X and Y between wires.
      DELTAX=XMAX-XMIN
      DELX=DELTAX/FLOAT(NXW-1)
      DELTAY=YMAX-YMIN
      DELY=DELTAY/FLOAT(NYW-1)
C   Find scale factor for X and Y.
      SCXY=1000./(1.+BETA*SIN(ALPHA))
      SCX=SCXY/DELTAX
      SCY=SCXY/DELTAY
C   Find increments in X and Y along wires.
      RINX=SCXY*BETA
      RINY=SCXY
      DDX=DELTAX/RINX
      DDY=DELTAY/RINY
C   Find numbers of pen increments in X and Y directions.
      INX=INT(RINX)
      INY=INT(RINY)
C   Compute function values at grid intersections and find 
C   least and greatest.
      RI=XMIN
      RJ=YMIN
      CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
      GMIN=G
      GMAX=G
      DO 10 I=1,NXW
        RIM1=I-1
        RI=RIM1*DELX+XMIN
        DO 10 J=1,NYW
          RJM1=J-1
          RJ=RJM1*DELY+YMIN
          CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
          IF(G.LT.GMIN) GMIN=G
          IF(G.GT.GMAX) GMAX=G
 10   CONTINUE
C   Find scale factor for function.
      XY=RINX*COS(ALPHA)
      IXY=INT(XY) 
      FNMAX=700.-XY
      SCFN=SCHT*FNMAX/(GMAX-GMIN)
C Set up perspective plot.
      CALL HARDCOPY_RANGE(0.0,1023.0,0.0,779.0,-199.0,1299.0,-249.0,
     1 891.0,1)     
      CALL MONITOR_RANGE(18,6,0.0,1023.0,0.0,779.0,0.0,1.0,0.0,0.75,1)
      CALL DWG_BATCH('threed',IERR)
      IF(IERR.EQ.1)WRITE(5,11)
 11   FORMAT(1X,'Unable open drawing file for threed.')
      RINYDT=RINY/1000.0
      CALL PLOT_R(RINY,0.0,3)
C  Kill time while waiting for plot to be set up.
      DO 12 I=1,10000
        AI=I
        AI=AI/10.0
        AITRI=RINY-AI*RINYDT
        CALL PLOT_R(AITRI,0.0,3)
 12   CONTINUE
C  Begin plotting.
C  Select font.
      CALL PALPHA('ROMAN.3 ',0)
C  Select colour.
      CALL PLOT_COLOR(1,1)
C  Draw title.
      CALL PSYM(-15.0,775.0,40.0,
     1 'Generalized Displacement Potential',0.0,34)
C  Draw axes.
      CALL PLOT_R(0.0,0.0,3)
      CALL PLOT_R(RINY,0.0,2)
      CALL PLOT_R(1000.0,XY,2)
C  Draw tick marks.
      CALL PLOT_R(1015.0,XY,3)
      CALL PLOT_R(1040.0,XY,2)
      RINYT=RINY+15.0 
      CALL PLOT_R(RINYT,0.0,3)
      RINYT=RINY+40.0
      CALL PLOT_R(RINYT,0.0,2)
      TICKX=0.9*SIN(ALPHA)
      TICKY=0.9*COS(ALPHA)
      XTICK=-15.0*TICKX
      YTICK=-15.0*TICKY
      CALL PLOT_R(XTICK,YTICK,3)
      XTICK=-40.0*TICKX
      YTICK=-40.0*TICKY
      CALL PLOT_R(XTICK,YTICK,2)
      XTICK=XTICK+RINY
      CALL PLOT_R(XTICK,YTICK,3)
      XTICK=-15.0*TICKX+RINY
      YTICK=-15.0*TICKY
      CALL PLOT_R(XTICK,YTICK,2)
C  Label axes.
      CALL PSYM(300.0,-100.0,25.0,'Radius',0.0,6)
      ALPHAD=180.0*ALPHA/PI
      ANGLE=90.0-ALPHAD
      XLX=RINY+170.0
      XLY=120.0*TICKY
      CALL PSYM(XLX,XLY,25.0,'Latitude',ANGLE,8)
      RINYT=RINY+60.0
      CALL PSYM(RINYT,-10.0,20.0,'0',0.0,1)
      ASIS=.TRUE.
      RINYT=RINY+90.0
      CALL PSYM(RINYT,-10.0,20.0,'A1'X,0.0,1)
      ASIS=.FALSE.
      XYT=XY-10.0
      CALL PSYM(1060.0,XYT,20.0,'90',0.0,2)
      ASIS=.TRUE.
      CALL PSYM(1110.0,XYT,20.0,'A1'X,0.0,1)
      ASIS=.FALSE.
      XTICK=-100.0*TICKX
      YTICK=-120.0*TICKY
      IF(ICORE.EQ.0) GO TO 13
      CALL PSYM(XTICK,YTICK,20.0,'CoE',ANGLE,3)
      GO TO 14
 13   CALL PSYM(XTICK,YTICK,20.0,'ICB',ANGLE,3)
 14   XTICK=XTICK+RINY
      CALL PSYM(XTICK,YTICK,20.0,'CMB',ANGLE,3)
C  Draw data box.
      CALL PLOT_R(1050.0,-200.0,3)
      CALL PLOT_R(1250.0,-200.0,2)
      CALL PLOT_R(1250.0,0.0,2)
      CALL PLOT_R(1050.0,0.0,2)
      CALL PLOT_R(1050.0,-200.0,2)
C Put data in the box.
      CALL PSYM(1065.0,-25.0,15.0,'m=',0.0,2)
      WRITE(NUMB,15)IAZ
 15   FORMAT(I1)
      XTICK=1065.0+PSMLEN('m=',2,15.0)
      CALL PSYM(XTICK,-25.0,15.0,NUMB,0.0,1)
      XTICK=XTICK+PSMLEN(NUMB,1,15.0)
      CALL PSYM(XTICK,-25.0,15.0,', ',0.0,2)
      XTICK=XTICK+PSMLEN(', ',2,15.0)
      IF(IPAR.EQ.0)GO TO 16
      CALL PSYM(XTICK,-25.0,15.0,'odd',0.0,3)
      GO TO 17
 16   CALL PSYM(1130.0,-25.0,15.0,'even',0.0,4)
 17   WRITE(NUMB,15)MM1
      CALL PSYM(1065.0,-75.0,15.0,NUMB,0.0,1)
      XTICK=1065.0+PSMLEN(NUMB,1,15.0)
      CALL PSYM(XTICK,-75.0,15.0,' X',0.0,2)
      XTICK=XTICK+PSMLEN(' X',2,15.0)
      WRITE(NUMB,18)NM1
      CALL PSYM(XTICK,-75.0,15.0,NUMB,0.0,2)
      XTICK=XTICK+PSMLEN(NUMB,2,15.0)
      CALL PSYM(XTICK,-75.0,15.0,', ',0.0,2)
      XTICK=XTICK+PSMLEN(', ',2,15.0)
      CALL PSYM(XTICK,-75.0,15.0,'INU=',0.0,4)
      XTICK=XTICK+PSMLEN('INU=',4,15.0)
      WRITE(NUMB,18)INU
 18   FORMAT(I2)
      CALL PSYM(XTICK,-75.0,15.0,NUMB,0.0,2)
      CALL PSYM(1065.0,-125.0,15.0,'N',0.0,1)
      XTICK=1065.0+PSMLEN('N',1,15.0)
      CALL PSYM(XTICK,-112.0,10.0,'2',0.0,1)
      XTICK=XTICK+PSMLEN('2',1,10.0)
      CALL PSYM(XTICK,-125.0,15.0,'=',0.0,1)
      XTICK=XTICK+PSMLEN('=',1,15.0)
      WRITE(NUMB,19)WVS
 19   FORMAT(F6.3)
      CALL PSYM(XTICK,-125.0,15.0,NUMB,0.0,6)
      XTICK=XTICK+PSMLEN(NUMB,6,15.0)
      CALL PSYM(XTICK,-125.0,15.0,', ',0.0,2)
      XTICK=XTICK+PSMLEN(', ',2,15.0)
      CALL PSYM(XTICK,-125.0,15.0,'f=',0.0,2)
      XTICK=XTICK+PSMLEN('F=',2,15.0)
      IF(ICOMP.EQ.0) GO TO 20
      CALL PSYM(XTICK,-125.0,15.0,'1',0.0,1)
      GO TO 21
 20   CALL PSYM(XTICK,-125.0,15.0,'R',0.0,1)
 21   CALL PSYM(1065.0,-175.0,15.0,'Period=',0.0,7)
      XTICK=1065.0+PSMLEN('Period',7,15.0)
      WRITE(NUMB,22)T
 22   FORMAT(F7.3)
      CALL PSYM(XTICK,-175.0,15.0,NUMB,0.0,7) 
      CALL PLOT_R(1000.0,XY,3)
C  Draw corner at minimum X, maximum Y.
      RI=XMIN
      RJ=YMAX
      CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
      CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,0,
     1 U,P,Q,R)
      CALL PLOT_R(PX,PY,2)
      CALL PLOT_COLOR(3,1)
C  Draw function along maximum value of Y.
      DO 23 I=1,INX
        PN=I
        RI=XMIN+PN*DDX
        CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
        CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,0,
     1  U,P,Q,R)
        CALL PLOT_R(PX,PY,2)
 23   CONTINUE
      RI=XMAX
      CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
      CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,0,
     1 U,P,Q,R)
      CALL PLOT_R(PX,PY,2)
C  Draw corner at maximum values of X and Y.
      CALL PLOT_COLOR(1,1)
      CALL PLOT_R(RINY,0.0,2)
      CALL PLOT_R(PX,PY,3)
C  Draw function along maximum value of X.
      CALL PLOT_COLOR(3,1)
      DO 24 I=1,INY
        PN=I
        RJ=YMAX-PN*DDY
        CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
        CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,0,
     1  U,P,Q,R)
        CALL PLOT_R(PX,PY,2)
 24   CONTINUE
      RJ=YMIN
      CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
      CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,0,
     1 U,P,Q,R)
      CALL PLOT_R(PX,PY,2)
C  Draw corner at maximum X, minimum Y.
      CALL PLOT_COLOR(1,1)
      CALL PLOT_R(0.0,0.0,2)
C  Draw remaining wires parallel to X-axis.
      CALL PLOT_COLOR(3,1)
      NYWM1=NYW-1
      DO 25 I=1,NYWM1
        CALL PLOT_R(PX,PY,3)
        DO 26 J=1,INX
          RJJ=FLOAT(J)
          RI=ABS(XMAX-RJJ*DDX)
          CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
          CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,
     1    1,U,P,Q,R)
          IF(IV.LT.0) GO TO 27
          CALL PLOT_R(PX,PY,2)
          GO TO 26
 27       CALL PLOT_R(PX,PY,3)
 26     CONTINUE
        RI=XMIN
        CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
        CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,1,
     1  U,P,Q,R)
        IF(IV.LT.0) GO TO 28
        CALL PLOT_R(PX,PY,2)
        GO TO 29
 28     CALL PLOT_R(PX,PY,3)
 29     CONTINUE
        RI=XMAX
        IF(I.EQ.NYWM1) RI=RI-DELX
        RJ=RJ+DELY
        CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
        CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,0,
     1  U,P,Q,R)
 25   CONTINUE
C  Draw remaining wires parallel to Y-axis.
      NXWM1=NXW-1
      DO 30 I=1,NXWM1
        CALL PLOT_R(PX,PY,3)
        DO 31 J=1,INY
          RJJ=FLOAT(J)
          RJ=ABS(YMAX-RJJ*DDY)
          CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
          CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,
     1    1,U,P,Q,R)
          IF(IV.LT.0) GO TO 32
          CALL PLOT_R(PX,PY,2)
          GO TO 31
 32       CALL PLOT_R(PX,PY,3)
 31     CONTINUE
        RJ=YMIN
        CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
        CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,1,
     1  U,P,Q,R)
        IF(IV.LT.0)GO TO 33
        CALL PLOT_R(PX,PY,2)
        GO TO 34
 33     CALL PLOT_R(PX,PY,3)
 34     CONTINUE
        RJ=YMAX
        RI=ABS(RI-DELX)
        CALL TWOD(G,RI,RJ,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
        CALL TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,IV,0,
     1  U,P,Q,R)
 30   CONTINUE
      CALL FLUSH_PLOT
      WRITE(6,35)
 35   FORMAT(1X,'Pause to look at perspective, type 0 to continue.')
      READ(5,*)ITRASH
      CALL CLEAR_PLOT
      RETURN
      END

      SUBROUTINE UVECT(UR,UT,UP,R,THETA,X,Y,IPAR,N,M,ZETA2S,SIG,IAZ,
     1U,P,Q,RR)
C Subroutine to calculate the magnitude and direction of displacement
C in units of the potential chi divided by 4*WE*WE*b.
      DIMENSION X(11),Y(6),U(11,6),P(11,6),Q(11,6),RR(11,6)
C Find PHI and its partial derivatives (PHIR and PHIT) at R and THETA.
      CALL INTERP(PHI,PHIT,PHIR,THETA,R,X,Y,IPAR,N,M,U,P,Q,RR)
      SIGS=SIG*SIG
      WVS=ZETA2S+SIGS-1.0
      AIAZ=IAZ
      Z=COS(THETA)
      ZS=Z*Z
      ST=SIN(THETA)
      STM1=1.0
      STM=1.0
      IAZM1=IAZ-1
      IF(IAZ.EQ.0) GO TO 10
      DO 11 I=1,IAZ
        STM=STM*ST
 11   CONTINUE
      IF(IAZ.EQ.1) GO TO 10      
      DO 12 I=1,IAZM1
        STM1=STM1*ST
 12   CONTINUE
 10   Z2SMZS=ZETA2S-ZS
      FACT=SIGS*(SIGS-1.0)
      UP2=0.0
      UP3=0.0
      UP1=SIGS*STM*ST*PHIR
      IF(R.EQ.0.0) GO TO 13
      UP2=-(WVS-SIGS)*Z*STM*PHIT/R
      UP3=-AIAZ*(SIG+ZS)*(WVS-SIGS)*STM1*PHI/R
 13   UP=(UP1+UP2+UP3)/(SIG*FACT)
      UT2=0.0
      UT3=0.0
      UT4=0.0
      UT1=SIGS*Z*STM*ST*PHIR
      IF(R.EQ.0.0) GO TO 14
      UT2=-SIGS*Z2SMZS*STM*PHIT/R
      UT3=-SIGS*Z2SMZS*STM1*AIAZ*Z*PHI/R
      UT4=-AIAZ*SIG*(WVS-SIGS)*Z*STM1*PHI/R
 14   UT=(UT1+UT2+UT3+UT4)/(SIGS*FACT)
      UR2=0.0
      UR3=0.0
      UR1=(SIGS-ZS)*PHIR
      IF(R.EQ.0.0) GO TO 15
      UR2=Z*(ST*PHIT+AIAZ*Z*PHI)/R
      UR3=AIAZ*SIG*PHI/R
 15   UR=STM*(UR1+UR2+UR3)/FACT
      RETURN
      END
 
      REAL FUNCTION ETA0(X)
      IF(X.GE.0.E0) GO TO 1
      A=X+1.E0
      B=A+A-3.E0
      ETA0=-B*A*A
      GO TO 2
    1 A=X-1.E0
      B=A+A+3.E0
      ETA0=B*A*A
    2 RETURN
      END

      REAL FUNCTION PSI0(X)
      IF(X.GE.0.E0) GO TO 1
      A=X+1.E0
      PSI0=X*A*A
      GO TO 2
    1 A=X-1.E0
      PSI0=X*A*A
    2 RETURN
      END

      REAL FUNCTION ETA0P(X)
      IF(X.GE.0.E0) GO TO 1
      ETA0P=(-6.E0*X-6.E0)*X
      GO TO 2
    1 ETA0P=(6.E0*X-6.E0)*X
    2 RETURN
      END

      REAL FUNCTION PSI0P(X)
      IF(X.GE.0.E0) GO TO 1
      PSI0P=(3.E0*X+4.E0)*X+1.E0
      GO TO 2
    1 PSI0P=(3.E0*X-4.E0)*X+1.E0
    2 RETURN
      END

      REAL FUNCTION ZETA0(X)
      IF(X.GE.0.E0) GO TO 1
      A=X+1.E0
      B=-A-A-A+4.E0
      ZETA0=A*B
      GO TO 2
    1 A=X-1.E0
      ZETA0=A*A
    2 RETURN
      END

      REAL FUNCTION ZETA0P(X)
      IF(X.GE.0.E0) GO TO 1
      ZETA0P=-6.E0*X-2.E0
      GO TO 2
    1 ZETA0P=2.E0*(X-1.E0)
    2 RETURN
      END 

      REAL FUNCTION PHI0(X)
      PHI0=X*(X+1.E0)
      RETURN
      END
     
      REAL FUNCTION PHI0P(X)
      PHI0P=2.E0*X+1.E0
      RETURN
      END

      SUBROUTINE TWOD(F,XI,YI,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
C This subroutine calculates the even/odd interpolate F at
C coordinates XI,YI given the grid vectors X(N),Y(M) and
C the corner values of function and its derivatives.
      DIMENSION X(11),Y(6),U(11,6),P(11,6),Q(11,6),R(11,6)
C Find I,J rectangle in the THETA-R plane. 
      N1=N-1
      M1=M-1
      DO 10 II=1,N1
        DX=XI-X(II)
        IF(DX.GE.0.0) I=II
 10   CONTINUE
      DO 11 JJ=1,M1
        DY=YI-Y(JJ)
        IF(DY.GE.0.0) J=JJ
 11   CONTINUE
C Calculate support functions in Z.
C Find element number in Z.
      IZ=N-I
C Find Zi and Zi+1.
      IP1=I+1
      CC=COS(X(IP1))
      DD=COS(X(I))
      DZM=DD-CC
      DZP=CC+DD
      DENOM=DZM*DZP
      Z=COS(XI)
      DZZMC=Z-CC
      DZZPC=CC+Z
      XJJR=DZZMC*DZZPC/DENOM
      DZZMD=Z-DD
      DZZPD=DD+Z
      XJJL=DZZMD*DZZPD/DENOM
      TZDD=2.E0*Z/DENOM
      ZDZIP1=Z/DD
      DDTZP1=DENOM/(2.E0*DD)
      IF(IPAR.EQ.1) GO TO 12
C Even case calculation.
      IF(IZ.GT.1) GO TO 13
C Support at Z=0.
      ETAZR=ZETA0(XJJR)
      PSIZR=0.E0
C Support at Z=Z1
      ETAZL=ZETA0(XJJL)+2.E0*PHI0(XJJL)
      PSIZL=DDTZP1*PHI0(XJJL)
      GO TO 14
 13   DDTZI=DENOM/(2.E0*CC)
      ZDZI=Z/CC
      ETAZR=ETA0(XJJR)
      PSIZR=DDTZI*PSI0(XJJR)
      ETAZL=ETA0(XJJL)
      PSIZL=DDTZP1*PSI0(XJJL)
      GO TO 14
C Odd case calculation.
 12   TZSDD=Z*TZDD
      DDTZSP=DDTZP1/DD
      IF(IZ.GT.1) GO TO 15
C Support at Z=0.
      ETAZR=0.E0
      PSIZR=Z*ZETA0(XJJR)
C Support at Z=Z1.
      ETAZL=ZDZIP1*(ZETA0(XJJL)+3.E0*DDTZSP*PHI0(XJJL))
      PSIZL=Z*DDTZSP*PHI0(XJJL)
      GO TO 14    
 15   ZDZI=Z/CC
      DDTZIS=DENOM/(2.E0*CC*CC)
      ETAZR=ZDZI*(ETA0(XJJR)-DDTZIS*PSI0(XJJR))
      PSIZR=Z*DDTZIS*PSI0(XJJR)
      ETAZL=ZDZIP1*(ETA0(XJJL)-DDTZSP*PSI0(XJJL))
      PSIZL=Z*DDTZSP*PSI0(XJJL)
 14   CONTINUE
C Calculate support functions in R.
      JP1=J+1
      DYR=YI-Y(J)
      DR=Y(JP1)-Y(J)
      XJR=DYR/DR    
      ETARR=ETA0(XJR)
      PSIRR=DR*PSI0(XJR)
      DYL=YI-Y(JP1)
      XJL=DYL/DR
      ETARL=ETA0(XJL)
      PSIRL=DR*PSI0(XJL)
C Calculate interpolated function PHI.
      F1=ETARR*ETAZR*U(IP1,J)+ETARR*PSIZR*P(IP1,J)+PSIRR*ETAZR*Q(IP1,J)+
     1 PSIRR*PSIZR*R(IP1,J)
      F2=ETARL*ETAZR*U(IP1,JP1)+ETARL*PSIZR*P(IP1,JP1)+
     1 PSIRL*ETAZR*Q(IP1,JP1)+PSIRL*PSIZR*R(IP1,JP1)
      F3=ETARL*ETAZL*U(I,JP1)+ETARL*PSIZL*P(I,JP1)+PSIRL*ETAZL*Q(I,JP1)+
     1 PSIRL*PSIZL*R(I,JP1)
      F4=ETARR*ETAZL*U(I,J)+ETARR*PSIZL*P(I,J)+PSIRR*ETAZL*Q(I,J)+
     1 PSIRR*PSIZL*R(I,J)
      F=F1+F2+F3+F4      
      ST=SIN(XI)
      STM=1.0
      IF(IAZ.EQ.0) GO TO 16
      DO 17 I=1,IAZ
        STM=STM*ST
 17   CONTINUE
 16   F=STM*F     
      RETURN
      END

      SUBROUTINE INTERP(F,FX,FY,XI,YI,X,Y,IPAR,N,M,U,P,Q,R)
C This subroutine calculates the even/odd interpolates F, Fx, Fy,
C the function and its two partial derivatives, at coordinates XI,YI
C given the grid vectors X(N),Y(M).
      DIMENSION X(11),Y(6),U(11,6),P(11,6),Q(11,6),R(11,6)
C Find I,J rectangle in THETA-R plane.
      N1=N-1
      M1=M-1
      DO 10 II=1,N1
        DX=XI-X(II)
        IF(DX.GE.0.0) I=II
 10   CONTINUE
      DO 11 JJ=1,M1
        DY=YI-Y(JJ)
        IF(DY.GE.0.0) J=JJ
 11   CONTINUE
C Calculate support functions in Z.
C Find element number in Z.
      IZ=N-I
C Find Zi and Zi+1.
      IP1=I+1
      CC=COS(X(IP1))
      DD=COS(X(I))
      DZM=DD-CC
      DZP=CC+DD
      DENOM=DZM*DZP
      Z=COS(XI)
      DZZMC=Z-CC
      DZZPC=CC+Z
      XJJR=DZZMC*DZZPC/DENOM
      DZZMD=Z-DD
      DZZPD=DD+Z
      XJJL=DZZMD*DZZPD/DENOM
      TZDD=2.E0*Z/DENOM
      ZDZIP1=Z/DD
      DDTZP1=DENOM/(2.E0*DD)
      IF(IPAR.EQ.1) GO TO 12
C Even case calculation.
      IF(IZ.GT.1) GO TO 13
C Support at Z=0.
      ETAZR=ZETA0(XJJR)
      PSIZR=0.E0
      ETAZPR=TZDD*ZETA0P(XJJR)
      PSIZPR=0.E0
C Support at Z=Z1.
      ETAZL=ZETA0(XJJL)+2.E0*PHI0(XJJL)
      PSIZL=DDTZP1*PHI0(XJJL)
      ETAZPL=TZDD*(ZETA0P(XJJL)+2.E0*PHI0P(XJJL))
      PSIZPL=ZDZIP1*PHI0P(XJJL)
      GO TO 14
 13   DDTZI=DENOM/(2.E0*CC)
      ZDZI=Z/CC
      ETAZR=ETA0(XJJR)
      PSIZR=DDTZI*PSI0(XJJR)
      ETAZPR=TZDD*ETA0P(XJJR)
      PSIZPR=ZDZI*PSI0P(XJJR)
      ETAZL=ETA0(XJJL)
      PSIZL=DDTZP1*PSI0(XJJL)
      ETAZPL=TZDD*ETA0P(XJJL)
      PSIZPL=ZDZIP1*PSI0P(XJJL)
      GO TO 14
C Odd case calculation.
 12   TZSDD=Z*TZDD
      DDTZSP=DDTZP1/DD
      IF(IZ.GT.1) GO TO 15
C Support at Z=0.
      ETAZR=0.E0
      PSIZR=Z*ZETA0(XJJR)
      ETAZPR=0.E0
      PSIZPR=TZSDD*ZETA0P(XJJR)+ZETA0(XJJR)
C Support at Z=Z1.
      ETAZL=ZDZIP1*(ZETA0(XJJL)+3.E0*DDTZSP*PHI0(XJJL))
      PSIZL=Z*DDTZSP*PHI0(XJJL)
      ETAZPL=(ZETA0(XJJL)+3.E0*DDTZSP*PHI0(XJJL)
     1 +TZSDD*ZETA0P(XJJL)+3.E0*ZDZIP1*ZDZIP1*PHI0P(XJJL))/DD
      PSIZPL=DDTZSP*PHI0(XJJL)+ZDZIP1*ZDZIP1*PHI0P(XJJL)
      GO TO 14    
 15   ZDZI=Z/CC
      DDTZIS=DENOM/(2.E0*CC*CC)
      ETAZR=ZDZI*(ETA0(XJJR)-DDTZIS*PSI0(XJJR))
      PSIZR=Z*DDTZIS*PSI0(XJJR)
      ETAZPR=(ETA0(XJJR)-DDTZIS*PSI0(XJJR)
     1 +TZSDD*ETA0P(XJJR)-ZDZI*ZDZI*PSI0P(XJJR))/CC
      PSIZPR=DDTZIS*PSI0(XJJR)+ZDZI*ZDZI*PSI0P(XJJR)
      ETAZL=ZDZIP1*(ETA0(XJJL)-DDTZSP*PSI0(XJJL))
      PSIZL=Z*DDTZSP*PSI0(XJJL)
      ETAZPL=(ETA0(XJJL)-DDTZSP*PSI0(XJJL)
     1 +TZSDD*ETA0P(XJJL)-ZDZIP1*ZDZIP1*PSI0P(XJJL))/DD
      PSIZPL=DDTZSP*PSI0(XJJL)+ZDZIP1*ZDZIP1*PSI0P(XJJL)
 14   CONTINUE
C Calculate support functions in R.
      JP1=J+1
      DYR=YI-Y(J)
      DR=Y(JP1)-Y(J)
      XJR=DYR/DR    
      ETARR=ETA0(XJR)
      PSIRR=DR*PSI0(XJR)
      ETARPR=ETA0P(XJR)/DR
      PSIRPR=PSI0P(XJR)
      DYL=YI-Y(JP1)
      XJL=DYL/DR
      ETARL=ETA0(XJL)
      PSIRL=DR*PSI0(XJL)
      ETARPL=ETA0P(XJL)/DR
      PSIRPL=PSI0P(XJL)
C Calculate interpolated function PHI.
      F1=ETARR*ETAZR*U(IP1,J)+ETARR*PSIZR*P(IP1,J)+PSIRR*ETAZR*Q(IP1,J)+
     1 PSIRR*PSIZR*R(IP1,J)
      F2=ETARL*ETAZR*U(IP1,JP1)+ETARL*PSIZR*P(IP1,JP1)+
     1 PSIRL*ETAZR*Q(IP1,JP1)+PSIRL*PSIZR*R(IP1,JP1)
      F3=ETARL*ETAZL*U(I,JP1)+ETARL*PSIZL*P(I,JP1)+PSIRL*ETAZL*Q(I,JP1)+
     1 PSIRL*PSIZL*R(I,JP1)
      F4=ETARR*ETAZL*U(I,J)+ETARR*PSIZL*P(I,J)+PSIRR*ETAZL*Q(I,J)+
     1 PSIRR*PSIZL*R(I,J)
      F=F1+F2+F3+F4
C Calculate interpolated derivative with respect to THETA.
      FX1=ETARR*ETAZPR*U(IP1,J)+ETARR*PSIZPR*P(IP1,J)+
     1 PSIRR*ETAZPR*Q(IP1,J)+PSIRR*PSIZPR*R(IP1,J)
      FX2=ETARL*ETAZPR*U(IP1,JP1)+ETARL*PSIZPR*P(IP1,JP1)+
     1 PSIRL*ETAZPR*Q(IP1,JP1)+PSIRL*PSIZPR*R(IP1,JP1)
      FX3=ETARL*ETAZPL*U(I,JP1)+ETARL*PSIZPL*P(I,JP1)+
     1 PSIRL*ETAZPL*Q(I,JP1)+PSIRL*PSIZPL*R(I,JP1)
      FX4=ETARR*ETAZPL*U(I,J)+ETARR*PSIZPL*P(I,J)+PSIRR*ETAZPL*Q(I,J)+
     1 PSIRR*PSIZPL*R(I,J)
      FX=FX1+FX2+FX3+FX4
      FX=-SIN(XI)*FX 
C Calculate interpolated derivative with respect to R.
      FY1=ETARPR*ETAZR*U(IP1,J)+ETARPR*PSIZR*P(IP1,J)+
     1 PSIRPR*ETAZR*Q(IP1,J)+PSIRPR*PSIZR*R(IP1,J)
      FY2=ETARPL*ETAZR*U(IP1,JP1)+ETARPL*PSIZR*P(IP1,JP1)+
     1 PSIRPL*ETAZR*Q(IP1,JP1)+PSIRPL*PSIZR*R(IP1,JP1)
      FY3=ETARPL*ETAZL*U(I,JP1)+ETARPL*PSIZL*P(I,JP1)+
     1 PSIRPL*ETAZL*Q(I,JP1)+PSIRPL*PSIZL*R(I,JP1)
      FY4=ETARPR*ETAZL*U(I,J)+ETARPR*PSIZL*P(I,J)+PSIRPR*ETAZL*Q(I,J)+
     1 PSIRPR*PSIZL*R(I,J)
      FY=FY1+FY2+FY3+FY4
      RETURN
      END
  
      SUBROUTINE TRANS(RI,RJ,G,PX,PY,X,Y,IPAR,N,M,IAZ,VINC,ALPHA,BETA,
     1 IV,JV,U,P,Q,R)
C   RI,RJ,G are the actual co-ordinates of the point to be plotted.
C   A,B,C are scaled three-dimensional co-ordinates with origin
C   at DELTAX,YMIN,GMIN. PX, PY are screen plot co-ordinates and 
C   IPX,IPY are their integer equivalents. The subroutine 
C   provides an option (JV=1), whereby the visibility (IV)
C   of the point in question can be computed. For JV=0, this
C   check is not made. When IV=1, the point is visible, when
C   IV=-1, it is invisible.
      DIMENSION X(11),Y(6),U(11,6),P(11,6),Q(11,6),R(11,6)
      COMMON XMIN,YMIN,GMIN,SCXY,SCX,SCY,SCFN,DDX,XMAX,YMAX
C   Assume point is visible.
      IV=1
C   Compute scaled three-dimensional co-ordinates.
      A=(RI-XMAX)*SCX
      B=(RJ-YMIN)*SCY
      C=(G-GMIN)*SCFN
C   Compute screen co-ordinates in pen increments.
      PX=B-BETA*A*SIN(ALPHA)
      PY=C-BETA*A*COS(ALPHA)
C   Compute integer screen co-ordinates in pen increments.
C   Decide whether or not to do visibility calculation.
      IF(JV.EQ.0) GO TO 10
C   Set x-co-ordinate on projector to maximum value.
      RIP=XMAX
C   If x-co-ordinate is on or in front of figure, leave
C   subroutine.
      IF(A.GE.0) GO TO 10
C   Determine if point is is behind YMAX side of perspective.
      IF(PX.LE.SCXY) GO TO 11
C   Compute x-co-ordinate on projector on YMAX side of
C   perspective.
      RIP=((SCXY-PX)/(BETA*SIN(ALPHA)))/SCX+XMAX
C   If point is on, or in front of, YMAX side of perspective,
C   leave subroutine.
      IF(B.GE.SCXY) GO TO 10
C   Find Y and Z co-ordinates of point on projector.
 11   AP=(RIP-XMAX)*SCX
      RJP=(PX+BETA*AP*SIN(ALPHA))/SCY+YMIN
      GD=PY+BETA*AP*COS(ALPHA)
C   Find height of surface above projector.
      CALL TWOD(F,RIP,RJP,X,Y,IPAR,N,M,IAZ,U,P,Q,R)
      FD=(F-GMIN)*SCFN
      IF(FD.GT.GD) GO TO 12
      RIP=RIP-VINC*DDX
      IF(RIP.GT.RI) GO TO 11
      GO TO 10
 12   IV=-IV
 10   CONTINUE
      RETURN
      END
 
      SUBROUTINE EIGEN(BML,BM,BMFS,INRR,CMI,CMM,CI,CM,EINVI,EINVM,M,
     1  IPAR,INU,NCR,ICORE,ICASE,WVS,INSV,B,SIG,ERR,IE,ALPH,BET,GAMM)
C This subroutine calculates the value of the determinant ERR 
C for a given value of the angular frequency SIG.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BML(11,34980),BM(34980),BMF(264,264),BMFS(264,264),
     1 CMI(24,44,4),CMM(24,44,4),CICB(24,44),CCMB(24,44),
     2 B(264),CI(24,44),CM(24,44),EINVI(24,24),EINVM(24,24),C(264,529)
C Construct symmetric storage vector at angular frequency SIG.
      CALL MAT(BML,BM,INSV,SIG,ICASE,WVS,ALPH,BET,GAMM)
C Restore full matrix.
      CALL MATF(BMF,BM,INRR)
C Find NM1.
      NM1=(NCR+2)/4-1
C Set H1LOVE to unity.
      H1LOVE=1.D0
      IF(M.GE.2) GO TO 10
      IF(M.EQ.1) GO TO 11
      IF(IPAR.EQ.0) GO TO 10
C Find inverse of degree one Love number for the inner core.
      H1LOVE=(ALPH*SIG+BET)*SIG+GAMM
      GO TO 10
 11   IF(IPAR.EQ.1) GO TO 10
C Find inverse of degree one Love number for the inner core.
      H1LOVE=(ALPH*SIG+BET)*SIG+GAMM
C Store full matrix at angular frequency SIG.
 10   DO 12 I=1,INRR
        DO 12 J=1,INRR
          BMFS(I,J)=BMF(I,J)
 12   CONTINUE
C Construct constraint matrices at angular frequency SIG.
      IF(INU.EQ.0) GO TO 13
      IF(ICORE.EQ.1) GO TO 14
      CALL CONMAT(CMI,CICB,INU,NCR,SIG,ICASE,H1LOVE)
 14   H1LOVE=1.D0
      CALL CONMAT(CMM,CCMB,INU,NCR,SIG,ICASE,H1LOVE)
C Find matrices connecting constrained variables to
C unconstrained variables.
C Find number of columns in constraint matrices.
 13   NCC=NCR-INU
      IF(INU.EQ.0) GO TO 15
      IF(ICORE.EQ.1) GO TO 16
      CALL CCALC(CI,EINVI,CICB,INU,NCC)
 16   CALL CCALC(CM,EINVM,CCMB,INU,NCC)
C Find constrained matrix.
 15   CALL BCON(BMF,INRR,CI,CM,INU,NCC,ICORE,M,IPAR,NM1) 
C Return constrained matrix to vector form.
      INRRC=INRR-(2-ICORE)*INU
      IF(ICORE.EQ.0) GO TO 17
      IF(M.EQ.0.AND.IPAR.EQ.0) INRRC=INRRC-NM1
 17   CALL SYMST(BMF,BM,INRRC)
C Calculate determinant of full matrix.
      CALL LINSOL(BMF,B,INRRC,C,DETLS,264,529)
      SIGN=1.D0
      IF(DETLS.LT.0.D0) SIGN=-SIGN
      DETLOG=DLOG10(DABS(DETLS))
      IE=IDINT(DETLOG)-1
      AIE=DFLOAT(IE)
      DETLOG=DETLOG-AIE
      ERR=SIGN*(10.D0**DETLOG)
      RETURN
      END

      SUBROUTINE MAT(BML,BM,INSV,SIG,ICASE,WVS,ALPH,BET,GAMM)
C This subroutine constructs the vector storage form of the main 
C matrix for a given angular frequency SIG.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BML(11,34980),BM(34980),A(11)
C Calculate square of angular frequency and related quantities.
      SIGS=SIG*SIG
      SIGSMU=SIGS-1.D0
      S4MS2=SIGS*SIGSMU
C Calculate frequency dependent coefficients A(10).
      A(1)=SIGS
      A(2)=-1.D0
      A(8)=SIG
      IF(ICASE.GT.1)GO TO 10
      A(9)=0.D0
      A(10)=0.D0
      A(11)=0.D0
      GO TO 11
 10   A(9)=S4MS2
      A(10)=A(9)
      A(11)=A(9)/((ALPH*SIG+BET)*SIG+GAMM)
 11   IF(ICASE.GT.2)GO TO 12
      A(3)=0.D0 
      A(4)=0.D0
      A(5)=0.D0
      A(6)=0.D0
      A(7)=0.D0
      GO TO 13
 12   A(7)=WVS/S4MS2
      A(6)=SIG*A(7)
      A(5)=SIG*A(6)
      A(4)=SIG*A(5)
      A(3)=SIG*A(4)
C Calculate main matrix.
 13   DO 14 I=1,INSV
        BM(I)=0.D0
        DO 14 K=1,11
          BM(I)=BM(I)+A(K)*(BML(K,I))
 14   CONTINUE  
      RETURN
      END

      SUBROUTINE MATF(BMF,BM,INRR)
C This subroutine restores the full form of the symmetric
C matrix BMF, given the vector storage form vector BM.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION BMF(264,264),BM(34980)
      DO 10 I=1,INRR
        DO 10 J=1,INRR
C Calculate row of symmetric storage vector.
          IF(I.GE.J) L=(I*(I-1))/2+J
          IF(I.LT.J) L=(J*(J-1))/2+I
          BMF(I,J)=BM(L)
 10   CONTINUE
      RETURN
      END

      SUBROUTINE SYMST(BMF,BM,INRR)
C This subroutine converts the square, symmetric matrix BMF
C to the symmetric storage vector BM.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BMF(264,264),BM(34980)
      DO 10 I=1,INRR
        DO 10 J=1,I
          K=(I*(I-1)/2)+J
          BM(K)=BMF(I,J)
 10   CONTINUE
      RETURN
      END

      SUBROUTINE CCALC(C,E,CM,NR,NCC)
C This subroutine finds the matrix C connecting the constrained
C variables to the unconstrained variables.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(24,44),CM(24,44),E(24,24),F(24,44),B(24),WKAREA(24,49)
C Find the partitions E and -F of the matrix CM.
      DO 10 I=1,NR
        DO 11 J=1,NR
          E(I,J)=CM(I,J)
 11     CONTINUE
        DO 12 JJ=1,NCC
          J=JJ+NR
          F(I,JJ)=-CM(I,J)
 12     CONTINUE
 10   CONTINUE
C Find the inverse of E.
      CALL LINSOL(E,B,NR,WKAREA,DET,24,49)
C Multiply the inverse of E into F to obtain C.
      DO 13 I=1,NR
        DO 13 J=1,NCC
          C(I,J)=0.D0
          DO 13 K=1,NR
            C(I,J)=C(I,J)+E(I,K)*F(K,J)
 13   CONTINUE
      RETURN
      END

      SUBROUTINE BCON(BMF,INRR,CI,CM,NR,NCC,ICORE,M,IPAR,NM1)
C This subroutine applies the constraint equations to find the
C constrained matrix.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BMF(264,264),CI(24,44),CM(24,44)
      IUL1=INRR-NR-NCC
      IUL2=INRR-NCC+1
      IUL1P1=IUL1+1
      IUL2M1=IUL2-1
      IF(ICORE.EQ.1.OR.NR.EQ.0) GO TO 10
C Apply constraints at inner core boundary.
      IL=NR+1
      IU=IL+NCC-1
      DO 11 I=IL,IU
        II=I-IL+1
        DO 12 J=IL,IUL1
          TEMP=0.D0
          DO 13 K=1,NR
            TEMP=TEMP+BMF(K,J)*CI(K,II)
 13       CONTINUE
          BMF(I,J)=BMF(I,J)+TEMP
          BMF(J,I)=BMF(J,I)+TEMP
 12     CONTINUE
        DO 14 J=IUL2,INRR
          TEMP=0.D0
          DO 15 K=1,NR
            TEMP=TEMP+BMF(K,J)*CI(K,II)
 15       CONTINUE
          BMF(I,J)=BMF(I,J)+TEMP
          BMF(J,I)=BMF(J,I)+TEMP
 14     CONTINUE
 11   CONTINUE
      DO 16 I=IL,IU
        II=I-IL+1
        DO 16 J=IL,IU
          JJ=J-IL+1
          TEMP=0.D0
          DO 17 K=1,NR
            DO 17 L=1,NR
              TEMP=TEMP+CI(K,II)*BMF(K,L)*CI(L,JJ)
 17       CONTINUE
        BMF(I,J)=BMF(I,J)+TEMP
 16   CONTINUE
      GO TO 18
C Adjust for no inner core boundary conditions.
 10   IL=1
C Test for even, axisymmetric case.
      IF(M.EQ.0.AND.IPAR.EQ.0) GO TO 19
      GO TO 18
C Apply constraints at geocentre for even, axisymmetric case.
 19   DO 20 J=1,IUL1
        TEMP=0.D0
        DO 21 K=1,NM1
          KK=K+1
          TEMP=TEMP+BMF(KK,J)
 21     CONTINUE
        BMF(1,J)=BMF(1,J)+TEMP
        BMF(J,1)=BMF(J,1)+TEMP
 20   CONTINUE  
      DO 22 J=IUL2,INRR
        TEMP=0.D0
        DO 23 K=1,NM1
          KK=K+1
          TEMP=TEMP+BMF(KK,J)
 23     CONTINUE
        BMF(1,J)=BMF(1,J)+TEMP
        BMF(J,1)=BMF(J,1)+TEMP
 22   CONTINUE
      TEMP=0.D0
      DO 24 K=1,NM1
        KK=K+1
        DO 24 L=1,NM1
          LL=L+1
          TEMP=TEMP+BMF(KK,LL)
 24   CONTINUE
      BMF(1,1)=BMF(1,1)+TEMP 
C Apply constraints at core-mantle boundary.
 18   IF(NR.EQ.0) GO TO 25
      DO 26 I=IUL2,INRR
        II=I-IUL2M1
        DO 27 J=IL,IUL1
          TEMP=0.D0
          DO 28 K=IUL1P1,IUL2M1
            KK=K-IUL1
            TEMP=TEMP+BMF(K,J)*CM(KK,II)
 28       CONTINUE
          BMF(I,J)=BMF(I,J)+TEMP
          BMF(J,I)=BMF(J,I)+TEMP
 27     CONTINUE
        DO 29 J=IUL2,INRR
          TEMP=0.D0
          DO 30 K=IUL1P1,IUL2M1
            KK=K-IUL1
            TEMP=TEMP+BMF(K,J)*CM(KK,II)
 30       CONTINUE
          BMF(I,J)=BMF(I,J)+TEMP
          BMF(J,I)=BMF(J,I)+TEMP
 29     CONTINUE
 26   CONTINUE 
      DO 31 I=IUL2,INRR
        II=I-IUL2M1
        DO 31 J=IUL2,INRR
          JJ=J-IUL2M1
          TEMP=0.D0
          DO 32 K=IUL1P1,IUL2M1
            KK=K-IUL1
            DO 32 L=IUL1P1,IUL2M1
              LL=L-IUL1
              TEMP=TEMP+CM(KK,II)*BMF(K,L)*CM(LL,JJ)
 32       CONTINUE
          BMF(I,J)=BMF(I,J)+TEMP
 31   CONTINUE
C Strike out rows and columns for variables determined
C by constraints.
 25   JJ=0
C Test for no inner core.
      IF(ICORE.EQ.1) GO TO 33
      IF(NR.EQ.0) GO TO 34 
      DO 35 I=1,NR
        CALL SORAC(BMF,INRR,I,JJ)
        JJ=JJ+1
 35   CONTINUE
 34   JJ=NR
      GO TO 36
C Test foe even, axisymmetric case.
 33   IF(M.EQ.0.AND.IPAR.EQ.0) GO TO 37
      GO TO 36
 37   DO 38 I=1,NM1
        II=I+1
        CALL SORAC(BMF,INRR,II,JJ)
        JJ=JJ+1
 38   CONTINUE
      JJ=NM1
 36   IF(NR.EQ.0) GO TO 39
      DO 40 I=IUL1P1,IUL2M1
        CALL SORAC(BMF,INRR,I,JJ)
        JJ=JJ+1
 40   CONTINUE
 39   CONTINUE
      RETURN
      END

      SUBROUTINE SORAC(B,INR,II,JJ)
C This subroutine strikes out the IIth row and IIth column of the
C matrix B, given that JJ rows and JJ columns have already been
C struck out.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION B(264,264)
C Find KK, the row and column number of the current matrix to be
C struck out.
      KK=II-JJ
      INRR=INR-JJ
      INRRM1=INRR-1
      DO 10 K=1,INRRM1
        DO 10 L=1,INRRM1
          IF(K.LT.KK.AND.L.LT.KK) GO TO 11
          IF(K.GE.KK.AND.L.GE.KK) GO TO 12
          IF(L.LT.KK) GO TO 13
          B(K,L)=B(K,L+1)
          GO TO 11
 13       B(K,L)=B(K+1,L)
          GO TO 11
 12       B(K,L)=B(K+1,L+1)
 11       CONTINUE
 10   CONTINUE
      DO 14 I=1,INRR
        B(I,INRR)=0.D0
        B(INRR,I)=0.D0
 14   CONTINUE
      RETURN
      END

      SUBROUTINE CMATP(BML,BMP,BMFS,INRR,CMI,CMM,CI,CM,
     1 EINVI,EINVM,M,IPAR,INU,NCR,ICORE,INSV,SIG,ICASE,WVS,
     2 ALPH,BET,GAMM)
C This subroutine calculates the derivative of the constrained
C matrix at angular frequency SIG.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BML(11,34980),BMP(34980),BMFS(264,264),BMPFS(264,264),
     1 CI(24,44),CM(24,44),EINVI(24,24),EINVM(24,24),
     2 CICBP(24,44),CCMBP(24,44),CMI(24,44,4),CMM(24,44,4),
     3 CIP(24,44),CMP(24,44)
C Construct the derivative of the symmetric storage vector at
C angular frequency SIG..
      CALL MATP(BML,BMP,INSV,SIG,ICASE,WVS,ALPH,BET,GAMM)
C Restore full storage form of the derivative matrix.
      CALL MATF(BMPFS,BMP,INRR)
C Find NM!.
      NM1=(NCR+2)/4-1
      H1LOVE=1.D0
      H1LOVP=0.D0
      IF(M.GE.2) GO TO 10
      IF(M.EQ.1) GO TO 11
      IF(IPAR.EQ.0) GO TO 10
C Find the inverse of the degree one Love number and its derivative.
      H1LOVE=(ALPH*SIG+BET)*SIG+GAMM
      H1LOVP=2.D0*ALPH*SIG+BET
      GO TO 10
 11   IF(IPAR.EQ.1) GO TO 10
C Find the inverse of the degree one Love number and its derivative.
      H1LOVE=(ALPH*SIG+BET)*SIG+GAMM
      H1LOVP=2.D0*ALPH*SIG+BET
C Construct the derivatives of the constraint matrices at angular
C frequency SIG.
 10   IF(INU.EQ.0) GO TO 12
      IF(ICORE.EQ.1) GO TO 13
      CALL CSMATP(CMI,CICBP,INU,NCR,SIG,ICASE,H1LOVE,H1LOVP)
 13   H1LOVE=1.D0
      H1LOVP=0.D0
      CALL CSMATP(CMM,CCMBP,INU,NCR,SIG,ICASE,H1LOVE,H1LOVP)
C Find derivatives of matrices connecting constrained variables to
C unconstrained variables.
C Find number of columns in derivatives of constraint matrices.
 12   NCC=NCR-INU
      IF(INU.EQ.0) GO TO 14
      IF(ICORE.EQ.1) GO TO 15
      CALL CCALCP(CIP,CICBP,CI,EINVI,INU,NCC)
 15   CALL CCALCP(CMP,CCMBP,CM,EINVM,INU,NCC)
C Find derivative of constrained matrix.
 14   CALL BCONP(BMFS,BMPFS,INRR,CI,CM,CIP,CMP,INU,NCC,ICORE,M,IPAR,NM1)       
C Return derivative of constrained matrix to vector form.
      INRRC=INRR-(2-ICORE)*INU
      IF(ICORE.EQ.0) GO TO 16
      IF(M.EQ.0.AND.IPAR.EQ.0) INRRC=INRRC-NM1
 16   CALL SYMST(BMPFS,BMP,INRRC)
      RETURN
      END        

      SUBROUTINE MATP(BML,BMP,INSV,SIG,ICASE,WVS,ALPH,BET,GAMM)
C This subroutine constructs the derivative of the vector storage
C form of the main matrix for a given angular frequency SIG.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BML(11,34980),BMP(34980),B(11)
C Calculate the square of the angular frequency and related quantities.
      SIGS=SIG*SIG
      SIGSMU=SIGS-1.D0
      TWOSIG=2.D0*SIG
C Calculate the frequency dependent coefficients B(11).
      B(1)=TWOSIG
      B(2)=0.D0
      B(8)=1.D0
      IF(ICASE.GT.1)GO TO 10
      B(9)=0.D0
      B(10)=0.D0
      B(11)=0.D0
      GO TO 11
 10   B(9)=TWOSIG*(SIGSMU+SIGS)
      B(10)=B(9)
      H1LOVE=(ALPH*SIG+BET)*SIG+GAMM
      H1LOVP=TWOSIG*ALPH+BET
      B(11)=B(9)/H1LOVE-SIGS*SIGSMU*H1LOVP/(H1LOVE*H1LOVE)
 11   IF(ICASE.GT.2)GO TO 12
      B(3)=0.D0
      B(4)=0.D0
      B(5)=0.D0
      B(6)=0.D0
      B(7)=0.D0
      GO TO 13
 12   FACT=WVS/SIGSMU
      B(5)=-TWOSIG*FACT/SIGSMU
      B(4)=FACT+SIG*B(5)
      B(3)=TWOSIG*FACT+SIGS*B(5)
      B(6)=-FACT/SIGS+B(5)/SIG
      B(7)=-2.D0*FACT/(SIG*SIGS)+B(5)/SIGS
C Calculate the derivative of the main matrix.
 13   DO 14 I=1,INSV
        BMP(I)=0.D0
        DO 14 K=1,11
          BMP(I)=BMP(I)+B(K)*(BML(K,I))
 14   CONTINUE
      RETURN
      END

      SUBROUTINE CONMAT(CML,CM,NR,NC,SIG,ICASE,H1LOVE)
C This subroutine constructs the constraint matrix at angular
C frequency SIG.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION CML(24,44,4),CM(24,44)
      FACT=1.D0
      IF(ICASE.EQ.1) FACT=0.D0
      DO 10 I=1,NR
        DO 10 J=1,NC
          IF(I.EQ.1)GO TO 11
          CM(I,J)=(((CML(I,J,4)*FACT*SIG)*SIG+CML(I,J,3)-CML(I,J,4)
     1    *FACT)*SIG+CML(I,J,2))*SIG+CML(I,J,1)
          GO TO 10
 11       CM(I,J)=(((CML(I,J,4)*FACT*SIG)*SIG+H1LOVE*CML(I,J,3)
     1    -CML(I,J,4)*FACT)*SIG+H1LOVE*CML(I,J,2))*SIG+H1LOVE*CML(I,J,1)
 10   CONTINUE
      RETURN
      END

      SUBROUTINE CSMATP(CML,CM,NR,NC,SIG,ICASE,H1LOVE,H1LOVP)
C This subroutine constructs the derivative of the constraint matrix
C at angular frequency SIG.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION CML(24,44,4),CM(24,44)
      TSIG=2.D0*SIG
      FACT=1.D0
      IF(ICASE.EQ.1) FACT=0.D0
      DO 10 I=1,NR
        DO 10 J=1,NC
          IF(I.EQ.1) GO TO 11
          CM(I,J)=((CML(I,J,4)*FACT*SIG)*TSIG+CML(I,J,3)-CML(I,J,4)
     1    *FACT)*TSIG+CML(I,J,2)
          GO TO 10
 11       CM(I,J)=((CML(I,J,4)*FACT*SIG)*TSIG+(H1LOVE+0.5D0*SIG*H1LOVP)
     1    *CML(I,J,3)-CML(I,J,4)*FACT)*TSIG+(H1LOVE+SIG*H1LOVP)
     2    *CML(I,J,2)+H1LOVP*CML(I,J,1)
 10   CONTINUE
      RETURN
      END

      SUBROUTINE CCALCP(C,CM,CMC,EINV,NR,NCC)
C This subroutine finds the derivative of the matrix C connecting
C the constrained variables to the unconstrained variables.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(24,44),CM(24,44),EP(24,24),FP(24,44),CMC(24,44),
     1 EINV(24,24)
C Find the partitions EP and -FP of the matrix CM
      DO 10 I=1,NR
        DO 11 J=1,NR
          EP(I,J)=CM(I,J)
 11     CONTINUE
        DO 12 JJ=1,NCC
          J=JJ+NR
          FP(I,JJ)=-CM(I,J)
 12     CONTINUE
 10   CONTINUE
C Subtract EP*CMC from FP.
      DO 13 I=1,NR
        DO 13 J=1,NCC
          DO 13 K=1,NR
            FP(I,J)=FP(I,J)-EP(I,K)*CMC(K,J)
 13   CONTINUE
C Multiply result by EINV to obtain derivative of C.
      DO 14 I=1,NR
        DO 14 J=1,NCC
          C(I,J)=0.D0
          DO 14 K=1,NR
            C(I,J)=C(I,J)+EINV(I,K)*FP(K,J)
 14   CONTINUE
      RETURN
      END

      SUBROUTINE BCONP(BMF,BMPF,INRR,CI,CM,CIP,CMP,NR,NCC,ICORE,
     1 M,IPAR,NM1)    
C This subroutine calculates the derivative of the constrained matrix
C at angular frequency SIG.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BMF(264,264),BMPF(264,264),CI(24,44),CM(24,44),
     1 CIP(24,44),CMP(24,44)
      IUL1=INRR-NR-NCC
      IUL2=INRR-NCC+1
      IUL1P1=IUL1+1
      IUL2M1=IUL2-1
      IF(ICORE.EQ.1.OR.NR.EQ.0) GO TO 10
C Apply constraints at the inner core boundary.
      IL=NR+1
      IU=IL+NCC-1
      DO 11 I=IL,IU
      II=I-IL+1
        DO 12 J=IL,IUL1
          TEMP1=0.D0
          TEMP2=0.D0
          DO 13 K=1,NR
            TEMP1=TEMP1+BMPF(K,J)*CI(K,II)
            TEMP2=TEMP2+BMF(K,J)*CIP(K,II)
 13       CONTINUE
          BMPF(I,J)=BMPF(I,J)+TEMP1+TEMP2
          BMPF(J,I)=BMPF(J,I)+TEMP1+TEMP2
 12     CONTINUE
        DO 14 J=IUL2,INRR
          TEMP1=0.D0
          TEMP2=0.D0
          DO 15 K=1,NR
            TEMP1=TEMP1+BMPF(K,J)*CI(K,II)
            TEMP2=TEMP2+BMF(K,J)*CIP(K,II)
 15       CONTINUE
          BMPF(I,J)=BMPF(I,J)+TEMP1+TEMP2
          BMPF(J,I)=BMPF(J,I)+TEMP1+TEMP2
 14     CONTINUE
 11   CONTINUE
      DO 16 I=IL,IU
        II=I-IL+1
        DO 16 J=IL,IU
          JJ=J-IL+1
          TEMP1=0.D0
          TEMP2=0.D0
          DO 17 K=1,NR
            DO 17 L=1,NR
              TEMP1=TEMP1+CI(K,II)*BMPF(K,L)*CI(L,JJ)
              TEMP2=TEMP2+CIP(K,II)*BMF(K,L)*CI(L,JJ)
 17       CONTINUE
          BMPF(I,J)=BMPF(I,J)+TEMP1+TEMP2
          BMPF(J,I)=BMPF(J,I)+TEMP2
 16   CONTINUE
      GO TO 18
C Adjust for no inner core boundary conditions.
 10   IL=1  
C Test for even, axisymmetric case.
      IF(M.EQ.0.AND.IPAR.EQ.0) GO TO 19
      GO TO 18
C Apply constraints at the geocentre for the even, axisymmetric case.
 19   DO 20 J=1,IUL1
        TEMP=0.D0
        DO 21 K=1,NM1
          KK=K+1
          TEMP=TEMP+BMPF(KK,J)
 21     CONTINUE
        BMPF(1,J)=BMPF(1,J)+TEMP
        BMPF(J,1)=BMPF(J,1)+TEMP
 20   CONTINUE
      DO 22 J=IUL2,INRR
        TEMP=0.D0
        DO 23 K=1,NM1
          KK=K+1
          TEMP=TEMP+BMPF(KK,J)
 23     CONTINUE
        BMPF(1,J)=BMPF(1,J)+TEMP
        BMPF(J,1)=BMPF(J,1)+TEMP
 22   CONTINUE
      TEMP=0.D0
      DO 24 K=1,NM1
      KK=K+1
      DO 24 L=1,NM1
        LL=L+1
        TEMP=TEMP+BMPF(KK,LL)
 24   CONTINUE
      BMPF(1,1)=BMPF(1,1)+TEMP
C Apply constraints at the core-mantle boundary.
 18   IF(NR.EQ.0) GO TO 25
      DO 26 I=IUL2,INRR
        II=I-IUL2M1
        DO 27 J=IL,IUL1
          TEMP1=0.D0
          TEMP2=0.D0
          DO 28 K=IUL1P1,IUL2M1
            KK=K-IUL1
            TEMP1=TEMP1+BMPF(K,J)*CM(KK,II)
            TEMP2=TEMP2+BMF(K,J)*CMP(KK,II)
 28       CONTINUE
          BMPF(I,J)=BMPF(I,J)+TEMP1+TEMP2
          BMPF(J,I)=BMPF(J,I)+TEMP1+TEMP2
 27     CONTINUE
        DO 29 J=IUL2,INRR
          TEMP1=0.D0
          TEMP2=0.D0
          DO 30 K=IUL1P1,IUL2M1
            KK=K-IUL1
            TEMP1=TEMP1+BMPF(K,J)*CM(KK,II)
            TEMP2=TEMP2+BMF(K,J)+CMP(KK,II)
 30       CONTINUE
          BMPF(I,J)=BMPF(I,J)+TEMP1+TEMP2
          BMPF(J,I)=BMPF(J,I)+TEMP1+TEMP2
 29     CONTINUE
 26   CONTINUE
      DO 31 I=IUL2,INRR
        II=I-IUL2M1
        DO 31 J=IUL2,INRR
          JJ=J-IUL2M1
          TEMP1=0.D0
          TEMP2=0.D0
          DO 32 K=IUL1P1,IUL2M1
            KK=K-IUL1
            DO 32 L=IUL1P1,IUL2M1
              LL=L-IUL1
              TEMP1=TEMP1+CM(KK,II)*BMPF(K,L)*CM(LL,JJ)
              TEMP2=TEMP2+CMP(KK,II)*BMF(K,L)*CM(LL,JJ)
 32       CONTINUE
          BMPF(I,J)=BMPF(I,J)+TEMP1+TEMP2
          BMPF(J,I)=BMPF(J,I)+TEMP2
 31   CONTINUE
C Strike out rows and columns for variables determined by constraints.
 25   JJ=0
C Test for no inner core.
      IF(ICORE.EQ.1) GO TO 33
      IF(NR.EQ.0) GO TO 34
      DO 35 I=1,NR
        CALL SORAC(BMPF,INRR,I,JJ)
        JJ=JJ+1
 35   CONTINUE
 34   JJ=NR
      GO TO 36
C Test for even, axisymmetric case.
 33   IF(M.EQ.0.AND.IPAR.EQ.0) GO TO 37
      GO TO 36
 37   DO 38 I=1,NM1
        II=I+1
        CALL SORAC(BMPF,INRR,II,JJ) 
        JJ=JJ+1
 38   CONTINUE
      JJ=NM1
 36   IF(NR.EQ.0) GO TO 39
      DO 40 I=IUL1P1,IUL2M1
        CALL SORAC(BMPF,INRR,I,JJ)
        JJ=JJ+1
 40   CONTINUE
 39   CONTINUE
      RETURN
      END

      SUBROUTINE MULT(BM,BI,B,INR)
C This subroutine multiplies the matrix BM in symmetric storage form
C into the vector BI to produce a new vector B.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION BM(34980),BI(264),B(264)
      DO 10 I=1,INR
        B(I)=0.D0
        DO 10 J=1,INR
C Calculate row of symmetric storage vector.
          IF(I.GE.J) L=(I*(I-1))/2+J
          IF(I.LT.J) L=(J*(J-1))/2+I
          B(I)=B(I)+BM(L)*BI(J)
 10   CONTINUE
      RETURN
      END
C
      SUBROUTINE LINSOL(A,B,N,C,DET,M1,M2)
C
C LINSOL solves the linear system Ax=b by Gauss-Jordan elimination
C with partial pivoting. A is the N x N coefficient matrix
C and B is the N-length constant vector b. The inverse of A
C is returned in A, the solution vector x is returned in B,
C and the determinant of A is returned in DET. C is the N x 2N+1
C augmented matrix used internally. M1 is the maximum dimension
C for N specified in the main programme and M2=2M1+1 is
C the maximum dimension for 2N+1 used in the main programme.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(M1,M1),B(M1),C(M1,M2),HOLD(M1,M2),
     1  IR(M1),IC(M1),IDET(M1)
C Construct the augmented matrix C.
      N2=2*N+1
      NP1=N+1
      NM1=N-1
C Insert the matrix A.
      DO 10 I=1,N
        DO 10 J=1,N
          C(I,J)=A(I,J)
 10   CONTINUE
C Insert the unknown vector.
      DO 11 J=1,N
        C(J,NP1)=B(J)
 11   CONTINUE
C Augment with the identity matrix.
      DO 12 I=1,N
        DO 12 J=1,N
          JJ=J+NP1
          C(I,JJ)=0.D0
          IF(I.EQ.J)C(I,JJ)=1.D0
 12   CONTINUE
C Begin Gauss-Jordan elimination.
      DET=1.D0
      DO 13 I=1,N
        IM1=I-1
C Search for largest pivot.
        PIVOT=0.D0
        DO 14 J=1,N
          DO 14 K=1,N
C Scan IR and IC vectors for invalid pivot subscripts.
            IF(I.EQ.1)GO TO 15
            DO 16 ISC=1,IM1
              DO 16 JSC=1,IM1
                IF(J.EQ.IR(ISC))GO TO 14
                IF(K.EQ.IC(JSC))GO TO 14
 16         CONTINUE
 15         IF(DABS(C(J,K)).LE.DABS(PIVOT))GO TO 14
            PIVOT=C(J,K)
            IR(I)=J
            IC(I)=K
 14     CONTINUE
        DET=DET*PIVOT
C Label next row.
        IRP1=IR(I)+1
C Label previous row.
        IRM1=IR(I)-1
C Divide row by pivot.
        DO 17 J=1,N2
             C(IR(I),J)=C(IR(I),J)/PIVOT
 17     CONTINUE
C Reduce elements below pivot to zero.
          IF(IRP1.GT.N)GO TO 18
          DO 19 K=IRP1,N
            ELL=C(K,IC(I))
            DO 20 J=1,N2
              C(K,J)=C(K,J)-C(IR(I),J)*ELL
 20         CONTINUE
 19       CONTINUE
 18       CONTINUE
C Reduce elements above pivot to zero.
          IF(IRM1.LT.1)GO TO 21
          DO 22 K=1,IRM1
            ELL=C(K,IC(I))
            DO 23 J=1,N2
              C(K,J)=C(K,J)-C(IR(I),J)*ELL
 23         CONTINUE
 22       CONTINUE
 21       CONTINUE
 13   CONTINUE     
C Unscramble equations and record row interchanges.
      DO 24 I=1,N
        IDET(IR(I))=IC(I)
        DO 24 J=1,N2
          HOLD(IC(I),J)=C(IR(I),J)
 24   CONTINUE
C Put equations back in original matrix.
      DO 25 I=1,N
        DO 25 J=1,N2
          C(I,J)=HOLD(I,J)
 25   CONTINUE
C Determine sign of determinant.
      ICH=0
      DO 26 I=1,NM1
        IP1=I+1
        DO 26 J=IP1,N
          IF(IDET(J).GE.IDET(I))GO TO 26
          IHOLD=IDET(J)
          IDET(J)=IDET(I)
          IDET(I)=IHOLD
          ICH=ICH+1
 26   CONTINUE
      IF((ICH/2)*2.NE.ICH)DET=-DET
C Put inverse in A and solution in B.
      DO 27 I=1,N
        B(I)=C(I,NP1)
        DO 27 J=1,N
          JJ=J+NP1
          A(I,J)=C(I,JJ)
 27   CONTINUE
      RETURN
      END
C
      SUBROUTINE SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C
C SPMAT computes the N1 by N1 matrix C connecting
C second derivatives to function values at N1 nodal points R(I).
C B is the N3=N1-2 by N2=2*N1-2 augmented matrix used internally
C in the Gaussian elimination procedure.
C M1, M2, M3 are the respective maximum dimensions for N1, N2, N3
C specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),B(M3,M2)
      N=N3+1
C Construct augmented matrix A.
      DO 11 I=1,N3
        DO 10 J=1,N2
          B(I,J)=0.D0
  10    CONTINUE
        B(I,I)=2.D0*(R(I+2)-R(I))
        B(I,I+N3)=1.D0
        IF(I.EQ.1.OR.I.EQ.N3) GO TO 11
        B(I,I-1)=R(I+1)-R(I)
        B(I,I+1)=R(I+2)-R(I+1)
  11  CONTINUE
C Apply conditions that third derivatives are constant
C in first two and last two intervals.
      B(1,2)=R(3)-R(2)-((R(2)-R(1))**2.D0)/(R(3)-R(2))
      B(N3,N3-1)=R(N3+1)-R(N3)-((R(N1)-R(N1-1))**2.D0)/(R(N1-1)-R(N1-2))
      B(1,1)=B(1,1)+((R(3)-R(1))*(R(2)-R(1)))/(R(3)-R(2))
      B(N3,N3)=B(N3,N3)
     1 +((R(N1)-R(N1-1))*(R(N1)-R(N1-2)))/(R(N1-1)-R(N1-2))
      NM1=N3-1
C Eliminate sub-diagonal.
      DO 13 I=1,NM1
        DIAG=B(I,I)
        ELL=B(I+1,I)
        DO 12 J=1,N2
          B(I,J)=B(I,J)/DIAG
          B(I+1,J)=B(I+1,J)-B(I,J)*ELL
  12    CONTINUE
  13  CONTINUE
C Eliminate super-diagonal.
      DO 15 I=1,NM1
        NO=N-I
        DIAG=B(NO,NO)
        ELL=B(NO-1,NO)
        DO 14 J=1,N2
          B(NO,J)=B(NO,J)/DIAG
          B(NO-1,J)=B(NO-1,J)-B(NO,J)*ELL
  14    CONTINUE
  15  CONTINUE
C Move inverse to position formerly occupied by A.
      DO 17 I=1,N3
        DO 16 J=1,N3
          B(I,J)=B(I,J+N3)
  16    CONTINUE
  17  CONTINUE
C Construct matrix B in N-2 by N augmentation space. 
      DO 19 I=1,N3
        DO 18 J=1,N1
          B(I,J+N3)=0.0D0
  18    CONTINUE
        IN=I+N3
        FACTOR=6.D0/((R(I+2)-R(I+1))*(R(I+1)-R(I)))
        B(I,IN)=FACTOR*(R(I+2)-R(I+1))
        B(I,IN+1)=-FACTOR*(R(I+2)-R(I))
        B(I,IN+2)=FACTOR*(R(I+1)-R(I))
  19  CONTINUE
C Multiply inverse of A into B to get N-2 by N version of C.
      DO 22 I=1,N3
        DO 21 J=1,N1
          SUM=0.D0
          DO 20 K=1,N3
            SUM=SUM+B(I,K)*B(K,J+N3)
  20      CONTINUE
          C(I+1,J)=SUM
  21    CONTINUE
  22  CONTINUE
C Add first and last rows to C.
      DO 23 J=1,N1
        C(1,J)=C(2,J)*(R(3)-R(1))/(R(3)-R(2))
     1 -C(3,J)*(R(2)-R(1))/(R(3)-R(2))
        C(N1,J)=C(N1-1,J)*(R(N1)-R(N1-2))/(R(N1-1)-R(N1-2))
     1 -C(N1-2,J)*(R(N1)-R(N1-1))/(R(N1-1)-R(N1-2))
  23  CONTINUE
      RETURN
      END
C
      SUBROUTINE INTPL(XV,YV,N1,C,R,S,M1)
C
C INTPL interpolates to find YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is
C the coefficient matrix, from subroutine SPMAT, relating
C the second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second derivative SUM2 at end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE
C Find interpolate YV at XV.
      YV=S(J)+(S(J+1)-S(J))*(XV-R(J))/(R(J+1)-R(J))
     1  +(XV-R(J))*(XV-R(J+1))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     2  +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END
