      SUBROUTINE FFT2N(LG,G,IS)
C FFT2N finds the discrete Fourier transform of the sequence G of length
C equal an integral power, N, of 2, for input IS=-1. For input IS=1,
C it finds the inverse of the DFT sequence G of length LG, again an integral
C power, N, of 2. The input sequences are first decimated into basic pairs,
C which are then successively transformed recursively by doubling to
C the final resulting sequence. On output, the resulting sequence replaces
C the input sequence.
C
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DOUBLE PRECISION PI,ARG,ARGM,FACT
      DIMENSION G(LG)
C Define pi.
      PI=3.14159265358979D0
C Define common factor in argument of W.
      ARG=PI*DFLOAT(IS)
C Determine if DFT or inverse DFT.
      FACT=1.D0
      IF(IS.EQ.1) FACT=FACT/DFLOAT(LG)
C Begin sequence decimation and scaling.
      J=1
      DO 10 I=1,LG
        IF(I.GT.J) GO TO 20
        HOLD=G(J)*FACT
        G(J)=G(I)*FACT
        G(I)=HOLD
 20     M=LG/2
 30     IF(J.LE.M) GO TO 40
        J=J-M
        M=M/2
        IF(M.GE.1) GO TO 30
 40     J=J+M
 10   CONTINUE
C If input sequence is of unit length, exit.
      IF(LG.EQ.1) GO TO 70
C Start calculation of DFTs by doubling.
C Set initial DFT length as power of 2.
      L=1
 50   LT2=L*2
      DO 60 M=1,L
        MM1=M-1
C Find argument of W raised to required power.
        ARGM=DFLOAT(MM1)*ARG/DFLOAT(L)
C Find W to required power.
        W=DCMPLX(DCOS(ARGM),DSIN(ARGM))
C Find double length transforms.
        DO 60 K=M,LG,LT2
          HOLD=W*G(K+L)
          G(K+L)=G(K)-HOLD
          G(K)=G(K)+HOLD
 60   CONTINUE
C Update DFT length.
      L=LT2
C Test if transform is complete.
      IF(L.LT.LG) GO TO 50
 70   CONTINUE
      RETURN   
      END
