C                  PROGRAMME PPINTP.FOR
C
C PPINTP.FOR does a natural spline interpolation of the VLBI pole path
C onto 3,513 equally spaced time points.  The record segment is set
C to begin at 4.0 years into the VLBI series and to end at 30.25 years,
C for a segment length of 26.25 years.  There are then 3,513 internal nodes
C at time points 48 to 3,560.  Values at the beginning and end of
C the record segment are found by assuming constant slope in the first two
C and last two intervals.  The corresponding calendar dates are found by
C calls to the subroutine CALDAT.FOR.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DAY(4000),XWOB(4000),YWOB(4000),C(4000,4000),
     1 B(4000,8000),GTX(4000),GTY(4000)
      OPEN(UNIT=1,FILE='ppy2009a.dat',STATUS='OLD')
      OPEN(UNIT=2,FILE='ppintp.dat',STATUS='UNKNOWN')
C Set number of points, N, and number of interpolates, M.
      N=3513
      M=3513
      AM=DFLOAT(M)
C Set segment length.
      T=26.25D0
C Find time increment.
      DT=T/AM
C Read in unused initial record segment, points 1 to 47.
      DO 10 I=1,47
        READ(1,11)K,IYYY,MM,ID,TD,X,Y,EX,EY
 11     FORMAT(1X,4I5,F10.6,4F11.6)
 10   CONTINUE
C Read in segment to be interpolated, points 48 to 3,560.
      DO 12 I=1,N
        IP1=I+1
        READ(1,11)K,IYYY,MM,ID,DAY(IP1),XWOB(IP1),YWOB(IP1),EXWOB,EYWOB
 12   CONTINUE
C Add nodes at beginning and end by assuming first derivatives are constant
C in first two and last two subintervals.
      DAY(1)=4.D0
C Find slope between second and third nodes.
      SLOPE=(XWOB(3)-XWOB(2))/(DAY(3)-DAY(2))
      XWOB(1)=SLOPE*(DAY(1)-DAY(2))+XWOB(2)
      SLOPE=(YWOB(3)-YWOB(2))/(DAY(3)-DAY(2))
      YWOB(1)=SLOPE*(DAY(1)-DAY(2))+YWOB(2)
      DAY(3515)=30.25D0
C Find slope betwen third-last and second-last nodes.
      SLOPE=(XWOB(3514)-XWOB(3513))/(DAY(3514)-DAY(3513))
      XWOB(3515)=SLOPE*(DAY(3515)-DAY(3514))+XWOB(3514)
      SLOPE=(YWOB(3514)-YWOB(3513))/(DAY(3514)-DAY(3513))
      YWOB(3515)=SLOPE*(DAY(3515)-DAY(3514))+YWOB(3514)
C Set number of nodes.
      N1=N+1
      N2=2*N1-2
      N3=N1-2
      M1=4000
      M2=8000
      M3=4000
C Construct interpolation matrix C.
      CALL SPMAT(N1,N2,N3,C,DAY,B,M1,M2,M3)
C Find equispaced time sequence.
C Find time from beginning of record.
      TIME=4.D0-DT/2.D0
      DO 13 I=1,M
        TIME=TIME+DT
C Convert time to days from beginning of the record.
        DAYS=365.25636D0*TIME
C Convert to Julian days.
        DAYS=DAYS+44089.993750D0+2400000.D0
C Find calendar date.
        IDAYS=INT(DAYS)
        CALL CALDAT(IDAYS,MM,ID,IYYY)
C Interpolate co-ordinates
        CALL INTPL(TIME,XX,N1,C,DAY,XWOB,M1)
        CALL INTPL(TIME,YY,N1,C,DAY,YWOB,M1)
        GTX(I)=XX
        GTY(I)=YY
C Write out equispaced time domain sequence.
        WRITE(2,14)I,IYYY,MM,ID,TIME,GTX(I),GTY(I)
 14     FORMAT(1X,4I5,F12.7,2F12.6)
 13   CONTINUE
      END
C
      SUBROUTINE SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C
C SPMAT computes the N1 by N1 matrix C connecting
C second derivatives to function values at N1 nodal points R(I).
C B is the N3=N1-2 by N2=2*N1-2 augmented matrix used internally
C in the Gaussian elimination procedure.
C M1, M2, M3 are the respective maximum dimensions for N1, N2, N3
C specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),B(M3,M2)
      N=N3+1
C Construct augmented matrix A.
      DO 11 I=1,N3
        DO 10 J=1,N2
          B(I,J)=0.D0
  10    CONTINUE
        B(I,I)=2.D0*(R(I+2)-R(I))
        B(I,I+N3)=1.D0
        IF(I.EQ.1.OR.I.EQ.N3) GO TO 11
        B(I,I-1)=R(I+1)-R(I)
        B(I,I+1)=R(I+2)-R(I+1)
  11  CONTINUE
C Apply conditions that third derivatives are constant
C in first two and last two intervals.
      B(1,2)=R(3)-R(2)-((R(2)-R(1))**2.D0)/(R(3)-R(2))
      B(N3,N3-1)=R(N3+1)-R(N3)-((R(N1)-R(N1-1))**2.D0)/(R(N1-1)-R(N1-2))
      B(1,1)=B(1,1)+((R(3)-R(1))*(R(2)-R(1)))/(R(3)-R(2))
      B(N3,N3)=B(N3,N3)
     1 +((R(N1)-R(N1-1))*(R(N1)-R(N1-2)))/(R(N1-1)-R(N1-2))
      NM1=N3-1
C Eliminate sub-diagonal.
      DO 13 I=1,NM1
        DIAG=B(I,I)
        ELL=B(I+1,I)
        DO 12 J=1,N2
          B(I,J)=B(I,J)/DIAG
          B(I+1,J)=B(I+1,J)-B(I,J)*ELL
  12    CONTINUE
  13  CONTINUE
C Eliminate super-diagonal.
      DO 15 I=1,NM1
        NO=N-I
        DIAG=B(NO,NO)
        ELL=B(NO-1,NO)
        DO 14 J=1,N2
          B(NO,J)=B(NO,J)/DIAG
          B(NO-1,J)=B(NO-1,J)-B(NO,J)*ELL
  14    CONTINUE
  15  CONTINUE
C Move inverse to position formerly occupied by A.
      DO 17 I=1,N3
        DO 16 J=1,N3
          B(I,J)=B(I,J+N3)
  16    CONTINUE
  17  CONTINUE
C Construct matrix B in N-2 by N augmentation space. 
      DO 19 I=1,N3
        DO 18 J=1,N1
          B(I,J+N3)=0.0D0
  18    CONTINUE
        IN=I+N3
        FACTOR=6.D0/((R(I+2)-R(I+1))*(R(I+1)-R(I)))
        B(I,IN)=FACTOR*(R(I+2)-R(I+1))
        B(I,IN+1)=-FACTOR*(R(I+2)-R(I))
        B(I,IN+2)=FACTOR*(R(I+1)-R(I))
  19  CONTINUE
C Multiply inverse of A into B to get N-2 by N version of C.
      DO 22 I=1,N3
        DO 21 J=1,N1
          SUM=0.D0
          DO 20 K=1,N3
            SUM=SUM+B(I,K)*B(K,J+N3)
  20      CONTINUE
          C(I+1,J)=SUM
  21    CONTINUE
  22  CONTINUE
C Add first and last rows to C.
      DO 23 J=1,N1
        C(1,J)=C(2,J)*(R(3)-R(1))/(R(3)-R(2))
     1 -C(3,J)*(R(2)-R(1))/(R(3)-R(2))
        C(N1,J)=C(N1-1,J)*(R(N1)-R(N1-2))/(R(N1-1)-R(N1-2))
     1 -C(N1-2,J)*(R(N1)-R(N1-1))/(R(N1-1)-R(N1-2))
  23  CONTINUE
      RETURN
      END
C
      SUBROUTINE INTPL(XV,YV,N1,C,R,S,M1)
C
C INTPL interpolates to find YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is
C the coefficient matrix, from subroutine SPMAT, relating
C the second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second deivative SUM2 at end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE
C Find interpolate YV at XV.
      YV=S(J)+(S(J+1)-S(J))*(XV-R(J))/(R(J+1)-R(J))
     1  +(XV-R(J))*(XV-R(J+1))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     2  +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END
