C A programme to test the subroutine CUMDF and its associated subroutines
C GAMI, SEGAMI, CFGAMI,and ALOGAM by computing the cumulative
C distribution function for the product of four spectra found from four, five
C seven and eight segments with 75\% overlap as given on page 197 of
C Earth Dynamics.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION ANU(4),A(4)
C Set number of random variables.
      N=4
C Enter degrees of freedom.
      ANU(1)=5.8522880D0
      ANU(2)=7.1850360D0
      ANU(3)=9.8583334D0
      ANU(4)=11.1968478D0
    6 WRITE(6,3)
    3 FORMAT(1X,'Enter z value')
      READ(5,*)Z
      CALL CUMDF(CDF,Z,ANU,A,N)
    1 WRITE(6,2)Z,CDF
    2 FORMAT(1X,2D20.8)
      WRITE(6,4)
    4 FORMAT(1X,'Enter 0 for new z, 1 for end')
      READ(5,*)ISW
      IF(ISW)5,6,5
    5 CONTINUE 
      END
C
      SUBROUTINE CUMDF(CDF,Z,ANU,A,N)
C
C A subroutine for the direct numerical calculation of the cumulative
C distribution function for N, chi-squared distributed random variables
C multiplied together.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION ANU(N),A(N),X(5),W(5),TN(11),XIJ(10,5),BMA(10)
C Take natural logarithm of Z.
      Z=DLOG(Z)
C Construct array of half-degrees of freedom.
      DO 10 I=1,N
        A(I)=ANU(I)/2.D0
 10   CONTINUE  
C Enter abscissas for 5 point Gaussian integration.
      X(1)=-0.906179845938664D0
      X(2)=-0.538469310105683D0
      X(3)=0.D0
      X(4)=-X(2)
      X(5)=-X(1)
C Enter weights for 5 point Gaussian integration.
      W(1)=0.236926885056189D0
      W(2)=0.478628670499366D0
      W(3)=0.568888888888889D0
      W(4)=W(2)
      W(5)=W(1)
C Find natural logarithms of weights.
      DO 11 I=1,5
        W(I)=DLOG(W(I))
 11   CONTINUE  
C Set node points according to 1-cos(pi*j/20), j=0,1,...,10.
      TN(1)=0.D0
      TN(2)=0.0123117D0
      TN(3)=0.0489435D0
      TN(4)=0.1089935D0
      TN(5)=0.1909830D0
      TN(6)=0.2928932D0
      TN(7)=0.4122148D0
      TN(8)=0.5460095D0
      TN(9)=0.6909830D0
      TN(10)=0.8435655D0
      TN(11)=1.D0
C Raise 1-cos(pi*j/20) to the power 4.
      DO 12 I=1,11
        TN(I)=TN(I)**4
 12   CONTINUE  
C Calculate abscissas for ith subinterval, jth integration point.
      DO 13 I=1,10
        IP1=I+1
C Find a.
        AA=TN(I)
C Find b.
        BB=TN(IP1)
C Find (b-a)/2
        BMA(I)=0.5D0*(BB-AA)
C Find (b+a)/2
        BPA=0.5D0*(BB+AA)
        DO 13 J=1,5
C Find abscissa t(i,j).
          TIJ=BMA(I)*X(J)+BPA
C Take logarithm of negative logarithm of abscissa.      
          XIJ(I,J)=DLOG(-DLOG(TIJ))
 13   CONTINUE    
      DO 14 I=1,10
C Find logarithm of (b-a)/2.
        BMA(I)=DLOG(BMA(I))
 14   CONTINUE  
C Begin calculation of cumulative distribution function specialized to N=4.
      CDF=0.D0
C Find contribution of inner integral.
      DO 15 I=1,10
        DO 15 IJ=1,5
C Find contribution of middle integral.
          DO 15 J=1,10
            DO 15 JJ=1,5
C Find contribution of outer integral.
              DO 15 K=1,10
                DO 15 KJ=1,5
C Find argument x (XARG) for incomplete gamma function P(a,x).
                  XARG=Z-XIJ(I,IJ)-XIJ(J,JJ)-XIJ(K,KJ)
                  XARG=DEXP(XARG)/16.D0
                  CALL GAMI(GP,A(N),XARG)
C Find logarithm of current term.
                  TERMLN=W(IJ)+W(JJ)+W(KJ)
     1            +(A(1)-1.D0)*XIJ(I,IJ)+(A(2)-1.D0)*XIJ(J,JJ)+
     2            (A(3)-1.D0)*XIJ(K,KJ)+DLOG(GP)+
     3            BMA(I)+BMA(J)+BMA(K)
C Find current term.
                  TERM=DEXP(TERMLN)
C Add current term to sum of previous terms.
                  CDF=CDF+TERM
 15   CONTINUE
C Revert to Z from the logarithm of Z.            
      Z=DEXP(Z)
C Divide by triple product of gamma functions in front of integrals.
      CDF=CDF*DEXP(-ALOGAM(A(1))-ALOGAM(A(2))-ALOGAM(A(3)))
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION ALOGAM(X)
C A function subprogramme to calculate the natural logarithm of
C the gamma function using the seven-term Lanczos formula.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION COEF(7)
C Set values of coefficients.
      COEF(1)=1.000000000178D0
      COEF(2)=76.180091729406D0
      COEF(3)=-86.505320327112D0
      COEF(4)=24.014098222230D0
      COEF(5)=-1.231739516140D0
      COEF(6)=0.001208580030D0
      COEF(7)=-0.000005363820D0
C Set value of square root of 2pi.
      RT2PI=2.506628274631D0
C Change argument to that of factorial.
      Z=X-1.D0
C Set initial term of seven-term series.
      SERIES=COEF(1)
C Sum remaining six terms.
      DO 10 I=1,6
        IP1=I+1
        AI=DFLOAT(I)
        DENOM=Z+AI
        SERIES=SERIES+COEF(IP1)/DENOM
 10   CONTINUE
C Multiply sum of series by square root of 2pi.
      SERIES=SERIES*RT2PI
C Set value of gamma for seven-term series.
      GAM=5.D0
C Set value of Z+1/2.
      ZPH=Z+0.5D0
C Set value of Z+1/2+GAM
      ZPHG=ZPH+GAM
C Find natural logarithm of coefficient of series sum.
      HOLD=ZPH*DLOG(ZPHG)-ZPHG
C Find natural logarithm of Gamma(x).
      ALOGAM=HOLD+DLOG(SERIES)
      RETURN
      END  
C
      SUBROUTINE SEGAMI(SEGI,A,X)
C
C Subroutine to calculate the incomplete gamma function by power series.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C Check if X is negative or A is not positive.
      IF(X.LT.0.D0.OR.A.LE.0.D0) GO TO 10
C Find natural logarithm of gamma(x).
      ALG=ALOGAM(A)
C Set maximum number of terms.
      ITM=100
C Test for zero argument.
      IF(X.EQ.0.D0) GO TO 11
C Set value of first term of series sum.
      SUM=1.D0/A
C Set value of first term.
      TERM=SUM
C Set initial value of inverse of coefficient of $X^N$.
      COEF=A
C Sum series.
      DO 12 N=1,ITM
C Update inverse of coefficient of $X^N$.
        COEF=COEF+1.D0
C Update term.
        TERM=TERM*X/COEF
C Store current value of sum.
        SUMOLD=SUM
C Update sum.
        SUM=SUM+TERM
C Check relative size of current term.
        IF(SUM.EQ.SUMOLD) GO TO 13
 12   CONTINUE
      PAUSE 'A too small or X too large'
 13   SEGI=SUM*DEXP(-X+A*DLOG(X)-ALG)
      RETURN
 10   PAUSE 'X is negative or A is not positive.'
      RETURN
 11   SEGI=0.D0
      RETURN
      END
C
      SUBROUTINE CFGAMI(CFGI,A,X)
C
C Subroutine to calculate the incomplete gamma function
C by continued fraction expansion.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C Find natural logarithm of gamma(x).
      ALG=ALOGAM(A)
C Set maximum number of terms.
      ITER=100
C Set initial value of continued fraction.
      CFOLD=0.D0
C Set initial valus of even and odd sequences a and b.
      AE=0.D0
      AO=1.D0
      BE=1.D0
      BO=0.D0
C Set initial values of even and odd sequences a and b.
      AAE=1.D0-A
      AAO=1.D0
      BBE=1.D0
      BBO=X
C Evaluate continued fraction, first by odd sequences, then by even.
      DO 10 N=1,ITER
C Calculate odd a and b.
        AO=BBO*AE+AAO*AO
        BO=BBO*BE+AAO*BO
        CF=AO/BO
C Test for convergence.
        IF(CF.EQ.CFOLD) GO TO 11
C Store old CF.
        CFOLD=CF
C Update odd sequence a.
        IF(N.GT.1)AA0=AA0+1.D0
C Calculate even a and b.
        AE=BBE*AO+AAE*AE
        BE=BBE*BO+AAE*BE
        CF=AE/BE
C Update even sequence a.
        AAE=AAE+1.D0
C Test for convergence.
        IF(CF.EQ.CFOLD) GO TO 11
C Store old CF.
        CFOLD=CF
 10   CONTINUE
      PAUSE 'A too large or X too small.'
 11   CFGI=1.D0-DEXP(-X+A*DLOG(X)-ALG)*CF
      RETURN
      END 
C
      SUBROUTINE GAMI(GI,A,X)
C
C Subroutine for the calculation of the incomplete gamma function.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C Check if X is negative or A is not positive.
      IF(X.LT.0.D0.OR.A.LE.0.D0) GO TO 10
C If X is less than A+1, use series calculation,
C otherwise use continued fraction expansion.
      IF(X.LT.A+1.D0) CALL SEGAMI(GI,A,X)
      IF(X.GE.A+1.D0) CALL CFGAMI(GI,A,X) 
      GO TO 11
 10   PAUSE 'X is negative or A is not positive.'
 11   CONTINUE
      RETURN
      END
