C
C A programme to test the linear equation solver subroutine LINSOL.
C The simple example of the 3x3 system considered in Section 1.5
C of Earth Dynamics is used for illustration.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(4,4),X(4),B(4),C(4,9),IR(4),IC(4),HOLD(4,9),IDET(4),
     1TEMP(4,4)
      OPEN(UNIT=1,FILE='testlinsol.dat',STATUS='UNKNOWN')
      M1=4
      M2=9
      A(1,1)=2.D0
      A(1,2)=-7.D0
      A(1,3)=4.D0
      A(2,1)=-3.D0
      A(2,2)=8.D0
      A(2,3)=5.D0
      A(3,1)=1.D0
      A(3,2)=9.D0
      A(3,3)=-6.D0
C Put coefficient matrix A in temporary storage aray TEMP.
      N=3
      DO 1 I=1,N
      DO 1 J=1,N
 1    TEMP(I,J)=A(I,J)
      B(1)=9.D0
      B(2)=6.D0
      B(3)=1.D0
      N2=2*N+1
      CALL LINSOL(A,B,N,C,DET,M1,M2)
C Write out augmented matrix C.
      WRITE(6,2)
      WRITE(1,2)
 2    FORMAT(21X,'Reduced array (1.538) is;')
      DO 3 II=1,N
      WRITE(6,4)(C(II,JJ),JJ=1,N2)
      WRITE(1,4)(C(II,JJ),JJ=1,N2)
 4    FORMAT(1X,7F8.5)
 3    CONTINUE
C Multiply A and its inverse.
      DO 5 I=1,N
      DO 5 J=1,N
      HOLD(I,J)=0.D0
      DO 5 K=1,N
      HOLD(I,J)=HOLD(I,J)+TEMP(I,K)*A(K,J)
 5    CONTINUE
      WRITE(6,6)
      WRITE(1,6)
 6    FORMAT(20X,'A times its inverse is;')
      DO 7 i=1,N
      WRITE(6,8)(HOLD(I,J),J=1,N)
      WRITE(1,8)(HOLD(I,J),J=1,N)
 7    CONTINUE
 8    FORMAT(1X,3F18.15)
      WRITE(6,9)DET
      WRITE(1,9)DET
 9    FORMAT(10X,'Determinant of A is=',F17.12)
      END
C
      SUBROUTINE LINSOL(A,B,N,C,DET,M1,M2)
C
C LINSOL solves the linear system Ax=b by Gauss-Jordan elimination
C with partial pivoting. A is the N x N coefficient matrix
C and B is the N-length constant vector b. The inverse of A
C is returned in A, the solution vector x is returned in B,
C and the determinant of A is returned in DET. C is the N x 2N+1
C augmented matrix used internally. M1 is the maximum dimension
C for N specified in the main programme and M2=2M1+1 is
C the maximum dimension for 2N+1 used in the main programme.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(M1,M1),B(M1),C(M1,M2),HOLD(M1,M2),
     1  IR(M1),IC(M1),IDET(M1)
C Construct the augmented matrix C.
      N2=2*N+1
      NP1=N+1
      NM1=N-1
C Insert the matrix A.
      DO 10 I=1,N
        DO 10 J=1,N
          C(I,J)=A(I,J)
 10   CONTINUE
C Insert the unknown vector.
      DO 11 J=1,N
        C(J,NP1)=B(J)
 11   CONTINUE
C Augment with the identity matrix.
      DO 12 I=1,N
        DO 12 J=1,N
          JJ=J+NP1
          C(I,JJ)=0.D0
          IF(I.EQ.J)C(I,JJ)=1.D0
 12   CONTINUE
C Begin Gauss-Jordan elimination.
      DET=1.D0
      DO 13 I=1,N
        IM1=I-1
C Search for largest pivot.
        PIVOT=0.D0
        DO 14 J=1,N
          DO 14 K=1,N
C Scan IR and IC vectors for invalid pivot subscripts.
            IF(I.EQ.1)GO TO 15
            DO 16 ISC=1,IM1
              DO 16 JSC=1,IM1
                IF(J.EQ.IR(ISC))GO TO 14
                IF(K.EQ.IC(JSC))GO TO 14
 16         CONTINUE
 15         IF(DABS(C(J,K)).LE.DABS(PIVOT))GO TO 14
            PIVOT=C(J,K)
            IR(I)=J
            IC(I)=K
 14     CONTINUE
        DET=DET*PIVOT
C Label next row.
        IRP1=IR(I)+1
C Label previous row.
        IRM1=IR(I)-1
C Divide row by pivot.
        DO 17 J=1,N2
             C(IR(I),J)=C(IR(I),J)/PIVOT
 17     CONTINUE
C Reduce elements below pivot to zero.
          IF(IRP1.GT.N)GO TO 18
          DO 19 K=IRP1,N
            ELL=C(K,IC(I))
            DO 20 J=1,N2
              C(K,J)=C(K,J)-C(IR(I),J)*ELL
 20         CONTINUE
 19       CONTINUE
 18       CONTINUE
C Reduce elements above pivot to zero.
          IF(IRM1.LT.1)GO TO 21
          DO 22 K=1,IRM1
            ELL=C(K,IC(I))
            DO 23 J=1,N2
              C(K,J)=C(K,J)-C(IR(I),J)*ELL
 23         CONTINUE
 22       CONTINUE
 21       CONTINUE
 13   CONTINUE     
C Unscramble equations and record row interchanges.
      DO 24 I=1,N
        IDET(IR(I))=IC(I)
        DO 24 J=1,N2
          HOLD(IC(I),J)=C(IR(I),J)
 24   CONTINUE
C Put equations back in original matrix.
      DO 25 I=1,N
        DO 25 J=1,N2
          C(I,J)=HOLD(I,J)
 25   CONTINUE
C Determine sign of determinant.
      ICH=0
      DO 26 I=1,NM1
        IP1=I+1
        DO 26 J=IP1,N
          IF(IDET(J).GE.IDET(I))GO TO 26
          IHOLD=IDET(J)
          IDET(J)=IDET(I)
          IDET(I)=IHOLD
          ICH=ICH+1
 26   CONTINUE
      IF((ICH/2)*2.NE.ICH)DET=-DET
C Put inverse in A and solution in B.
      DO 27 I=1,N
        B(I)=C(I,NP1)
        DO 27 J=1,N
          JJ=J+NP1
          A(I,J)=C(I,JJ)
 27   CONTINUE
      RETURN
      END

