C                      PROGRAMME DISPF.FOR
C
C This programme calculates the three components of the displacement
C fields of dip-slip and strike-slip faults and combinations thereof.
C Surface values of the three components of displacement are placed
C in the output file plot.dat, as functions of surface co-ordinates
C along the strike and orthogonal to the strike, in peparation for
C contour plotting.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION U(3),UD(3),US(3),X(3)
      DOUBLE PRECISION L
      OPEN(UNIT=1,FILE='plot.dat',STATUS='UNKNOWN')
C Define pi.
      PI=3.141592653589793D0
C Enter fault parameters.
      WRITE(6,10)
 10   FORMAT(1X,'Enter dip angle in degrees.')
      READ(5,*)DIP
      WRITE(6,11)
 11   FORMAT(1X,'Enter slip in metres.')
      READ(5,*)SLIP
      WRITE(6,12)
 12   FORMAT(1X,'Enter rake angle in degrees.')
      READ(5,*)RAKE
      WRITE(6,13)
 13   FORMAT(1X,'Enter upper edge of fault, down-dip in metres.')
      READ(5,*)DU
      WRITE(6,14)
 14   FORMAT(1X,'Enter lower edge of fault, down-dip in metres.')
      READ(5,*)DL
      WRITE(6,15)
 15   FORMAT(1X,'Enter fault half-length in metres.')
      READ(5,*)L
C Convert dip and rake to radians.
      DIP=DIP*PI/180.D0
      RAKE=RAKE*PI/180.D0
C Find dip-slip and strike-slip components.
      SLIPD=SLIP*DCOS(RAKE)
      SLIPS=SLIP*DSIN(RAKE)
C Generate surface plot file, plot.dat, for ten fault lengths
C by ten fault lengths.
      X(3)=0.D0
      DO 16 I=0,50
          AI=DFLOAT(I)
          X(1)=0.4D0*AI*L
          DO 17 J=0,50
               AJ=DFLOAT(J)
               X(2)=0.4D0*AJ*L    
               CALL DIPS(UD,X,DIP,SLIPD,DU,DL,L)
               CALL STRIKES(US,X,DIP,SLIPS,DU,DL,L)
               DO 18 K=1,3
                    U(K)=UD(K)+US(K)
 18              CONTINUE
              WRITE(1,19)X(1),X(2),U(1),U(2),U(3)
 19           FORMAT(1X,5D14.6)
 17        CONTINUE
 16     CONTINUE
      END
C
      SUBROUTINE DIPS(U,X,DIP,SLIP,DU,DL,L)
C
C This subroutine calculates the displacement vector U(3),
C at the field point X(3), for a dip-slip fault, dipping at angle DIP,
C with slip magnitude SLIP, upper boundary measured down-dip at DU,
C lower boundary down-dip at DL, fault extending along the strike
C from -L to L.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION U(3),X(3)
      DOUBLE PRECISION L
C  Set value of pi.
      PI=3.141592653589793D0
C Find parameters of field point distances normal and parallel to
C the fault and its image.
      R2=X(2)*DSIN(DIP)-X(3)*DCOS(DIP)
      R3=X(2)*DCOS(DIP)+X(3)*DSIN(DIP)
      Q2=X(2)*DSIN(DIP)+X(3)*DCOS(DIP)
      Q3=-X(2)*DCOS(DIP)+X(3)*DSIN(DIP)
C Find U(1).
      CALL DIP1(U1,L,DL,X,DIP,R2,R3,Q2,Q3)
      U(1)=U1
      CALL DIP1(U1,-L,DU,X,DIP,R2,R3,Q2,Q3)
      U(1)=U(1)+U1
      CALL DIP1(U1,-L,DL,X,DIP,R2,R3,Q2,Q3)
      U(1)=U(1)-U1
      CALL DIP1(U1,L,DU,X,DIP,R2,R3,Q2,Q3)
      U(1)=U(1)-U1
C Find U(2).
      CALL DIP2(U2,L,DL,X,DIP,R2,R3,Q2,Q3)
      U(2)=U2
      CALL DIP2(U2,-L,DU,X,DIP,R2,R3,Q2,Q3)
      U(2)=U(2)+U2
      CALL DIP2(U2,-L,DL,X,DIP,R2,R3,Q2,Q3)
      U(2)=U(2)-U2
      CALL DIP2(U2,L,DU,X,DIP,R2,R3,Q2,Q3)
      U(2)=U(2)-U2          
C Find U(3).
      CALL DIP3(U3,L,DL,X,DIP,R2,R3,Q2,Q3)
      U(3)=U3
      CALL DIP3(U3,-L,DU,X,DIP,R2,R3,Q2,Q3)
      U(3)=U(3)+U3
      CALL DIP3(U3,-L,DL,X,DIP,R2,R3,Q2,Q3)
      U(3)=U(3)-U3
      CALL DIP3(U3,L,DU,X,DIP,R2,R3,Q2,Q3)
      U(3)=U(3)-U3
C Scale displacements.
      U(1)=SLIP*U(1)/(12.D0*PI)
      U(2)=SLIP*U(2)/(12.D0*PI)
      U(3)=SLIP*U(3)/(12.D0*PI)
      RETURN
      END
C
      SUBROUTINE DIP1(U1,XI1,XI,X,DIP,R2,R3,Q2,Q3)
C
C This subroutine evaluates the one-component, U1,
C of the double integral for specified limits XI1 and XI.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(3)
C Find distance R, from fault to field point and
C distance Q from image fault tofield point.
      R=(X(1)-XI1)*(X(1)-XI1)+R2*R2+(R3-XI)*(R3-XI)
      R=DSQRT(R)
      Q=(X(1)-XI1)*(X(1)-XI1)+Q2*Q2+(Q3+XI)*(Q3+XI)
      Q=DSQRT(Q)
C Find fault plane co-ordinates XI2 and XI3.
      XI2=XI*DCOS(DIP)
      XI3=XI*DSIN(DIP)
C Evaluate U1.
      U1=(X(2)-XI2)*DSIN(DIP)*(2.D0/R+4.D0/Q-4.D0*XI3*X(3)/Q**3
     1 -3.D0/(Q+X(3)+XI3))-DCOS(DIP)*(3.D0*DLOG(Q+X(3)+XI3)
     2 +2.D0*(X(3)-XI3)/R+4.D0*(X(3)-XI3)/Q
     3 +4.D0*XI3*X(3)*(X(3)+XI3)/Q**3)
     4 +3.D0*(DLOG(Q+X(3)+XI3)-DSIN(DIP)*DLOG(Q+Q3+XI))/DCOS(DIP)
     5 +6.D0*X(3)*(DCOS(DIP)/Q-Q2*DSIN(DIP)/(Q*(Q+Q3+XI)))
      RETURN
      END
C
      SUBROUTINE DIP2(U2,XI1,XI,X,DIP,R2,R3,Q2,Q3)
C
C This subroutine evaluates the two-component, U2,
C of the double integral for specified limits XI1 and XI.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION X(3)
      DOUBLE PRECISION K
C Find distance R from fault to field point and
C distance Q from image fault to field point.
      R=(X(1)-XI1)*(X(1)-XI1)+R2*R2+(R3-XI)*(R3-XI)
      R=DSQRT(R)
      Q=(X(1)-XI1)*(X(1)-XI1)+Q2*Q2+(Q3+XI)*(Q3+XI)
      Q=DSQRT(Q)
C Find projections of Q on plane X(1)=0, and on plane Q3=0.
      H=Q2*Q2+(Q3+XI)*(Q3+XI)
      H=DSQRT(H)
      K=(X(1)-XI1)*(X(1)-XI1)+Q2*Q2
      K=DSQRT(K)
C Find fault plane co-ordinates XI2 and XI3.
      XI2=XI*DCOS(DIP)
      XI3=XI*DSIN(DIP)
C Evaluate U2.
      U2=DSIN(DIP)*(-DLOG(R+X(1)-XI1)+DLOG(Q+X(1)-XI1)
     1 +4.D0*XI3*X(3)/(Q*(Q+X(1)-XI1))+3.D0*(X(1)-XI1)/(Q+X(3)+XI3)
     2 +(X(2)-XI2)**2*(2.D0/(R*(R+X(1)-XI1))+4.D0/(Q*(Q+X(1)-XI1))
     3 -4.D0*XI3*X(3)*(2.D0*Q+X(1)-XI1)/(Q**3*(Q+X(1)-XI1)**2.D0)))
     4 -DCOS(DIP)*((X(2)-XI2)*(2.D0*(X(3)-XI3)/(R*(R+X(1)-XI1))
     5 +4.D0*(X(3)-XI3)/(Q*(Q+X(1)-XI1))+4.D0*XI3*X(3)*
     6 (X(3)+XI3)*(2.D0*Q+X(1)-XI1)/(Q**3*(Q+X(1)-XI1)**2.D0))
     7 +6.D0*DATAN((X(1)-XI1)*(X(2)-XI2)/((H+X(3)+XI3)*(Q+H)))
     8 -3.D0*DATAN((X(1)-XI1)*(R3-XI)/(R2*R))
     9 +6.D0*DATAN((X(1)-XI1)*(Q3+XI)/(Q2*Q)))
      U2=U2+6.D0*DATAN(((K-Q2*DCOS(DIP))*(Q-K)+(Q3+XI)*K*DSIN(DIP))/
     1 ((X(1)-XI1)*(Q3+XI)*DCOS(DIP)))/DCOS(DIP)
     2 +6.D0*X(3)*(((DSIN(DIP)**2-DCOS(DIP)**2)*(Q3+XI)
     3 +2.D0*Q2*DCOS(DIP)*DSIN(DIP))/(Q*(Q+X(1)-XI1))
     4 +(X(1)-XI1)*DSIN(DIP)**2/(Q*(Q+Q3+XI)))
      RETURN
      END     
C
      SUBROUTINE DIP3(U3,XI1,XI,X,DIP,R2,R3,Q2,Q3)
C
C This subroutine evaluates the three-component, U3,
C of the double integral for specified limits XI1 and XI.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION X(3)
C Find distance R from fault to field point and
C distance Q from image fault to fiekd point.
      R=(X(1)-XI1)*(X(1)-XI1)+R2*R2+(R3-XI)*(R3-XI)
      R=DSQRT(R)
      Q=(X(1)-XI1)*(X(1)-XI1)+Q2*Q2+(Q3+XI)*(Q3+XI)
      Q=DSQRT(Q)
C Find projection of Q on plane X(1)=0.
      H=Q2*Q2+(Q3+XI)*(Q3+XI)
      H=DSQRT(H)
C Find fault plane co-ordinates XI2 and XI3.
      XI2=XI*DCOS(DIP)
      XI3=XI*DSIN(DIP)
C Evaluate U3.
      U3=DSIN(DIP)*((X(2)-XI2)*(2.D0*(X(3)-XI3)/(R*(R+X(1)-XI3))
     1 +4.D0*(X(3)-XI3)/(Q*(Q+X(1)-XI1))-4.D0*XI3*X(3)*(X(3)+XI3)*
     2 (2.D0*Q+X(1)-XI1)/(Q**3*(Q+X(1)-XI1)**2))
     3 -6.D0*DATAN((X(1)-XI1)*(X(2)-XI2)/((H+X(3)+XI3)*(Q+H)))
     4 +3.D0*DATAN((X(1)-XI1)*(R3-XI)/(R2*R))
     5 -6.D0*DATAN((X(1)-XI1)*(Q3+XI)/(Q2*Q)))
     6 +DCOS(DIP)*(DLOG(R+X(1)-XI1)-DLOG(Q+X(1)-XI1)
     7 -2.D0*(X(3)-XI3)**2/(R*(R+X(1)-XI1))
     8 -4.D0*((X(3)+XI3)**2-XI3*X(3))/(Q*(Q+X(1)-XI1))-4.D0*XI3*X(3)*
     9 (X(3)+XI3)**2*(2.D0*Q+X(1)-XI1)/(Q**3*(Q+X(1)-XI1)**2.D0))
      U3=U3+6.D0*X(3)*(DCOS(DIP)*DSIN(DIP)*
     1 (2.D0*(Q3+XI)/(Q*(Q+X(1)-XI1))+(X(1)-XI1)/(Q*(Q+Q3+XI)))
     2 -Q2*(DSIN(DIP)**2-DCOS(DIP)**2)/(Q*(Q+X(1)-XI1)))
      RETURN
      END
C
      SUBROUTINE STRIKES(U,X,DIP,SLIP,DU,DL,L)
C
C This subroutine calculates the displacement vector U(3),
C at the field point X(3), for a strike-slip fault, dipping at angle DIP,
C with slip magnitude SLIP, upper boundary measured down-dip at DU,
C lower boundary down-dip at DL, fault extending along the strike
C from -L to L.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION U(3),X(3)
      DOUBLE PRECISION L
C  Set value of pi.
      PI=3.141592653589793D0
C Find parameters of field point distances normal and parallel to
C the fault and its image.
      R2=X(2)*DSIN(DIP)-X(3)*DCOS(DIP)
      R3=X(2)*DCOS(DIP)+X(3)*DSIN(DIP)
      Q2=X(2)*DSIN(DIP)+X(3)*DCOS(DIP)
      Q3=-X(2)*DCOS(DIP)+X(3)*DSIN(DIP)
C Find U(1).
      CALL STRIKE1(U1,L,DL,X,DIP,R2,R3,Q2,Q3)
      U(1)=U1
      CALL STRIKE1(U1,-L,DU,X,DIP,R2,R3,Q2,Q3)
      U(1)=U(1)+U1
      CALL STRIKE1(U1,-L,DL,X,DIP,R2,R3,Q2,Q3)
      U(1)=U(1)-U1
      CALL STRIKE1(U1,L,DU,X,DIP,R2,R3,Q2,Q3)
      U(1)=U(1)-U1
C Find U(2).
      CALL STRIKE2(U2,L,DL,X,DIP,R2,R3,Q2,Q3)
      U(2)=U2
      CALL STRIKE2(U2,-L,DU,X,DIP,R2,R3,Q2,Q3)
      U(2)=U(2)+U2
      CALL STRIKE2(U2,-L,DL,X,DIP,R2,R3,Q2,Q3)
      U(2)=U(2)-U2
      CALL STRIKE2(U2,L,DU,X,DIP,R2,R3,Q2,Q3)
      U(2)=U(2)-U2          
C Find U(3).
      CALL STRIKE3(U3,L,DL,X,DIP,R2,R3,Q2,Q3)
      U(3)=U3
      CALL STRIKE3(U3,-L,DU,X,DIP,R2,R3,Q2,Q3)
      U(3)=U(3)+U3
      CALL STRIKE3(U3,-L,DL,X,DIP,R2,R3,Q2,Q3)
      U(3)=U(3)-U3
      CALL STRIKE3(U3,L,DU,X,DIP,R2,R3,Q2,Q3)
      U(3)=U(3)-U3
C Scale displacements.
      U(1)=SLIP*U(1)/(12.D0*PI)
      U(2)=SLIP*U(2)/(12.D0*PI)
      U(3)=SLIP*U(3)/(12.D0*PI)
      RETURN
      END
C
      SUBROUTINE STRIKE1(U1,XI1,XI,X,DIP,R2,R3,Q2,Q3)
C
C This subroutine evaluates the one-component, U1, 
C of the double integral for specified limits XI1 and XI.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(3)
      DOUBLE PRECISION K
C Find distance R from fault to field point and
C distance Q from image fault to field point.
      R=(X(1)-XI1)*(X(1)-XI1)+R2*R2+(R3-XI)*(R3-XI)
      R=DSQRT(R)
      Q=(X(1)-XI1)*(X(1)-XI1)+Q2*Q2+(Q3+XI)*(Q3+XI)
      Q=DSQRT(Q)
C Find projection of Q on plane Q3=0.
      K=(X(1)-XI1)*(X(1)-XI1)+Q2*Q2
      K=DSQRT(K)
C Find fault plane co-ordinates XI2 and XI3.
      XI2=XI*DCOS(DIP)
      XI3=XI*DSIN(DIP)
C Evaluate U1.
      U1=(X(1)-XI1)*(2.D0*R2/(R*(R+R3-XI))
     1 -(4.D0*Q2-2.D0*X(3)*DCOS(DIP))/(Q*(Q+Q3+XI))
     2 -3.D0*DTAN(DIP)/(Q+X(3)+XI3)+4.D0*Q2*X(3)*DSIN(DIP)/Q**3
     3 -4.D0*Q2*Q3*X(3)*DSIN(DIP)*(2.D0*Q+Q3+XI)/(Q**3*(Q+Q3+XI)**2))
     4 -6.D0*DTAN(DIP)**2*DATAN(((K-Q2*DCOS(DIP))*(Q-K)
     5 +(Q3+XI)*K*DSIN(DIP))/((X(1)-XI1)*(Q3+XI)*DCOS(DIP)))
     6 +3.D0* DATAN((X(1)-XI1)*(R3-XI)/(R2*R))
     7 -3.D0*DATAN((X(1)-XI1)*(Q3+XI) /(Q2*Q))
      RETURN
      END
C
      SUBROUTINE STRIKE2(U2,XI1,XI,X,DIP,R2,R3,Q2,Q3)
C
C This subroutine evaluates the two-component, U2,
C of the double integral for specified limits XI1 and XI.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION X(3)
C Find fault to field point distance R, and image fault to field point
C distance Q.
      R=(X(1)-XI1)*(X(1)-XI1)+R2*R2+(R3-XI)*(R3-XI)
      R=DSQRT(R)
      Q=(X(1)-XI1)*(X(1)-XI1)+Q2*Q2+(Q3+XI)*(Q3+XI)
      Q=DSQRT(Q)
C Find fault plane co-ordinates XI2 and XI3.
      XI2=XI*DCOS(DIP)
      XI3=XI*DSIN(DIP)
C Evaluate U2.
      U2=DSIN(DIP)*(3.D0*DTAN(DIP)*DLOG(Q+X(3)+XI3)/(DCOS(DIP))
     1 -DLOG(R+R3-XI)-(1.D0+3.D0*DTAN(DIP)**2)*DLOG(Q+Q3+XI))
     2 +2.D0*R2*R2*DSIN(DIP)/(R*(R+R3-XI))+2.D0*R2*DCOS(DIP)/R
     3 -2.D0*DSIN(DIP)*(2.D0*X(3)*(Q2*DCOS(DIP)-Q3*DSIN(DIP))
     4 +Q2*(Q2+X(2)*DSIN(DIP)))/(Q*(Q+Q3+XI))
     5 -3.D0*DTAN(DIP)*(X(2)-XI3)/(Q+X(3)+XI3)
     6 +2.D0*(Q2*DCOS(DIP)-Q3*DSIN(DIP)-X(3)*DSIN(DIP)**2)/Q
     7 +4.D0*Q2*X(3)*DSIN(DIP)*(X(2)-XI2+Q3*DCOS(DIP))/Q**3
     8 -4.D0*Q2*Q2*Q3*X(3)*DSIN(DIP)**2*(2.D0*Q+Q3+XI)/
     9 (Q**3*(Q+Q3+XI)**2)
      RETURN
      END
C
      SUBROUTINE STRIKE3(U3,XI1,XI,X,DIP,R2,R3,Q2,Q3)
C
C This subroutine evaluates the three-component, U3, 
C of the double integral for specified limits XI1 and XI.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION X(3)
C Find fault to field point distance R, and image fault to field point
C distance Q.
      R=(X(1)-XI1)*(X(1)-XI1)+R2*R2+(R3-XI)*(R3-XI)
      R=DSQRT(R)
      Q=(X(1)-XI1)*(X(1)-XI1)+Q2*Q2+(Q3+XI)*(Q3+XI)
      Q=DSQRT(Q)
C Find fault plane co-ordinates XI2 and XI3.
      XI2=XI*DCOS(DIP)
      XI3=XI*DSIN(DIP)
C Evaluate U3.
      U3=DCOS(DIP)*(DLOG(R+R3-XI)
     1+(1.D0+3.D0*DTAN(DIP)**2)*DLOG(Q+Q3+XI)
     2 -3.D0*DTAN(DIP)*DLOG(Q+X(3)+XI3)/DCOS(DIP))
     3 +2.D0*R2*DSIN(DIP)/R+2.D0*DSIN(DIP)*(Q2+X(2)*DSIN(DIP))/Q
     4 -2.D0*R2*R2*DCOS(DIP)/(R*(R+R3-XI))
     5 +(4.D0*Q2*X(3)*DSIN(DIP)**2-2.D0*(Q2+X(2)*DSIN(DIP))*
     6 (X(3)+Q3*DSIN(DIP)))/(Q*(Q+Q3+XI))
     7 +4.D0*Q2*X(3)*DSIN(DIP)*(X(3)+XI3-Q3*DSIN(DIP))/Q**3
     8 -4.D0*Q2*Q2*Q3*X(3)*DCOS(DIP)*DSIN(DIP)*(2.D0*Q+Q3+XI)/
     9 (Q**3.D0*(Q+Q3+XI)**2)
      RETURN
      END
    
