C                  PROGRAMME ICFS.FOR
C
C ICFS.FOR calculates the three fundamental solutions of the sixth
C order spheroidal system, regular at the geocentre, for n>0.
C For n=0, it calculates the one solution of the fourth order
C spheroidal system, regular at the geocentre, other than
C the trivial solution y5 constant. Interpolation of Earth model
C properties is made by calls to the subroutines SPMAT and INTPL to
C perform cubic spline interpolation. Power series expansions at
C the geocentre are continued by fourth order Runge-Kutta integration
C by the subroutine RK4 with calls to the subroutine YPRIME to
C evaluate derivatives at each step.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION R(100),RHO(100),GZERO(100),RI(300),RHOI(300),
     1 GZEROI(300),ENAME(10),NM(4),NI(4),B(98,198),C(100,100),
     2 Y(6,6),YSCAL(6,3),YSCAL2(6,3),YA(6),A(6,6),AS(6,6),CAUG(6,13),
     3 RM1(6,3),RM2(6,3),YP(6,6),YIC(6,3),Y1(6,6),Y2(6,6),DELTA(6,3),
     4 FS(6,3)
      DOUBLE PRECISION MU(100),LAMBDA(100),MUI(30),LAMBDAI(300),
     1 K1(6,6)
      CHARACTER*20 EMODEL
C Enter Earth model name.
      WRITE(6,10)
 10   FORMAT(1X,'Type in Earth model file name.')
      READ(5,11)EMODEL
 11   FORMAT(A20)
C Open Earth model file.
      OPEN(UNIT=1,FILE=EMODEL,STATUS='OLD')
C Open fundamental solutions files.
      OPEN(UNIT=2,FILE='fsolns.dat',STATUS='UNKNOWN')
      OPEN(UNIT=3,FILE='fs1.dat',STATUS='UNKNOWN')
      OPEN(UNIT=4,FILE='fs2.dat',STATUS='UNKNOWN')
      OPEN(UNIT=7,FILE='fs3.dat',STATUS='UNKNOWN')
C Set angular frequency of Earth's rotation (WGS 84).
      WE=7.292115D-5
C Calculate square of angular frequency of Earth's rotation.
      WES=WE*WE
C Set value of pi.
      PI=3.141592653589793D0
C Set value of universal constant of gravitation (CODATA 2006).
      G=6.67428D-11
C Enter period, azimuthal number, inertial and Coriolis switches.
      WRITE(6,12)
 12   FORMAT(1X,'Enter period in hours, azimuthal number and',1X,
     1 'inertial and Coriolis switches'/'(1 for in, 0 for out).')
      READ(5,*)TPER,MAZ,INERT,ICOR
C Convert period to seconds.
      TPER=3600.D0*TPER
C Calculate dimensionless angular frequency.
      SIG=PI/(TPER*WE)
C Calculate square of dimensionless angular frequency.
      SIGS=SIG*SIG
C Compute square of angular frequency.
      ANGS=DFLOAT(INERT)
      ANGS=2.D0*PI*ANGS/TPER
      ANGS=ANGS*ANGS
C Compute Coriolis term.
      COR=2.D0*DFLOAT(MAZ)*DFLOAT(ICOR)
      COR=2.D0*PI*COR*WE/TPER
C Set maximum relative error tolerance for integrations.
      EPS=1.D-5
C Set minimum starting radius for variable Earth properties.
      RMIN=1200.D0
C Set maximum dimensions for interpolation.
      M1=100
      M2=198
      M3=98
C Read in and write out inner core values for Earth model.
C Read in and write out headers.
C Read in Earth model name.
      READ(1,13)(ENAME(I),I=1,10)
 13   FORMAT(10A7)
      WRITE(2,14)(ENAME(I),I=1,10)
      WRITE(6,14)(ENAME(I),I=1,10)
 14   FORMAT(5X,10A7)
C Read additional headers.
      READ(1,15)NN
 15   FORMAT(I10)
C Read in number of model points in each region NM(I)
C and number of integration steps NI(I).
      READ(1,16)(NM(I),NI(I),I=1,4)
 16   FORMAT(8I10)
C Write out Earth properties.
      WRITE(6,17)
 17   FORMAT(/,/,5X,'Radius',6X,'Rho',4X,'Lambda',7X,'Mu',6X,'Gzero')
      WRITE(6,18)
 18   FORMAT(6X,'(km)',5X,'(gm/cc)',2X,'(kbars)',4X,'(kbars)',2X,
     1 '(cm/sec/sec)',/)
C Read in inner core model.
      N1=NM(1)
      READ(1,19)(R(I),RHO(I),LAMBDA(I),MU(I),GZERO(I),I=1,N1)
 19   FORMAT(1X,F10.1,F10.2,F10.1,F10.1,F10.1)
C Scale inner core model to SI values and store.
      DO 20 I=1,N1
        RI(I)=R(I)*1.D3
        RHOI(I)=RHO(I)*1.D3
        LAMBDAI(I)=LAMBDA(I)*1.D8
        MUI(I)=MU(I)*1.D8
        GZEROI(I)=GZERO(I)*1.D-2
C Write out inner core model to screen.
        WRITE(6,19)R(I),RHO(I),LAMBDA(I),MU(I),GZERO(I)
 20   CONTINUE
C Store radius of the inner core boundary.
      RIOB=RI(N1)
C Set up interpolation for the inner core.
      N1=NM(1)
      N2=2*N1-2
      N3=N1-2
C Put inner core values in active locations.
      DO 21 I=1,N1
        J=I
        R(I)=RI(J)
        RHO(I)=RHOI(J)
        MU(I)=MUI(J)
        LAMBDA(I)=LAMBDAI(J)
        GZERO(I)=GZEROI(J)
 21   CONTINUE
C Calculate gravity gradient at geocentre.
      GAMMA=(4.D0*PI*G*RHO(1)-2.D0*WES)/3.D0
C Set minimum radius for variable Earth properties.
      R(1)=RMIN
C Set gravity at minimum radius.
      GZERO(1)=GAMMA*RMIN
C Construct interpolation matrix.
      CALL SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C Enter degree of fundamental solutions to be computed.
      WRITE(6,22)
 22   FORMAT(1X,'Enter degree N of fundamental solutions to',1X,
     1 'be computed')
      READ(5,*)N
      AN=DFLOAT(N)
C Write degree of fundamental solutions.
      WRITE(2,23)N
 23   FORMAT(/,5X,'Degree of computed fundamental solutions =',I3)
C Write period of fundamental solutions.
      WRITE(2,24)TPER
 24   FORMAT(/,5X,'Period (sec.) of computed fundamental solutions',1X,
     1 '=',F9.1)
C Write azimuthal number of fundamental solutions.
      WRITE(2,25)MAZ
 25   FORMAT(/,5X,'Azimuthal number of computed fundamental solutions',
     1 1X,'=',I3)
C Test if N=0.
      IF(N.EQ.0) GO TO 26
C Begin power series expansions for fundamental solutions for N>0.
C Set initial values of fundamental solutions.
C Set initial values of first fundamental solution.
      Y(1,1)=1.D0
      Y(2,1)=2.D0*(AN-1.D0)*MU(1)
      Y(3,1)=1.D0/AN
      Y(4,1)=Y(2,1)/AN
      Y(5,1)=4.D0*PI*G*RHO(1)/AN
      Y(6,1)=0.D0
C Set initial values of second fundamental solution.
      Y(1,2)=0.D0
      Y(2,2)=0.D0
      Y(3,2)=0.D0
      Y(4,2)=0.D0
      Y(5,2)=1.D0/AN
      Y(6,2)=1.D0
C Set initial values of third fundamental solution.
      Y(3,3)=P2(AN,LAMBDA(1),MU(1))/P1(AN,LAMBDA(1),MU(1))
      Y(2,3)=-Q1(AN,LAMBDA(1),MU(1))*Y(3,3)+Q2(AN,LAMBDA(1),MU(1))
      Y(1,3)=-AN*Y(3,3)+1.D0/MU(1)
      Y(4,3)=1.D0
      Y(5,3)=2.D0*PI*G*RHO(1)*((AN+3.D0)*Y(1,3)-AN*(AN+1.D0)*Y(3,3))/
     1 (2.D0*AN+3.D0)
      Y(6,3)=(AN+2.D0)*Y(5,3)-4.D0*PI*G*RHO(1)*Y(1,3)
C Calculate coefficients of second terms in series expansions.
C Calculate coefficients of second terms
C for first fundamental solution.
      YSCAL(3,1)=RHO(1)*((3.D0-AN)*GAMMA+ANGS+2.D0*WES-COR/AN)/
     1 P1(AN,LAMBDA(1),MU(1))
      YSCAL(2,1)=-Q1(AN,LAMBDA(1),MU(1))*YSCAL(3,1)
      YSCAL(1,1)=-AN*YSCAL(3,1)
      YSCAL(4,1)=0.D0
      YSCAL(5,1)=2.D0*PI*G*RHO(1)*((AN+3.D0)*YSCAL(1,1)
     1 -AN*(AN+1.D0)*YSCAL(3,1))/(2.D0*AN+3.D0)
      YSCAL(6,1)=(AN+2.D0)*YSCAL(5,1)-4.D0*PI*G*RHO(1)*YSCAL(1,1)
C Calculate coefficients of second terms
C for second fundamental solution.
      YSCAL(3,2)=RHO(1)/P1(AN,LAMBDA(1),MU(1))
      YSCAL(2,2)=-Q1(AN,LAMBDA(1),MU(1))*YSCAL(3,2)
      YSCAL(1,2)=-AN*YSCAL(3,2)
      YSCAL(4,2)=0.D0
      YSCAL(5,2)=2.D0*PI*G*RHO(1)*((AN+3.D0)*YSCAL(1,2)
     1 -AN*(AN+1.D0)*YSCAL(3,2))/(2.D0*AN+3.D0)
      YSCAL(6,2)=(AN+2.D0)*YSCAL(5,2)-4.D0*PI*G*RHO(1)*YSCAL(1,2)
C Generate matrix for evaluation of second terms in power series
C expansion of third fundamental solution.
      CALL MATRIX(A,RHO,LAMBDA,MU,PI,G,AN,WE)
C Set right-hand side for calculation of second terms
C in third solution.
      YA(1)=0.D0
      YA(2)=RHO(1)*(-(4.D0*GAMMA+ANGS+2.D0*WES)*Y(1,3)
     1 +(AN*(AN+1.D0)*GAMMA+COR)*Y(3,3)-Y(6,3))
      YA(3)=0.D0
      YA(4)=RHO(1)*((GAMMA+COR/(AN*(AN+1.D0)))*Y(1,3)
     1 -(ANGS-COR/(AN*(AN+1.D0)))*Y(3,3)-Y(5,3))
      YA(5)=0.D0
      YA(6)=0.D0
C Store matrix A in AS.
      DO 27 I=1,6
        DO 28 J=1,6
          AS(I,J)=A(I,J)
 28     CONTINUE
 27   CONTINUE  
C Solve for second terms in third solution.
      CALL LINSOL(A,YA,6,CAUG,DET,6,13)
      YSCAL(1,3)=YA(1)
      YSCAL(2,3)=YA(2)
      YSCAL(3,3)=YA(3)
      YSCAL(4,3)=YA(4)
      YSCAL(5,3)=YA(5)
      YSCAL(6,3)=YA(6)
C Set right-hand side for calculation of third terms
C in first solution.
      YA(1)=0.D0
      YA(2)=RHO(1)*(-(4.D0*GAMMA+ANGS+2.D0*WES)*YSCAL(1,1)
     1 +(AN*(AN+1.D0)*GAMMA+COR)*YSCAL(3,1)-YSCAL(6,1))
      YA(3)=0.D0
      YA(4)=RHO(1)*((GAMMA+COR/(AN*(AN+1.D0)))*YSCAL(1,1)
     1 -(ANGS-COR/(AN*(AN+1.D0)))*YSCAL(3,1)-YSCAL(5,1))
      YA(5)=0.D0
      YA(6)=0.D0
C Copy stored matrix AS into matrix A.
      DO 29 I=1,6
        DO 30 J=1,6
          A(I,J)=AS(I,J)
 30     CONTINUE
 29   CONTINUE
C Solve for third terms in first solution.
      CALL LINSOL(A,YA,6,CAUG,DET,6,13)
      YSCAL2(1,1)=YA(1)
      YSCAL2(2,1)=YA(2)
      YSCAL2(3,1)=YA(3)
      YSCAL2(4,1)=YA(4)
      YSCAL2(5,1)=YA(5)
      YSCAL2(6,1)=YA(6)
C Set right-hand side for calculation of third terms
C in second solution.
      YA(1)=0.D0
      YA(2)=RHO(1)*(-(4.D0*GAMMA+ANGS+2.D0*WES)*YSCAL(1,2)
     1 +(AN*(AN+1.D0)*GAMMA+COR)*YSCAL(3,2)-YSCAL(6,2))
      YA(3)=0.D0
      YA(4)=RHO(1)*((GAMMA+COR/(AN*(AN+1.D0)))*YSCAL(1,2)
     1 -(ANGS-COR/(AN*(AN+1.D0)))*YSCAL(3,2)-YSCAL(5,2))
      YA(5)=0.D0
      YA(6)=0.D0
C Copy stored matrix AS into matrix A.
      DO 31 I=1,6
        DO 32 J=1,6
          A(I,J)=AS(I,J)
 32     CONTINUE
 31   CONTINUE
C Solve for third terms in second solution.
      CALL LINSOL(A,YA,6,CAUG,DET,6,13)
      YSCAL2(1,2)=YA(1)
      YSCAL2(2,2)=YA(2)
      YSCAL2(3,2)=YA(3)
      YSCAL2(4,2)=YA(4)
      YSCAL2(5,2)=YA(5)
      YSCAL2(6,2)=YA(6)
C Set right-hand side for calculation of third terms
C in third solution.
      YA(1)=0.D0
      YA(2)=RHO(1)*(-(4.D0*GAMMA+ANGS+2.D0*WES)*YSCAL(1,3)
     1 +(AN*(AN+1.D0)*GAMMA+COR)*YSCAL(3,3)-YSCAL(6,3))
      YA(3)=0.D0
      YA(4)=RHO(1)*((GAMMA+COR/(AN*(AN+1.D0)))*YSCAL(1,3)
     1 -(ANGS-COR/(AN*(AN+1.D0)))*YSCAL(3,3)-YSCAL(5,3))
      YA(5)=0.D0
      YA(6)=0.D0
C Copy stored matrix AS into matrix A and add 2.0 to diagonal of A.
      DO 33 I=1,6
        DO 34 J=1,6
          A(I,J)=AS(I,J)
          IF(I.EQ.J)A(I,J)=A(I,J)+2.D0
 34     CONTINUE
 33   CONTINUE
C Solve for third terms in third solution.
      CALL LINSOL(A,YA,6,CAUG,DET,6,13)
      YSCAL2(1,3)=YA(1)
      YSCAL2(2,3)=YA(2)
      YSCAL2(3,3)=YA(3)
      YSCAL2(4,3)=YA(4)
      YSCAL2(5,3)=YA(5)
      YSCAL2(6,3)=YA(6)
      GO TO 35
C Begin power series expansion of fundamental solution
C for degree N=0.
 26   CONTINUE
C Set first two solutions to zero.
      DO 36 I=1,6
        DO 37 J=1,2
          Y(I,J)=0.D0
          YSCAL(I,J)=0.D0
          YSCAL2(I,J)=0.D0
 37     CONTINUE  
 36   CONTINUE
C Set values for first three terms of fundamental solution.
      Y(1,3)=1.D0
      Y(2,3)=3.D0*LAMBDA(1)+2.D0*MU(1)
      Y(3,3)=0.D0
      Y(4,3)=0.D0
      Y(5,3)=2.D0*PI*G*RHO(1)
      Y(6,3)=0.D0
      YSCAL(1,3)=-RHO(1)*(4.D0*GAMMA+ANGS+2.D0*WES)/
     1 (10.D0*(LAMBDA(1)+2.D0*MU(1)))
      YSCAL(2,3)=(5.D0*LAMBDA(1)+6.D0*MU(1))*YSCAL(1,3)
      YSCAL(3,3)=0.D0
      YSCAL(4,3)=0.D0
      YSCAL(5,3)=PI*G*RHO(1)*YSCAL(1,3)
      YSCAL(6,3)=0.D0
      YSCAL2(1,3)=-RHO(1)*(4.D0*GAMMA+ANGS+2.D0*WES)*YSCAL(1,3)/
     1 (4.D0*(7.D0*LAMBDA(1)+14.D0*MU(1)))
      YSCAL2(2,3)=(7.D0*LAMBDA(1)+10.D0*MU(1))*YSCAL2(1,3)
      YSCAL2(3,3)=0.D0
      YSCAL2(4,3)=0.D0
      YSCAL2(5,3)=2.D0*PI*G*RHO(1)*YSCAL2(1,3)/3.D0
      YSCAL2(6,3)=0.D0
 35   CONTINUE
C Calculate terms in power series expansions at minimum radius RMIN.
      XV=RMIN
      XVS=XV*XV
      XV3=VX*XVS
      XV4=XVS*XVS
      DO 38 I=1,6
        DO 39 J=1,3
          RM1(I,J)=XVS*YSCAL(I,J)
          RM2(I,J)=XV4*YSCAL2(I,J)
 39     CONTINUE
 38   CONTINUE  
C Write out terms in power series expansions.
      DO 40 J=1,3
        WRITE(2,41)J
 41     FORMAT(/,5X,'Fundamental solution number ',I3)
        WRITE(2,42)
 42     FORMAT(/,6X,'first term',10X,'second term',9X,'third term')
        WRITE(2,43)(Y(I,J),RM1(I,J),RM2(I,J),I=1,6)
 43     FORMAT(5X,D15.8,5X,D15.8,5X,D15.8)
 40   CONTINUE
C Set last three columns of arrays Y(6,6) and YP(6,6) to zero
C for inner core.
      DO 44 I=1,6
        DO 45 J=4,6
          Y(I,J)=0.D0
          YP(I,J)=0.D0
 45     CONTINUE
 44   CONTINUE
C Put inner core solutions in array YIC(6,3)
C and free solutions array FS(6,3).
      DO 46 I=1,6
        DO 47 J=1,3
          YIC(I,J)=Y(I,J)
          FS(I,J)=Y(I,J)
 47     CONTINUE
 46   CONTINUE
C Write out starting values of free solutions.
      RAD=XV*1.D-3
      WRITE(3,48)RAD,(FS(I,1),I=1,6)
      WRITE(4,48)RAD,(FS(I,2),I=1,6)
      WRITE(7,48)RAD,(FS(I,3),I=1,6)
 48   FORMAT(F7.2,6D14.6)
C Begin Runge-Kutta integration for the inner core.
C Find maximum ratio of second coefficient to first term
C in power series expansions.
      CALL REL(ERRMAX,YSCAL,YIC,N)
C Set initial stepsize so that fourth order method would have relative error
C bound EPS.
      H=(EPS/ERRMAX)**(0.2D0)
C Find solutions and derivatives at minimum radius RMIN
C by power series expansions.
      DO 49 I=1,6
        DO 50 J=1,3
C Find solutions.
          Y(I,J)=Y(I,J)+XVS*YSCAL(I,J)+XV4*YSCAL2(I,J)
C Find derivatives.
          YP(I,J)=2.D0*XV*YSCAL(I,J)+4.D0*XV3*YSCAL2(I,J)
 50     CONTINUE
 49   CONTINUE
C Set inner core index.
      IR=1
 51   CONTINUE
C Calculate derivatives at beginning of current step.
      CALL YPRIME(XV,Y,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,
     1 IR,RMIN,ANGS,COR,WES)
C Initialize function values at beginning of integration step.
      DO 52 I=1,6
        DO 53 J=1,6
          Y1(I,J)=Y(I,J)
          Y2(I,J)=Y(I,J)
 53     CONTINUE
 52   CONTINUE
C Integrate solution forward one full step.
      CALL RK4(XV,Y1,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,H,IR,
     1 RMIN,ANGS,COR,WES)
C Integrate solution forward two half steps.
C Halve step.
      HH=0.5D0*H
C Reset radius.
      XV=XV-H
      CALL RK4(XV,Y2,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,HH,IR,
     1 RMIN,ANGS,COR,WES)
      CALL YPRIME(XV,Y2,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,IR,
     1 RMIN,ANGS,COR,WES)
      CALL RK4(XV,Y2,A,K1,N1,C,R,RHO,MU,LAMBDA,GZERO,N,PI,G,HH,IR,
     1 RMIN,ANGS,COR,WES)
C Find error matrix and maximum relative error.
      DO 54 I=1,6
        DO 55 J=1,3
          DELTA(I,J)=Y1(I,J)-Y2(I,J)
 55     CONTINUE
 54   CONTINUE
C Copy Y2(6,6) into inner core solution array YIC(6,3).
      DO 56 I=1,6
        DO 57 J=1,3
          YIC(I,J)=Y2(I,J)
 57     CONTINUE
 56   CONTINUE
C Find maximum relative error.
      CALL REL(ERRMAX,DELTA,YIC,N)
C Scale maximum relative error by specified relative error tolerance.
      ERRMAX=ERRMAX/EPS
C Test if step was successful.
      IF(ERRMAX.LT.1.D0) GO TO 58
C Step too large, reduce stepsize and repeat.
      XV=XV-H
      H=0.9D0*H*(ERRMAX**(-0.25D0))
      GO TO 51
 58   CONTINUE
C Step successful, accept solution and improve truncation error.
      DO 59 I=1,6
        DO 60 J=1,3
          Y(I,J)=Y2(I,J)+DELTA(I,J)/15.D0
          FS(I,J)=Y(I,J)
 60     CONTINUE
 59   CONTINUE
      RAD=XV*1.D-3
      WRITE(3,48)RAD,(FS(I,1),I=1,6)
      WRITE(4,48)RAD,(FS(I,2),I=1,6)
      WRITE(7,48)RAD,(FS(I,3),I=1,6)
C Increase stepsize.
      H=0.9D0*H*(ERRMAX**(-0.20D0))
C Test if solution is finished.
      XVT=XV+H
      IF(XVT.LT.R(N1)) GO TO 51
C Find remaining distance to the ICB.
      H=R(N1)-XV
C Compute fraction of radius.
      RATIO=H/R(N1)
C Continue integration only if remaining distance is more than
C 1.D-9 of radius.
      IF(RATIO.GT.1.D-9) GO TO 51
C Convert solutions back to y-variables.
C Set value of alpha for first two solutions.
      ANM2=AN-2.D0
C Convert three solutions.
      DO 61 J=1,3
C Set value of alpha for third solution.
        IF(J.EQ.3) ANM2=AN
C Find constant equal to the radius of the ICB to the power of alpha.
        CONST=RIOB**ANM2
        Y(2,J)=CONST*Y(2,J)
        Y(4,J)=CONST*Y(4,J)
        CONST=CONST*RIOB
        Y(1,J)=CONST*Y(1,J)
        Y(3,J)=CONST*Y(3,J)
        Y(6,J)=CONST*Y(6,J)
        CONST=CONST*RIOB
        Y(5,J)=CONST*Y(5,J)
 61   CONTINUE
      END
C  
      SUBROUTINE SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C
C SPMAT computes the N1 by N1 matrix C connecting
C second derivatives to function values at N1 nodal points R(I).
C B is the N3=N1-2 by N2=2*N1-2 augmented matrix used internally
C in the Gaussian elimination procedure.
C M1, M2, M3 are the respective maximum dimensions for N1, N2, N3
C specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),B(M3,M2)
      N=N3+1
C Construct augmented matrix A.
      DO 11 I=1,N3
        DO 10 J=1,N2
          B(I,J)=0.D0
  10    CONTINUE
        B(I,I)=2.D0*(R(I+2)-R(I))
        B(I,I+N3)=1.D0
        IF(I.EQ.1.OR.I.EQ.N3) GO TO 11
        B(I,I-1)=R(I+1)-R(I)
        B(I,I+1)=R(I+2)-R(I+1)
  11  CONTINUE
C Apply conditions that third derivatives are constant
C in first two and last two intervals.
      B(1,2)=R(3)-R(2)-((R(2)-R(1))**2.D0)/(R(3)-R(2))
      B(N3,N3-1)=R(N3+1)-R(N3)-((R(N1)-R(N1-1))**2.D0)/(R(N1-1)-R(N1-2))
      B(1,1)=B(1,1)+((R(3)-R(1))*(R(2)-R(1)))/(R(3)-R(2))
      B(N3,N3)=B(N3,N3)
     1        +((R(N1)-R(N1-1))*(R(N1)-R(N1-2)))/(R(N1-1)-R(N1-2))
      NM1=N3-1
C Eliminate sub-diagonal.
      DO 13 I=1,NM1
        DIAG=B(I,I)
        ELL=B(I+1,I)
        DO 12 J=1,N2
          B(I,J)=B(I,J)/DIAG
          B(I+1,J)=B(I+1,J)-B(I,J)*ELL
  12    CONTINUE
  13  CONTINUE
C Eliminate super-diagonal.
      DO 15 I=1,NM1
        NO=N-I
        DIAG=B(NO,NO)
        ELL=B(NO-1,NO)
        DO 14 J=1,N2
          B(NO,J)=B(NO,J)/DIAG
          B(NO-1,J)=B(NO-1,J)-B(NO,J)*ELL
  14    CONTINUE
  15  CONTINUE
C Move inverse to position formerly occupied by A.
      DO 17 I=1,N3
        DO 16 J=1,N3
          B(I,J)=B(I,J+N3)
  16    CONTINUE
  17  CONTINUE
C Construct matrix B in N-2 by N augmentation space. 
      DO 19 I=1,N3
        DO 18 J=1,N1
          B(I,J+N3)=0.0D0
  18    CONTINUE
        IN=I+N3
        FACTOR=6.D0/((R(I+2)-R(I+1))*(R(I+1)-R(I)))
        B(I,IN)=FACTOR*(R(I+2)-R(I+1))
        B(I,IN+1)=-FACTOR*(R(I+2)-R(I))
        B(I,IN+2)=FACTOR*(R(I+1)-R(I))
  19  CONTINUE
C Multiply inverse of A into B to get N-2 by N version of C.
      DO 22 I=1,N3
        DO 21 J=1,N1
          SUM=0.D0
          DO 20 K=1,N3
            SUM=SUM+B(I,K)*B(K,J+N3)
  20      CONTINUE
          C(I+1,J)=SUM
  21    CONTINUE
  22  CONTINUE
C Add first and last rows to C.
      DO 23 J=1,N1
        C(1,J)=C(2,J)*(R(3)-R(1))/(R(3)-R(2))
     1        -C(3,J)*(R(2)-R(1))/(R(3)-R(2))
        C(N1,J)=C(N1-1,J)*(R(N1)-R(N1-2))/(R(N1-1)-R(N1-2))
     1         -C(N1-2,J)*(R(N1)-R(N1-1))/(R(N1-1)-R(N1-2))
  23  CONTINUE
      RETURN
      END
C
      SUBROUTINE INTPL(XV,YV,N1,C,R,S,M1)
C
C INTPL interpolates to find YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is the
C coefficient matrix, from subroutine SPMAT, relating the
C second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second deivative SUM2 at end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE
C Find interpolate YV at XV.
      YV=S(J)+(S(J+1)-S(J))*(XV-R(J))/(R(J+1)-R(J))
     1 +(XV-R(J))*(XV-R(J+1))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     2 +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION P1(AN,LAMBDA,MU)
C Finds value of function P1 for series solution at geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE PRECISION LAMBDA,MU
      P1=2.D0*AN*(AN*(AN+2.D0)*LAMBDA+(AN*(AN+2.D0)-1.D0)*MU)
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION P2(AN,LAMBDA,MU)
C Finds value of function P2 for series solution at geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE PRECISION LAMBDA,MU
      P2=AN*(AN+5.D0+(AN+3.D0)*(LAMBDA/MU))
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION Q1(AN,LAMBDA,MU)
C Finds value of function Q1 for series solution at geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE PRECISION LAMBDA,MU
      Q1=2.D0*AN*((AN+2.D0)*LAMBDA+(AN+1.D0)*MU)
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION Q2(AN,LAMBDA,MU)
C Finds value of function Q2 for series solution at geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE PRECISION LAMBDA,MU
      Q2=2.D0*(AN+1.D0)+(AN+3.D0)*(LAMBDA/MU)
      RETURN
      END 
C
      SUBROUTINE MATRIX(A,RHO,LAMBDA,MU,PI,G,AN,WE)
C This subroutine calculates the 6X6 matrix A required to find the second
C terms in the power series expansions of the third free solution at the
C geocentre.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(6,6),RHO(100)
      DOUBLE PRECISION LAMBDA(100),MU(100)
      BETA=1.D0/(LAMBDA(1)+2.D0*MU(1))
      DELTA=2.D0*MU(1)*(3.D0*LAMBDA(1)+2.D0*MU(1))*BETA
      GAMMA=(4.D0*PI*G*RHO(1)-2.D0*WE*WE)/3.D0
      AN1=AN*(AN+1.D0)
      EPSLN=4.D0*AN1*MU(1)*(LAMBDA(1)+MU(1))*BETA-2.D0*MU(1)
      A(1,1)=AN+3.D0+2.D0*LAMBDA(1)*BETA
      A(1,2)=-BETA
      A(1,3)=-AN1*LAMBDA(1)*BETA
      A(1,4)=0.D0
      A(1,5)=0.D0
      A(1,6)=0.D0
      A(2,1)=-2.D0*DELTA
      A(2,2)=AN+2.D0+4.D0*MU(1)*BETA
      A(2,3)=AN1*DELTA
      A(2,4)=-AN1
      A(2,5)=0.D0
      A(2,6)=0.D0
      A(3,1)=1.D0
      A(3,2)=0.D0
      A(3,3)=AN+2.D0
      A(3,4)=-1.D0/MU(1)
      A(3,5)=0.D0
      A(3,6)=0.D0
      A(4,1)=DELTA
      A(4,2)=LAMBDA(1)*BETA
      A(4,3)=-EPSLN
      A(4,4)=AN+5.D0
      A(4,5)=0.D0
      A(4,6)=0.D0
      A(5,1)=-3.D0*GAMMA-2.D0*WE*WE
      A(5,2)=0.D0
      A(5,3)=0.D0
      A(5,4)=0.D0
      A(5,5)=AN+4.D0
      A(5,6)=-1.D0
      A(6,1)=0.D0
      A(6,2)=0.D0
      A(6,3)=AN1*(3.D0*GAMMA+2.D0*WE*WE)
      A(6,4)=0.D0
      A(6,5)=-AN1
      A(6,6)=AN+5.D0
      RETURN
      END
C
      SUBROUTINE LINSOL(A,B,N,C,DET,M1,M2)
C
C LINSOL solves the linear system Ax=b by Gauss-Jordan
C elimination with partial pivoting. A is the N x N 
C coefficient matrix and B is the N-length constant vector b.
C The inverse of A is returned in A, the solution vector x
C is returned in B, and the determinant of A is returned
C in DET. C is the N x 2N+1 augmented matrix used
C internally. M1 is the maximum dimension for N specified
C in the main programme and M2=2M1+1 is the maximum dimension
C for 2N+1 used in the main programme.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(M1,M1),X(M1),B(M1),C(M1,M2),HOLD(M1,M2),
     1IR(M1),IC(M1),IDET(M1)
C Construct the augmented matrix C.
      N2=2*N+1
      NP1=N+1
      NM1=N-1
C Insert the matrix A.
      DO 10 I=1,N
        DO 10 J=1,N
          C(I,J)=A(I,J)
 10     CONTINUE
C Insert the unknown vector.
      DO 11 J=1,N
        C(J,NP1)=B(J)
 11   CONTINUE
C Augment with the identity matrix.
      DO 12 I=1,N
        DO 12 J=1,N
        JJ=J+NP1
          C(I,JJ)=0.D0
            IF(I.EQ.J)C(I,JJ)=1.D0
 12   CONTINUE
c      write(9,200)((c(i,j),j=1,n2),i=1,n)
c 200  format(1x,7d11.4)
C Begin Gauss-Jordan elimination.
      DET=1.D0
      DO 13 I=1,N
        IM1=I-1
C Search for largest pivot.
        PIVOT=0.D0
        DO 14 J=1,N
          DO 14 K=1,N
C Scan IR and IC vectors for invalid pivot subscripts.
            IF(I.EQ.1)GO TO 15
            DO 16 ISC=1,IM1
              DO 16 JSC=1,IM1
              IF(J.EQ.IR(ISC))GO TO 14
              IF(K.EQ.IC(JSC))GO TO 14
 16           CONTINUE
 15           IF(DABS(C(J,K)).LE.DABS(PIVOT))GO TO 14
              PIVOT=C(J,K)
              IR(I)=J
              IC(I)=K
 14       CONTINUE
          DET=DET*PIVOT
C Label next row.
          IRP1=IR(I)+1
C Label previous row.
          IRM1=IR(I)-1
C Divide row by pivot.
          DO 17 J=1,N2
            C(IR(I),J)=C(IR(I),J)/PIVOT
 17       CONTINUE
C Reduce elements below pivot to zero.
          IF(IRP1.GT.N)GO TO 18
          DO 19 K=IRP1,N
            ELL=C(K,IC(I))
            DO 20 J=1,N2
              C(K,J)=C(K,J)-C(IR(I),J)*ELL
 20         CONTINUE
 19       CONTINUE
 18       CONTINUE
C Reduce elements above pivot to zero.
          IF(IRM1.LT.1)GO TO 21
          DO 22 K=1,IRM1
            ELL=C(K,IC(I))
            DO 23 J=1,N2
              C(K,J)=C(K,J)-C(IR(I),J)*ELL
 23         CONTINUE
 22       CONTINUE
 21       CONTINUE
 13   CONTINUE     
C Unscramble equations and record row interchanges.
      DO 24 I=1,N
        IDET(IR(I))=IC(I)
        DO 24 J=1,N2
        HOLD(IC(I),J)=C(IR(I),J)
 24   CONTINUE
C Put equations back in original matrix.
      DO 25 I=1,N
        DO 25 J=1,N2
        C(I,J)=HOLD(I,J)
 25   CONTINUE
C Determine sign of determinant.
      ICH=0
      DO 26 I=1,NM1
      IP1=I+1
        DO 26 J=IP1,N
        IF(IDET(J).GE.IDET(I))GO TO 26
        IHOLD=IDET(J)
        IDET(J)=IDET(I)
        IDET(I)=IHOLD
        ICH=ICH+1
 26   CONTINUE
      IF((ICH/2)*2.NE.ICH)DET=-DET
C Put inverse in A and solution in B.
      DO 27 I=1,N
      B(I)=C(I,NP1)
        DO 27 J=1,N
        JJ=J+NP1
        A(I,J)=C(I,JJ)
 27   CONTINUE
      RETURN
      END  
C
      SUBROUTINE REL(ERRMAX,DELTA,YIC,N)
C This subroutine is used in the variable step size Runge-Kutta integration
C of the fundamental solutions in the Inner Core. It finds the maximum
C relative error in the elements of the difference array, DELTA(6,3), compared
C to the elements of the fundamental solution array, YIC(6,3),for degree N.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DELTA(6,3),YIC(6,3)
C Zero relative error.
      ERRMAX=0.D0
      DO 1 I=1,6
C Branch for degee zero.
        IF(N.EQ.0)GO TO 2
        GO TO 3
C If degree is zero skip y_3 and y_4.
 2      IF(I.EQ.3.OR.I.EQ.4)GO TO 1
 3      DO 4 J=1,3
C Third solution is only fundamental solution for degree zero.
        IF(N.EQ.0.AND.J.NE.3)GO TO 4
        IF(YIC(I,J).EQ.0.D0)GO TO 4
        RATIO=DABS(DELTA(I,J)/YIC(I,J))
        ERRMAX=DMAX1(ERRMAX,RATIO)
 4      CONTINUE
 1    CONTINUE
      RETURN
      END  
C
      SUBROUTINE YPRIME(R,Y,A,YP,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,PI,G,
     1 IR,RMIN,ANGS,COR,WES)
C This subroutine finds the derivatives, YP(6,6), of the six
C fundamental solutions represented by the propagator matrix,
C Y(6,6), in the mantle and crust (shell), at an arbitrary radius R
C for degree N>0. For N=0, the third and fourth solution vectors can
C be ignored. In the inner core, it finds the derivatives of the
C fundamental solutions in terms of the z-variables, which are
C transformations of the y-variables. 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION RM(100),RHOM(100),GZEROM(100),C(100,100),
     1 Y(6,6),YP(6,6),A(6,6)
      DOUBLE PRECISION MUM(100),LAMBDAM(100),MU,LAMBDA 
C Zero fill derivative matrix YP(I,J) and coefficient matrix A(I,J).
      DO 10 I=1,6
        DO 11 J=1,6
          YP(I,J)=0.D0
          A(I,J)=0.D0
 11     CONTINUE
 10   CONTINUE
C Set maximum number of points for interpolation.
      M1=100
C If R is less than or equal to RMIN, take properties to be those at the
C geocentre.
      IF(R.LE.RMIN) GO TO 12
C Interpolate Earth properties at radius R.
      CALL INTPL(R,RHO,N1,C,RM,RHOM,M1)
      CALL INTPL(R,MU,N1,C,RM,MUM,M1)
      CALL INTPL(R,LAMBDA,N1,C,RM,LAMBDAM,M1)         
      CALL INTPL(R,GZERO,N1,C,RM,GZEROM,M1)
      GO TO 13
C Assign geocentric Earth properties.
 12   RHO=RHOM(1)
      MU=MUM(1)
      LAMBDA=LAMBDAM(1)
      GZERO=(4.D0*PI*G*RHO-2.D0*WES)*R/3.D0
C Construct matrix A.
 13   AN=DFLOAT(N)
C Define factors.
      F1=AN*(AN+1.D0)
      F2=LAMBDA+2.D0*MU
      F3=3.D0*LAMBDA+2.D0*MU
      F4=RHO*GZERO
      F5=4.D0*PI*G*RHO
C Assign values to matrix elements.
      A(1,1)=-2.D0*LAMBDA/(F2*R)
      A(1,2)=1.D0/F2
      A(1,3)=F1*LAMBDA/(F2*R)
      A(1,4)=0.D0
      A(1,5)=0.D0
      A(1,6)=0.D0
      A(2,1)=4.D0*(-F4+MU*F3/(F2*R))/R-2.D0*RHO*WES-ANGS*RHO
      A(2,2)=-4.D0*MU/(F2*R)
      A(2,3)=F1*(F4-2.D0*MU*F3/(F2*R))/R+COR*RHO
      A(2,4)=F1/R
      A(2,5)=0.D0
      A(2,6)=-RHO
      IF(N.EQ.0.AND.IR.EQ.1) GO TO 14
      A(3,1)=-1.D0/R
      A(3,2)=0.D0
      A(3,3)=1.D0/R
      A(3,4)=1.D0/MU
      A(3,5)=0.D0
      A(3,6)=0.D0
      A(4,1)=(F4-2.D0*MU*F3/(F2*R))/R
C If degree is not zero, add Coriolis term.
      IF(N.NE.0) A(4,1)=A(4,1)+COR*RHO/F1
      A(4,2)=-LAMBDA/(F2*R)
      A(4,3)=2.D0*MU*((2.D0*F1-1.D0)*LAMBDA+(2.D0*F1-2.D0)*MU)/(F2*R*R)
     1 -ANGS*RHO
C If degree is not zero, add Coriolis term.
      IF(N.NE.0) A(4,3)=A(4,3)+COR*RHO/F1
      A(4,4)=-3.D0/R
      A(4,5)=-RHO/R
      A(4,6)=0.D0
 14   CONTINUE
      A(5,1)=F5
      A(5,2)=0.D0
      A(5,3)=0.D0
      A(5,4)=0.D0
      A(5,5)=0.D0
      A(5,6)=1.D0
      IF (N.EQ.0.AND.IR.EQ.1) GO TO 15
      A(6,1)=0.D0
      A(6,2)=0.D0
      A(6,3)=-F5*F1/R
      A(6,4)=0.D0
      A(6,5)=F1/(R*R)
 15   CONTINUE
      A(6,6)=-2.D0/R
C Branch to inner core calculation
      IF(IR.EQ.1) GO TO 16
C Multiply A into Y to find derivative matrix for shell.
      DO 17 I=1,6
        DO 18 J=1,6
          DO 19 K=1,6
            YP(I,J)=YP(I,J)+A(I,K)*Y(K,J)
 19       CONTINUE
 18     CONTINUE    
 17   CONTINUE
      GO TO 20
C Modify matrix A for inner core calculation.
 16   A(1,2)=A(1,2)/R
      A(2,1)=A(2,1)*R
      A(2,3)=A(2,3)*R
      A(2,6)=A(2,6)*R
      A(3,4)=A(3,4)/R
      A(4,1)=A(4,1)*R
      A(4,3)=A(4,3)*R
      A(4,5)=A(4,5)*R*R
      A(5,1)=A(5,1)/R
      A(5,6)=A(5,6)/R
      A(6,5)=A(6,5)*R
C Multiply A into Y to find YP for inner core.
      DO 21 I=1,6
        DO 22 J=1,3
          DO 23 K=1,6
            YP(I,J)=YP(I,J)+A(I,K)*Y(K,J)
 23       CONTINUE
 22     CONTINUE    
 21   CONTINUE
C Correct YP to give derivatives of z-variables.
C Set value of alpha plus one for first two solutions.
      ANM1=AN-1.D0
      DO 24 J=1,3
C Set value of alpha plus one for third solution
        IF(J.EQ.3) ANM1=AN+1.D0
        YP(1,J)=YP(1,J)-ANM1*Y(1,J)/R
        YP(2,J)=YP(2,J)-(ANM1-1.D0)*Y(2,J)/R
        YP(3,J)=YP(3,J)-ANM1*Y(3,J)/R
        YP(4,J)=YP(4,J)-(ANM1-1.D0)*Y(4,J)/R
        YP(5,J)=YP(5,J)-(ANM1+1.D0)*Y(5,J)/R
        YP(6,J)=YP(6,J)-ANM1*Y(6,J)/R
 24   CONTINUE
 20   CONTINUE
      RETURN
      END
C
      SUBROUTINE RK4(R,Y,A,K1,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,
     1PI,G,H,IR,RMIN,ANGS,COR,WES)
C This subroutine completes the last three steps of a fourth order
C Runge-Kutta integration, given the derivatives K1(6,6) at the starting
C radius R.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION RM(100),RHOM(100),GZEROM(100),C(100,100),
     1 Y(6,6),YY(6,6),A(6,6)
      DOUBLE PRECISION MUM(100),LAMBDAM(100),MU,LAMBDA,
     1 K1(6,6),K2(6,6),K3(6,6),K4(6,6)
C Set half stepsize, one sixth stepsize.
      HH=0.5D0*H
      H6=H/6.D0
C Set number of solutions for shell.
      NS=6
C If region is inner core, reset number of solutions to three.
      IF(IR.EQ.1)NS=3
C Complete remaining steps of Runge-Kutta integration.
      DO 10 I=1,6
        DO 11 J=1,NS
          YY(I,J)=Y(I,J)+HH*K1(I,J)
 11     CONTINUE  
 10   CONTINUE
C Increment radius and solve for new derivatives.
      R=R+HH    
      CALL YPRIME(R,YY,A,K2,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,PI,G,IR,
     1 RMIN,ANGS,COR,WES)
      DO 12 I=1,6
        DO 13 J=1,NS
          YY(I,J)=Y(I,J)+HH*K2(I,J)
 13     CONTINUE  
 12   CONTINUE
C Find new derivatives.
      CALL YPRIME(R,YY,A,K3,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,PI,G,IR,
     1 RMIN,ANGS,COR,WES)
      DO 14 I=1,6
        DO 15 J=1,NS
          YY(I,J)=Y(I,J)+H*K3(I,J)
 15     CONTINUE  
 14   CONTINUE
C Increment radius and solve for new derivatives.    
      R=R+HH
      CALL YPRIME(R,YY,A,K4,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,PI,G,IR,
     1 RMIN,ANGS,COR,WES)
      DO 16 I=1,6
        DO 17 J=1,NS
          Y(I,J)=Y(I,J)+H6*(K1(I,J)+2.D0*(K2(I,J)+K3(I,J))+K4(I,J))
 17     CONTINUE  
 16     CONTINUE
      RETURN
      END    
