C                          PROGRAMME MAT.FOR
C
C MAT.FOR calculates the eleven matrices required for Core Mode calculations
C and places them in the output file matres.dat. A small constant
C stratification is assumed, and the real Earth decompression factor is read
C in from the file decomp.dat. Boundary conditions at the inner core and
C outer core-mantle boundaries are applied, using internal Love numbers read
C in from the file love.dat.
C 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BM(264,264,11),BS(34980),RI(6),ZJ(11),
     1 W(12),X(12),ETA(12),PSI(12),ETAP(12),PSIP(12),
     2 H1NA(48),H1NB(48),ENAME(10),
     3 R(100),F(100),B(98,198),C(100,100),CMI(24,44,4),CMM(24,44,4),
     4 ETAR(12,10),ETAL(12,10),PSIR(12,10),PSIL(12,10),
     5 ETAPR(12,10),ETAPL(12,10),PSIPR(12,10),PSIPL(12,10)
      OPEN(UNIT=1,FILE='love.dat',STATUS='OLD')
      OPEN(UNIT=2,FILE='decomp.dat',STATUS='OLD')
      OPEN(UNIT=3,FILE='matres.dat',STATUS='UNKNOWN')
C Read in and write out Earth model name.
      READ(1,10)(ENAME(I),I=1,10)
 10   FORMAT(10A8)
      WRITE(6,11)(ENAME(I),I=1,10)
 11   FORMAT(/5X,'Love numbers based on',1X,10A8)
C Read in number of terms in Love number expansion, radii of inner core
C boundary and core-mantle boundary, and gravity at the inner core boundary
C and at the core-mantle boundary.
      READ(1,12) NLN
 12   FORMAT(I10)
      READ(1,13) RIOB,RCMB
 13   FORMAT(2D23.15)
      READ(1,13) GZEROC,GZEROM
      WRITE(6,14)
 14   FORMAT(1X,'Parameters are:'/
     1'    ICORE (0 For inner core, 1 for no inner core.),'/
     2'    ICOMP (0 for compressible, 1 for incompressible.),'/
     3'    PARITY (0 for even, 1 for odd.), and'/
     4'    M (the azimuthal number).')
      WRITE(6,15)
 15   FORMAT(1X,'Enter values of ICORE, ICOMP, IPAR and M.')
      READ(5,*)ICORE,ICOMP,IPAR,M
      WRITE(6,16)
 16   FORMAT(1X,'Enter number of elements in r-direction',
     1' and number of elements in z-direction.')       
      READ(5,*)MM1,NM1
C Calculate maximum number of Love numbers that can be used for
C the application of boundary conditions.
      INUM1=(NLN-IPAR-M-1)/2+1
      INUM2=4*NM1
      IF(INUM1.LE.INUM2) INU=INUM1
      IF(INUM1.GT.INUM2) INU=INUM2
C Test for too many constraints.
      NCC=4*NM1+2-INU
      IF(NCC.GE.2) GO TO 17
      WRITE(6,18)
 18   FORMAT(1X,'Too many constraints, start over.')
      GO TO 19
C Test if enough Love numbers available.
 17   ISUM=(INU-1)*2+IPAR+M+1
      IF(ISUM.LE.NLN) GO TO 20
      WRITE(6,21)
 21   FORMAT(1X,'Not enough Love numbers available, start over.')
      GO TO 19
C Write out parameters.
C Write out Earth model name.
 20   WRITE(3,10)(ENAME(I),I=1,10)
      WRITE(3,22)ICORE,ICOMP,IPAR,M,RIOB,RCMB
 22   FORMAT(4I4,2D23.15)
C Write out gravity at the inner core boundary and
C at the core-mantle boundary.
      WRITE(3,13) GZEROC,GZEROM
C Write out number of elements in each co-ordinate direction
C and number of Love numbers used.
      WRITE(3,23) MM1,NM1,INU
 23   FORMAT(3I4)
C Read in inner core Love numbers and equivalent radii. 
      READ(1,24)(H1NA(I),I=1,NLN)
 24   FORMAT(D23.15)
C Read in shell (mantle and crust) Love numbers and
C equivalent radii.
      READ(1,24)(H1NB(I),I=1,NLN)
C Read in frequency coefficients for degree-one inner core
C Love number.
      READ(1,25)ALPHA,BETA,GAMMA
 25   FORMAT(3D23.15)
C Read in decompression factor f.
C Read in and write out Earth model name.
      READ(2,10)(ENAME(I),I=1,10)
      WRITE(6,26)(ENAME(I),I=1,10)
 26   FORMAT(5X,'Decompression factor based on',1X,10A8)
      READ(2,12) N1
      DO 27 I=1,N1
        J=N1-I+1
        READ(2,13)R(J),F(J)
 27   CONTINUE
C Prepare for interpolation of decompression factor f.
      M1=100
      M2=198
      M3=98
      N2=2*N1-2
      N3=N1-2
      CALL SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C Set angular frequency of Earth's rotation (WGS 84).
      WE=7.292115D-5  
C Set value of pi.
      PI=3.141592653589793D0
C Write out frequency coefficients.
      WRITE(3,25)ALPHA,BETA,GAMMA
C Set values of Gaussian integration points.
      X(1)=-0.981560634246719D0
      X(2)=-0.904117256370475D0
      X(3)=-0.769902674194305D0
      X(4)=-0.587317954286617D0
      X(5)=-0.367831498998180D0
      X(6)=-0.125233408511469D0
      X(7)=-X(6)
      X(8)=-X(5)
      X(9)=-X(4)
      X(10)=-X(3)
      X(11)=-X(2)
      X(12)=-X(1)
C Assign Gaussian integration weights.
      W(1)=0.047175336386512D0
      W(2)=0.106939325995318D0
      W(3)=0.160078328543346D0
      W(4)=0.203167426723066D0
      W(5)=0.233492536538355D0
      W(6)=0.249147045813403D0
      W(7)=W(6)
      W(8)=W(5)
      W(9)=W(4)
      W(10)=W(3)
      W(11)=W(2)
      W(12)=W(1)
C Calculate cubic splines in radius at Gaussian integration points.
      DO 28 I=1,12
        YI=0.5D0*(X(I)+1.D0)
        ETA(I)=ETA0(YI)
        PSI(I)=PSI0(YI)
        ETAP(I)=ETA0P(YI)
        PSIP(I)=PSI0P(YI)
 28   CONTINUE
C Number of elements in r-direction is MM1, number in z-direction
C is NM1, where there are MM1P1 nodes in r-direction, NM1P1 nodes
C in z-direction.
      MM1P1=MM1+1
      NM1P1=NM1+1
C Compute number of columns in constraint matrices and
C in reduced constraint matrices.
      NC=NM1P1*4
      NCR=NC-2
C Compute order of main matrix and length of reduced
C symmetric storage vectors BS.
      INR=4*MM1P1*NM1P1
      INRR=INR-2*MM1P1
      IF(ICORE.EQ.0)GO TO 29
      INRR=INRR-4*NM1P1+2
      IF(M.EQ.0.AND.IPAR.EQ.0)INRR=INRR+NM1P1
 29   INSV=(INRR*(INRR+1))/2
C Set node points in radius.
      RI(1)=RIOB/RCMB
      IF(ICORE.EQ.1) RI(1)=0.D0
      AMM1=DFLOAT(MM1)
      DRI=(1.D0-RI(1))/AMM1
      DO 30 I=1,MM1
        IP1=I+1
        AI=DFLOAT(I)
        RI(IP1)=RI(1)+AI*DRI
 30   CONTINUE
C Set node points in z.
      ZJ(1)=0.D0
      ANM1=DFLOAT(NM1)
      DTHETA=PI/(2.D0*ANM1)
      DO 31 I=1,NM1
        CI=DFLOAT(NM1-I)
        CI=CI*DTHETA
        IP1=I+1
        ZJ(IP1)=DCOS(CI)
 31   CONTINUE
C Calculate support functions in z.
      DO 32 I=1,NM1
        IP1=I+1
        CC=ZJ(I)
        DD=ZJ(IP1)
        DZM=DD-CC
        DZP=CC+DD
        DENOM=DZM*DZP
        DO 33 JJ=1,12
          Z=0.5D0*(DZM*X(JJ)+DZP)
          DZZMC=Z-CC
          DZZPC=CC+Z
          XJJR=DZZMC*DZZPC/DENOM
          DZZMD=Z-DD
          DZZPD=DD+Z
          XJJL=DZZMD*DZZPD/DENOM
          TZDD=2.D0*Z/DENOM
          ZDZIP1=Z/DD
          DDTZP1=DENOM/(2.D0*DD)
          IF(IPAR.EQ.1) GO TO 34
C Even case calculation.
          IF(I.GT.1) GO TO 35
C Support at z=0.
          ETAR(JJ,I)=ZETA0(XJJR)
          PSIR(JJ,I)=0.D0
          ETAPR(JJ,I)=TZDD*ZETA0P(XJJR)
          PSIPR(JJ,I)=0.D0
C Support at z=z1.
          ETAL(JJ,I)=ZETA0(XJJL)+2.D0*PHI0(XJJL)
          PSIL(JJ,I)=DDTZP1*PHI0(XJJL)
          ETAPL(JJ,I)=TZDD*(ZETA0P(XJJL)+2.D0*PHI0P(XJJL))
          PSIPL(JJ,I)=ZDZIP1*PHI0P(XJJL)
          GO TO 33
 35       DDTZI=DENOM/(2.D0*CC)
          ZDZI=Z/CC
          ETAR(JJ,I)=ETA0(XJJR)
          PSIR(JJ,I)=DDTZI*PSI0(XJJR)
          ETAPR(JJ,I)=TZDD*ETA0P(XJJR)
          PSIPR(JJ,I)=ZDZI*PSI0P(XJJR)
          ETAL(JJ,I)=ETA0(XJJL)
          PSIL(JJ,I)=DDTZP1*PSI0(XJJL)
          ETAPL(JJ,I)=TZDD*ETA0P(XJJL)
          PSIPL(JJ,I)=ZDZIP1*PSI0P(XJJL)
          GO TO 33
C Odd case calculation.
 34       TZSDD=Z*TZDD
          DDTZSP=DDTZP1/DD
          IF(I.GT.1) GO TO 36
C Support at z=0.
          ETAR(JJ,I)=0.D0
          PSIR(JJ,I)=Z*ZETA0(XJJR)
          ETAPR(JJ,I)=0.D0
          PSIPR(JJ,I)=TZSDD*ZETA0P(XJJR)+ZETA0(XJJR)
C Support at z=z1.
          ETAL(JJ,I)=ZDZIP1*(ZETA0(XJJL)+3.D0*DDTZSP*PHI0(XJJL))
          PSIL(JJ,I)=Z*DDTZSP*PHI0(XJJL)
          ETAPL(JJ,I)=(ZETA0(XJJL)+3.D0*DDTZSP*PHI0(XJJL)
     1     +TZSDD*ZETA0P(XJJL)+3.D0*ZDZIP1*ZDZIP1*PHI0P(XJJL))/DD
          PSIPL(JJ,I)=DDTZSP*PHI0(XJJL)+ZDZIP1*ZDZIP1*PHI0P(XJJL)
          GO TO 33    
 36       ZDZI=Z/CC
          DDTZIS=DENOM/(2.D0*CC*CC)
          ETAR(JJ,I)=ZDZI*(ETA0(XJJR)-DDTZIS*PSI0(XJJR))
          PSIR(JJ,I)=Z*DDTZIS*PSI0(XJJR)
          ETAPR(JJ,I)=(ETA0(XJJR)-DDTZIS*PSI0(XJJR)
     1     +TZSDD*ETA0P(XJJR)-ZDZI*ZDZI*PSI0P(XJJR))/CC
          PSIPR(JJ,I)=DDTZIS*PSI0(XJJR)+ZDZI*ZDZI*PSI0P(XJJR)
          ETAL(JJ,I)=ZDZIP1*(ETA0(XJJL)-DDTZSP*PSI0(XJJL))
          PSIL(JJ,I)=Z*DDTZSP*PSI0(XJJL)
          ETAPL(JJ,I)=(ETA0(XJJL)-DDTZSP*PSI0(XJJL)
     1     +TZSDD*ETA0P(XJJL)-ZDZIP1*ZDZIP1*PSI0P(XJJL))/DD
          PSIPL(JJ,I)=DDTZSP*PSI0(XJJL)+ZDZIP1*ZDZIP1*PSI0P(XJJL)
 33     CONTINUE
 32   CONTINUE
C Calculate eleven matrices.
      CALL MATRICES(BM,INR,CMI,CMM,INU,NC,M,ICORE,ICOMP,
     1 IPAR,MM1,NM1,RI,ZJ,X,W,ETA,PSI,ETAP,PSIP,ETAR,PSIR,ETAPR,PSIPR,
     2 ETAL,PSIL,ETAPL,PSIPL,H1NA,H1NB,
     3 RIOB,RCMB,GZEROC,GZEROM,WE,R,F,C,N1,M1)
C Write out eleven matrices.
      DO 37 I=1,11
C Put in symmetric storage vector BS.
        CALL SYMST(BM,BS,INRR,I)
        WRITE(3,24)(BS(K),K=1,INSV)
 37   CONTINUE
C Write out constraint matrices.
      IF(ICORE.EQ.1) GO TO 38
      WRITE(3,24)(((CMI(I,J,K),K=1,4),J=1,NCR),I=1,INU)
 38   WRITE(3,24)(((CMM(I,J,K),K=1,4),J=1,NCR),I=1,INU)
 19   CONTINUE
      END

      SUBROUTINE MATRICES(BM,INR,CMI,CMM,INU,NC,M,ICORE,ICOMP,
     1 IPAR,MM1,NM1,RI,ZJ,X,W,ETA,PSI,ETAP,PSIP,ETAR,PSIR,ETAPR,PSIPR,
     2 ETAL,PSIL,ETAPL,PSIPL,H1NA,H1NB,
     3 RIOB,RCMB,GZEROC,GZEROM,WE,RR,F,CC,N1,M1)
C SUBROUTINE MATRICES constructs the eleven matrices, BM, required
C for solving for core modes in the case of small stratification.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DI(4,4,8),DIB(4,4),DNJ(6,24,10),DNJ1(6,24,10),
     1 DJ(4),DJ1(4),W(12),X(12),ETA(12),PSI(12),ETAP(12),PSIP(12),
     2 ETAR(12,10),PSIR(12,10),ETAPR(12,10),PSIPR(12,10),
     3 ETAL(12,10),PSIL(12,10),ETAPL(12,10),PSIPL(12,10),
     4 RI(6),ZJ(11),AM(3,3,8),DMIJ(3,4),DMI1J(3,4),DMI1J1(3,4),
     5 DMIJ1(3,4),BM(264,264,11),H1NA(48),H1NB(48),
     6 RR(100),F(100),CC(100,100),CMI(24,44,4),CMM(24,44,4)
C Begin construction of the matrices.
      MM1P1=MM1+1
      NM1P1=NM1+1
      MT4=4*MM1P1
      NT4=4*NM1P1
C Convert azimuthal number to a real double precision variable.
      RM=DFLOAT(M)
C Zero matrices.
      DO 10 I=1,INR
        DO 10 J=1,INR
          DO 10 K=1,11
            BM(I,J,K)=0.D0
 10   CONTINUE
C Zero D vector integrals.
      DO 11 L=1,6
        DO 11 N=1,INU
            DO 11 KJ=1,NM1
              DNJ(L,N,KJ)=0.D0
              DNJ1(L,N,KJ)=0.D0
 11   CONTINUE
C Zero constraint matrices.
      DO 12 I=1,INU
        DO 12 J=1,NC
          DO 12 K=1,4
            CMI(I,J,K)=0.D0
            CMM(I,J,K)=0.D0
 12   CONTINUE
C Begin do loops over finite elements.
      DO 13 I=1,MM1
        A=RI(I)
        B=RI(I+1)
        DR=B-A
        CONSTI=0.5D0*DR
        DO 13 J=1,NM1      
          C=ZJ(J)
          D=ZJ(J+1)
          DZ=D-C
          CONSTJ=0.5D0*DZ
C Calculate contribution from i,jth element.
          DO 14 JJ=1,12
            ETAZR=ETAR(JJ,J)
            PSIZR=PSIR(JJ,J)
            ETAZPR=ETAPR(JJ,J)
            PSIZPR=PSIPR(JJ,J)
            ETAZL=ETAL(JJ,J)
            PSIZL=PSIL(JJ,J)
            ETAZPL=ETAPL(JJ,J)
            PSIZPL=PSIPL(JJ,J)
            Z=0.5D0*(DZ*X(JJ)+(C+D))
            CALL AIJ(AM,M,Z,RUMZSM,RM,ZS,UMZS)
            WJ=CONSTJ*W(JJ)
C Set up to find contributions of surface integrals.
            IF(I.NE.1) GO TO 15
C Construct boundary D vectors.
            DJ(1)=ETAZR
            DJ(2)=ETAZPR
            DJ(3)=PSIZR
            DJ(4)=PSIZPR
            DJ1(1)=ETAZL
            DJ1(2)=ETAZPL
            DJ1(3)=PSIZL
            DJ1(4)=PSIZPL
C Calculate integrals of D vectors over boundary. 
            DO 16 INN=1,INU
              N=(INN-1)*2+IPAR+M
              AN=N
              FACT=(2.D0*AN+1.D0)*FACTOR(N-M)*RUMZSM*PMN(Z,M,N)
     1        /FACTOR(N+M)            
              FACT=FACT*WJ
              DNJ(1,INN,J)=DNJ(1,INN,J)+FACT*(RM*ZS*DJ(1)-Z*UMZS*DJ(2))
              DNJ(2,INN,J)=DNJ(2,INN,J)+FACT*(RM*ZS*DJ(3)-Z*UMZS*DJ(4))
              DNJ(3,INN,J)=DNJ(3,INN,J)-FACT*ZS*DJ(1)
              DNJ(4,INN,J)=DNJ(4,INN,J)-FACT*ZS*DJ(3)
              DNJ(5,INN,J)=DNJ(5,INN,J)+FACT*DJ(1)
              DNJ(6,INN,J)=DNJ(6,INN,J)+FACT*DJ(3)
              DNJ1(1,INN,J)=DNJ1(1,INN,J)+FACT*(RM*ZS*DJ1(1)-Z*UMZS*
     1        DJ1(2))
              DNJ1(2,INN,J)=DNJ1(2,INN,J)+FACT*(RM*ZS*DJ1(3)-Z*UMZS*
     1        DJ1(4))
              DNJ1(3,INN,J)=DNJ1(3,INN,J)-FACT*ZS*DJ1(1)
              DNJ1(4,INN,J)=DNJ1(4,INN,J)-FACT*ZS*DJ1(3)
              DNJ1(5,INN,J)=DNJ1(5,INN,J)+FACT*DJ1(1) 
              DNJ1(6,INN,J)=DNJ1(6,INN,J)+FACT*DJ1(3)
 16         CONTINUE
C Find volume integral contributions.
 15         CONTINUE
            DO 14 II=1,12
              ETARR=ETA(II)
              PSIRR=DR*PSI(II)
              ETARPR=ETAP(II)/DR
              PSIRPR=PSIP(II)
              IC=13-II
              ETARL=ETA(IC)
              PSIRL=-DR*PSI(IC)
              ETARPL=-ETAP(IC)/DR
              PSIRPL=PSIP(IC)
              R=0.5D0*(DR*X(II)+(B+A))
C Find physical radius and interpolate decompression factor f.
              RP=R*RCMB
              CALL INTPL(RP,FRP,N1,CC,RR,F,M1)
              IF(ICOMP.EQ.1) FRP=1.D0
              WJI=CONSTI*WJ*W(II)*FRP
C Construct D matrices.
              CALL DIJ(DMIJ,ETARR,ETAZR,PSIRR,PSIZR,
     1        ETARPR,ETAZPR,PSIRPR,PSIZPR,R)
              CALL DIJ(DMI1J,ETARL,ETAZR,PSIRL,PSIZR,
     1        ETARPL,ETAZPR,PSIRPL,PSIZPR,R)
              CALL DIJ(DMI1J1,ETARL,ETAZL,PSIRL,PSIZL,
     1        ETARPL,ETAZPL,PSIRPL,PSIZPL,R)
              CALL DIJ(DMIJ1,ETARR,ETAZL,PSIRR,PSIZL,
     1        ETARPR,ETAZPL,PSIRPR,PSIZPL,R)
C Multiply matrices and add contributions to main matrices.
              CALL DTAD(DI,DMIJ,AM,DMIJ,WJI)
              IBI=4*((I-1)*NM1P1+(J-1))
              JBI=IBI
              CALL ADD(DI,BM,IBI,JBI)
              CALL DTAD(DI,DMIJ,AM,DMI1J,WJI)
              JBI=JBI+NT4
              CALL ADD(DI,BM,IBI,JBI)
              CALL TRANS(DI)
              IBI=IBI+NT4
              JBI=JBI-NT4
              CALL ADD(DI,BM,IBI,JBI)
              CALL DTAD(DI,DMIJ,AM,DMI1J1,WJI)
              IBI=IBI-NT4
              JBI=JBI+NT4+4
              CALL ADD(DI,BM,IBI,JBI)
              CALL TRANS(DI)
              IBI=IBI+NT4+4
              JBI=JBI-NT4-4
              CALL ADD(DI,BM,IBI,JBI)
              CALL DTAD(DI,DMIJ,AM,DMIJ1,WJI)
              IBI=IBI-NT4-4
              JBI=JBI+4
              CALL ADD(DI,BM,IBI,JBI)
              CALL TRANS(DI)
              IBI=IBI+4
              JBI=JBI-4
              CALL ADD(DI,BM,IBI,JBI)
              CALL DTAD(DI,DMI1J,AM,DMI1J,WJI)
              IBI=IBI+NT4-4
              JBI=JBI+NT4
              CALL ADD(DI,BM,IBI,JBI)
              CALL DTAD(DI,DMI1J,AM,DMI1J1,WJI)
              JBI=JBI+4
              CALL ADD(DI,BM,IBI,JBI)
              CALL TRANS(DI)
              IBI=IBI+4
              JBI=JBI-4
              CALL ADD(DI,BM,IBI,JBI)
              CALL DTAD(DI,DMI1J,AM,DMIJ1,WJI)
              IBI=IBI-4
              JBI=JBI-NT4+4
              CALL ADD(DI,BM,IBI,JBI)
              CALL TRANS(DI)
              IBI=IBI-NT4+4
              JBI=JBI+NT4-4
              CALL ADD(DI,BM,IBI,JBI)
              CALL DTAD(DI,DMI1J1,AM,DMI1J1,WJI)
              IBI=IBI+NT4
              JBI=JBI+4
              CALL ADD(DI,BM,IBI,JBI)
              CALL DTAD(DI,DMI1J1,AM,DMIJ1,WJI)
              JBI=JBI-NT4
              CALL ADD(DI,BM,IBI,JBI)
              CALL TRANS(DI)
              IBI=IBI-NT4
              JBI=JBI+NT4
              CALL ADD(DI,BM,IBI,JBI)
              CALL DTAD(DI,DMIJ1,AM,DMIJ1,WJI)
              JBI=JBI-NT4
              CALL ADD(DI,BM,IBI,JBI)
 14       CONTINUE
 13   CONTINUE
C Add boundary contributions to main matrices.
      FACT=-4.D0*WE*WE
      IF(ICOMP.EQ.1) F(1)=1.D0
      FACTA=-F(1)*RI(1)*RIOB*FACT/GZEROC
      FACTB=RCMB*FACT/GZEROM
      DO 17 INN=1,INU
        N=(INN-1)*2+IPAR+M
        AN=N
        FACTN=FACTOR(N+M)/((2.D0*AN+1.D0)*FACTOR(N-M))
        IF(N.EQ.1) GO TO 18
        IMAT=9
        FACTNA=FACTA*H1NA(N+1)*FACTN
        GO TO 19
 18     IMAT=11
        FACTNA=FACTA*FACTN
 19     FACTNB=FACTB*H1NB(N+1)*FACTN
        DO 17 J=1,NM1
          DO 17 K=1,NM1
            IF(ICORE.EQ.1) GO TO 20
C Add contribution of inner core boundary.
            CALL DTD(DIB,DNJ,DNJ,FACTNA,J,K,INN)
            IBI=4*(J-1)
            JBI=4*(K-1)
            CALL ADDB(DIB,BM,IBI,JBI,IMAT)
            CALL DTD(DIB,DNJ,DNJ1,FACTNA,J,K,INN)
            JBI=JBI+4
            CALL ADDB(DIB,BM,IBI,JBI,IMAT)
            CALL DTD(DIB,DNJ1,DNJ,FACTNA,J,K,INN)
            IBI=IBI+4
            JBI=JBI-4
            CALL ADDB(DIB,BM,IBI,JBI,IMAT)
            CALL DTD(DIB,DNJ1,DNJ1,FACTNA,J,K,INN)
            JBI=JBI+4
            CALL ADDB(DIB,BM,IBI,JBI,IMAT)
C Add contribution of core-mantle boundary.
 20         CALL DTD(DIB,DNJ,DNJ,FACTNB,J,K,INN)
            IBI=4*(MM1*NM1P1+(J-1))
            JBI=4*(MM1*NM1P1+(K-1))
            CALL ADDB(DIB,BM,IBI,JBI,10)
            CALL DTD(DIB,DNJ,DNJ1,FACTNB,J,K,INN)
            JBI=JBI+4
            CALL ADDB(DIB,BM,IBI,JBI,10)
            CALL DTD(DIB,DNJ1,DNJ,FACTNB,J,K,INN)
            IBI=IBI+4
            JBI=JBI-4
            CALL ADDB(DIB,BM,IBI,JBI,10)
            CALL DTD(DIB,DNJ1,DNJ1,FACTNB,J,K,INN)
            JBI=JBI+4
            CALL ADDB(DIB,BM,IBI,JBI,10)
 17   CONTINUE
C Construct constraint matrices for elastic boundary conditions.
      DO 21 INN=1,INU
C Find degree.
        N=(INN-1)*2+IPAR+M
        IF(N.EQ.1) GO TO 22
        FACTNA=FACT*H1NA(N+1)*RIOB/GZEROC
        GO TO 23
 22     FACTNA=FACT*RIOB/GZEROC
 23     FACTNB=FACT*H1NB(N+1)*RCMB/GZEROM
        DO 21 J=1,NM1 
          JCI=(J-1)*4
          JCIP4=JCI+4
          IF(ICORE.EQ.1) GO TO 24
C Add dc term at inner core boundary.
          DJ(1)=DNJ(1,INN,J)
          DJ(2)=RI(1)*DNJ(3,INN,J)
          DJ(3)=DNJ(2,INN,J)
          DJ(4)=RI(1)*DNJ(4,INN,J)
          CALL ADDC(DJ,CMI,1,INN,JCI)
          DJ1(1)=DNJ1(1,INN,J)
          DJ1(2)=RI(1)*DNJ1(3,INN,J)
          DJ1(3)=DNJ1(2,INN,J)
          DJ1(4)=RI(1)*DNJ1(4,INN,J)
          CALL ADDC(DJ1,CMI,1,INN,JCIP4)
C Add dc term at core-mantle boundary.
 24       DJ(1)=DNJ(1,INN,J)
          DJ(2)=DNJ(3,INN,J)
          DJ(3)=DNJ(2,INN,J)
          DJ(4)=DNJ(4,INN,J)
          CALL ADDC(DJ,CMM,1,INN,JCI)
          DJ1(1)=DNJ1(1,INN,J)
          DJ1(2)=DNJ1(3,INN,J)
          DJ1(3)=DNJ1(2,INN,J)
          DJ1(4)=DNJ1(4,INN,J)
          CALL ADDC(DJ1,CMM,1,INN,JCIP4)
          IF(ICORE.EQ.1) GO TO 25
C Add linear term at inner core boundary.
          DJ(1)=RM*DNJ(5,INN,J)
          DJ(2)=0.D0
          DJ(3)=RM*DNJ(6,INN,J)
          DJ(4)=0.D0
          CALL ADDC(DJ,CMI,2,INN,JCI)
          DJ1(1)=RM*DNJ1(5,INN,J)
          DJ1(2)=0.D0
          DJ1(3)=RM*DNJ1(6,INN,J)
          DJ1(4)=0.D0
          CALL ADDC(DJ1,CMI,2,INN,JCIP4)
C Add linear term at core-mantle boundary.
 25       DJ(1)=RM*DNJ(5,INN,J)
          DJ(2)=0.D0
          DJ(3)=RM*DNJ(6,INN,J)
          DJ(4)=0.D0
          CALL ADDC(DJ,CMM,2,INN,JCI)
          DJ1(1)=RM*DNJ1(5,INN,J)
          DJ1(2)=0.D0
          DJ1(3)=RM*DNJ1(6,INN,J)
          DJ1(4)=0.D0
          CALL ADDC(DJ1,CMM,2,INN,JCIP4)
          IF(ICORE.EQ.1) GO TO 26
C Add quadratic term at inner core boundary.
          DJ(1)=0.D0
          DJ(2)=RI(1)*DNJ(5,INN,J)
          DJ(3)=0.D0
          DJ(4)=RI(1)*DNJ(6,INN,J)
          CALL ADDC(DJ,CMI,3,INN,JCI)
          DJ1(1)=0.D0
          DJ1(2)=RI(1)*DNJ1(5,INN,J)
          DJ1(3)=0.D0
          DJ1(4)=RI(1)*DNJ1(6,INN,J)
          CALL ADDC(DJ1,CMI,3,INN,JCIP4)
C Add quadratic term at core-mantle boundary.
 26       DJ(1)=0.D0
          DJ(2)=DNJ(5,INN,J)
          DJ(3)=0.D0
          DJ(4)=DNJ(6,INN,J)
          CALL ADDC(DJ,CMM,3,INN,JCI)
          DJ1(1)=0.D0
          DJ1(2)=DNJ1(5,INN,J)
          DJ1(3)=0.D0
          DJ1(4)=DNJ1(6,INN,J)
          CALL ADDC(DJ1,CMM,3,INN,JCIP4)
          IF(ICORE.EQ.1) GO TO 27
C Add chi term at inner core boundary.
          DJ(1)=FACTNA*DNJ(5,INN,J)
          DJ(2)=0.D0
          DJ(3)=FACTNA*DNJ(6,INN,J)
          DJ(4)=0.D0
          CALL ADDC(DJ,CMI,4,INN,JCI)
          DJ1(1)=FACTNA*DNJ1(5,INN,J)
          DJ1(2)=0.D0
          DJ1(3)=FACTNA*DNJ1(6,INN,J)
          DJ1(4)=0.D0
          CALL ADDC(DJ1,CMI,4,INN,JCIP4)
C Add chi term at core-mantle boundary.
 27       DJ(1)=FACTNB*DNJ(5,INN,J)
          DJ(2)=0.D0
          DJ(3)=FACTNB*DNJ(6,INN,J)
          DJ(4)=0.D0
          CALL ADDC(DJ,CMM,4,INN,JCI)
          DJ1(1)=FACTNB*DNJ1(5,INN,J)
          DJ1(2)=0.D0
          DJ1(3)=FACTNB*DNJ1(6,INN,J)
          DJ1(4)=0.D0
          CALL ADDC(DJ1,CMM,4,INN,JCIP4)
 21   CONTINUE
C Apply evenness, oddness conditions at the equator, IPAR=0
C for even solution, IPAR=1 for odd solution, and find column
C numbers to be struck out of constraint matrices and strike
C them out.
      II=3-2*IPAR
      IIP1=II+1
      IF(ICORE.EQ.1) GO TO 28
      CALL SOC(CMI,NC,II,0,INU)
      CALL SOC(CMI,NC,IIP1,1,INU)
 28   CALL SOC(CMM,NC,II,0,INU)
      CALL SOC(CMM,NC,IIP1,1,INU)
C Strike out rows and columns of main matrices.
      DO 29 I=1,MM1P1
C Calculate II, the row and column number to be struck out.
        II=NT4*(I-1)+3-2*IPAR
        IIP1=II+1
C Calculate JJ, the number of rows and columns already
C struck out. 
        JJ=2*I-2
        JJP1=JJ+1
        CALL SORAC(BM,INR,II,JJ)
        CALL SORAC(BM,INR,IIP1,JJP1)
 29   CONTINUE
C Apply conditions at Earth's centre for case with
C no inner core.
      IF(ICORE.EQ.0) GO TO 30
      INRR=INR-2*MM1P1
      JJ=0
      DO 31 I=1,NM1P1
        II=(I-1)*4+1
        IF(II.GT.1)II=II-2
        IIP1=II+1
        IIP2=II+2
        IIP3=II+3
        IF(M.EQ.0.AND.IPAR.EQ.0)GO TO 32
        CALL SORAC(BM,INRR,II,JJ)
        JJ=JJ+1
 32     CALL SORAC(BM,INRR,IIP1,JJ)
        JJ=JJ+1
        IF(I.EQ.1)GO TO 31
        CALL SORAC(BM,INRR,IIP2,JJ)
        JJ=JJ+1
        CALL SORAC(BM,INRR,IIP3,JJ)
        JJ=JJ+1
 31   CONTINUE
 30   RETURN   
      END

      SUBROUTINE SORAC(B,INR,II,JJ)
C This subroutine strikes out the IIth rows and the IIth columns
C of the matrices B, given that JJ rows and columns have already
C been struck out.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION B(264,264,11)
C Find KK, the row and column numbers of the current matrices to be
C struck out.
      KK=II-JJ
      INRR=INR-JJ
      INRRM1=INRR-1
      DO 10 J=1,11
        DO 10 K=1,INRRM1
          DO 10 L=1,INRRM1
            IF(K.LT.KK.AND.L.LT.KK) GO TO 11
            IF(K.GE.KK.AND.L.GE.KK) GO TO 13
            IF(L.LT.KK) GO TO 14
            B(K,L,J)=B(K,L+1,J)
            GO TO 11
 14         B(K,L,J)=B(K+1,L,J)
            GO TO 11
 13         B(K,L,J)=B(K+1,L+1,J)
 11         CONTINUE
 10   CONTINUE
C Zero fill last row and column.
      DO 12 J=1,11
        DO 12 I=1,INRR
          B(I,INRR,J)=0.D0
          B(INRR,I,J)=0.D0
 12   CONTINUE
      RETURN
      END

      SUBROUTINE SOC(CM,NC,II,JJ,INU)
C This subroutine strikes out the IIth columns of the constraint
C matrices CM, given that JJ columns have already been struck out. 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION CM(24,44,4)
C Find LL, the column number of the current matrices to be
C struck out.
      LL=II-JJ
C Find current number of columns.
      ICR=NC-JJ
C Move columns to left
      ICRM1=ICR-1
      DO 10 J=1,4
        DO 10 K=1,INU
          DO 10 L=1,ICRM1
            IF(L.LT.LL) GO TO 10
            CM(K,L,J)=CM(K,L+1,J)
 10   CONTINUE
C Zero fill last column.
      DO 11 J=1,4
        DO 11 K=1,INU
          CM(K,ICR,J)=0.D0
 11   CONTINUE
      RETURN
      END

      SUBROUTINE SYMST(BM,B,INRR,II)
C Symmetric storage vector conversion. The subroutine SYMST
C takes the IIth symmetric matrix BM of dimension INRR by INRR
C and puts it in the symmetric storage vector B of length
C INRR*(INRR+1)/2.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION BM(264,264,11),B(34980)
      DO 10 I=1,INRR
        DO 10 J=1,I
          K=(I*(I-1)/2)+J
          B(K)=BM(I,J,II)
 10   CONTINUE
      RETURN
      END

      SUBROUTINE AIJ(AM,M,Z,RUMZSM,RM,ZS,UMZS)
C This subroutine finds the 3x3 matrices A appearing in
C the functional integrals for the i,jth element.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION AM(3,3,8)
      MM1=M-1
      ZS=Z*Z
      UMZS=1.D0-ZS
C Find FACT equal to 1-ZS to the power M and FACT1 equal to
C 1-ZS to the power M-1.
      FACT=1.D0
      FACT1=1.D0
      IF(M.EQ.0) GO TO 10
      DO 11 I=1,M
        FACT=FACT*UMZS
 11   CONTINUE
      IF(M.EQ.1) GO TO 10 
      DO 12 I=1,MM1
        FACT1=FACT1*UMZS
 12   CONTINUE
C Find 1-ZS to the power M/2.
 10   RUMZSM=DSQRT(FACT)
C Evaluate first matrix times 1-ZS to the power M.
      AM(1,1,1)=FACT1*RM*RM*(1.D0+ZS)
      AM(1,2,1)=0.D0
      AM(1,3,1)=-RM*Z*FACT
      AM(2,1,1)=AM(1,2,1)
      AM(2,2,1)=FACT
      AM(2,3,1)=0.D0
      AM(3,1,1)=AM(1,3,1)
      AM(3,2,1)=AM(2,3,1)
      AM(3,3,1)=UMZS*FACT
C Evaluate the second matrix.
      AM(1,1,2)=RM*RM*ZS
      AM(1,2,2)=-RM*ZS
      AM(1,3,2)=-RM*Z*UMZS
      AM(2,1,2)=AM(1,2,2)
      AM(2,2,2)=ZS
      AM(2,3,2)=Z*UMZS
      AM(3,1,2)=AM(1,3,2)
      AM(3,2,2)=AM(2,3,2)
      AM(3,3,2)=UMZS*UMZS
C Evaluate the part of the third matrix proportional to
C the fourth power of the angular frequency.
      AM(1,1,3)=0.D0
      AM(1,2,3)=0.D0
      AM(1,3,3)=0.D0
      AM(2,1,3)=0.D0
      AM(2,2,3)=1.D0
      AM(2,3,3)=0.D0
      AM(3,1,3)=0.D0
      AM(3,2,3)=0.D0
      AM(3,3,3)=0.D0
C Evaluate the part of the third matrix proportional to
C the third power of the angular frequency.
      AM(1,1,4)=0.D0
      AM(1,2,4)=RM
      AM(1,3,4)=0.D0
      AM(2,1,4)=AM(1,2,4)
      AM(2,2,4)=0.D0
      AM(2,3,4)=0.D0
      AM(3,1,4)=0.D0
      AM(3,2,4)=0.D0
      AM(3,3,4)=0.D0
C Evaluate the part of the third matrix proportional to
C the square of the angular frequency.
      AM(1,1,5)=RM*RM
      AM(1,2,5)=RM*ZS
      AM(1,3,5)=0.D0
      AM(2,1,5)=AM(1,2,5)
      AM(2,2,5)=-2.D0*ZS
      AM(2,3,5)=-Z*UMZS
      AM(3,1,5)=0.D0
      AM(3,2,5)=AM(2,3,5)
      AM(3,3,5)=0.D0
C Evaluate the part of the third matrix linear in
C the angular frequency.
      AM(1,1,6)=2.D0*RM*RM*ZS
      AM(1,2,6)=-RM*ZS
      AM(1,3,6)=-RM*Z*UMZS
      AM(2,1,6)=AM(1,2,6)
      AM(2,2,6)=0.D0
      AM(2,3,6)=0.D0
      AM(3,1,6)=AM(1,3,6)
      AM(3,2,6)=0.D0
      AM(3,3,6)=0.D0
C Evaluate the part of the third matrix independent of
C the angular frequency.
      AM(1,1,7)=RM*RM*ZS*ZS
      AM(1,2,7)=-RM*ZS*ZS
      AM(1,3,7)=-RM*Z*ZS*UMZS
      AM(2,1,7)=AM(1,2,7)
      AM(2,2,7)=ZS*ZS
      AM(2,3,7)=Z*ZS*UMZS
      AM(3,1,7)=AM(1,3,7)
      AM(3,2,7)=AM(2,3,7)
      AM(3,3,7)=ZS*UMZS*UMZS
C Evaluate the fourth matrix times M times (1-ZS) to the power M.
      AM(1,1,8)=FACT1*2.D0*RM*RM*ZS
      AM(1,2,8)=RM*FACT
      AM(1,3,8)=-RM*Z*FACT
      AM(2,1,8)=AM(1,2,8)
      AM(2,2,8)=0.D0
      AM(2,3,8)=0.D0
      AM(3,1,8)=AM(1,3,8)
      AM(3,2,8)=0.D0
      AM(3,3,8)=0.D0   
C Multiply matrix elements AM(I,J,2) through AM(I,J,7) by common
C factor 1-ZS to the power M.
      DO 13 I=1,3
        DO 13 J=1,3
          DO 13 K=2,7
            AM(I,J,K)=FACT*AM(I,J,K)
 13   CONTINUE
      RETURN
      END

      SUBROUTINE DIJ(DM,ETAR,ETAZ,PSIR,PSIZ,ETARP,ETAZP,PSIRP,PSIZP,R)
C This subroutine constructs the 3x4 matrices of polynomial splines
C Di,j, Di+1,j, Di+1,j+1, and Di,j+1 arising at the corners of
C the finite element Ri,j.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DM(3,4)
      DM(1,1)=ETAR*ETAZ
      DM(1,2)=PSIR*ETAZ
      DM(1,3)=ETAR*PSIZ
      DM(1,4)=PSIR*PSIZ
      DM(2,1)=R*ETARP*ETAZ
      DM(2,2)=R*PSIRP*ETAZ
      DM(2,3)=R*ETARP*PSIZ
      DM(2,4)=R*PSIRP*PSIZ
      DM(3,1)=ETAR*ETAZP
      DM(3,2)=PSIR*ETAZP
      DM(3,3)=ETAR*PSIZP
      DM(3,4)=PSIR*PSIZP
      RETURN
      END

      SUBROUTINE DTAD(DI,DM1,A,DM2,WJI)
C DTAD calculates the 4x4 matrix products of D transpose
C times A times D and weights them with the Gaussian integration
C weight WJI.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DI(4,4,8),DM1(3,4),A(3,3,8),DM2(3,4),AD(3,4)
      DO 10 L=1,8
        DO 11 I=1,3
          DO 11 K=1,4
            AD(I,K)=0.D0
            DO 11 J=1,3
              AD(I,K)=AD(I,K)+A(I,J,L)*DM2(J,K)
 11     CONTINUE
        DO 10 I=1,4
          DO 10 K=1,4
            DI(I,K,L)=0.D0
            DO 10 J=1,3
              DI(I,K,L)=DI(I,K,L)+DM1(J,I)*AD(J,K)*WJI
 10   CONTINUE
      RETURN
      END

      SUBROUTINE DTD(DI,DNJ,DNK,FACTN,J,K,INN)
C DTD calculates the 4x4 matrix DI which is the product of
C the vectors DNJ and DNK transpose. 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DI(4,4),DNJ(6,24,10),DNK(6,24,10)
      DO 10 II=1,4
        DO 10 JJ=1,4
          DI(II,JJ)=0.D0
 10   CONTINUE
      DI(1,1)=DNJ(5,INN,J)*DNK(5,INN,K)*FACTN
      DI(1,3)=DNJ(5,INN,J)*DNK(6,INN,K)*FACTN
      DI(3,1)=DNJ(6,INN,J)*DNK(5,INN,K)*FACTN
      DI(3,3)=DNJ(6,INN,J)*DNK(6,INN,K)*FACTN
      RETURN
      END

      SUBROUTINE ADD(DI,BM,IBI,JBI)
C This subroutine adds the eight volume integral
C contributions from the 4x4 matrix DI to the main
C matrix BM, given the initial location indices
C for the elements of DI in the main matrix, IBI, JBI.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DI(4,4,8),BM(264,264,11)
      DO 10 J=1,8
        DO 10 K=1,4
          DO 10 L=1,4
            IB=IBI+K
            JB=JBI+L
            BM(IB,JB,J)=BM(IB,JB,J)+DI(K,L,J)
 10   CONTINUE
      RETURN
      END

      SUBROUTINE ADDB(DIB,BM,IBI,JBI,I)
C Subroutine to add boundary contributions to matrices.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DIB(4,4),BM(264,264,11)
      DO 10 K=1,4
        DO 10 L=1,4
          IB=IBI+K
          JB=JBI+L
          BM(IB,JB,I)=BM(IB,JB,I)+DIB(K,L)
 10   CONTINUE
      RETURN
      END

      SUBROUTINE ADDC(D,CM,I,IN,JCI)
C Subroutine to add contributions to constraint matrices.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION D(4),CM(24,44,4)
      DO 10 K=1,4
        JC=JCI+K
        CM(IN,JC,I)=CM(IN,JC,I)+D(K)
 10   CONTINUE
      RETURN
      END

      SUBROUTINE TRANS(DI)
C This subroutine transposes the 4X4 matrix DI.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION DI(4,4,8)
      DO 10 K=1,8
        DO 10 I=1,3
          IP1=I+1
          DO 10 J=IP1,4
            HOLD=DI(J,I,K)
            DI(J,I,K)=DI(I,J,K)
            DI(I,J,K)=HOLD
 10   CONTINUE
      RETURN
      END


      DOUBLE PRECISION FUNCTION ETA0(X)
C This function calculates the canonical cubic local
C Hermite spline, eta, supporting the function. 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      IF(X.GE.0.D0) GO TO 10
      A=X+1.D0
      B=A+A-3.D0
      ETA0=-B*A*A
      GO TO 11
 10   A=X-1.D0
      B=A+A+3.D0
      ETA0=B*A*A
 11   RETURN
      END 

      DOUBLE PRECISION FUNCTION PSI0(X)
C This function calculates the canonical cubic local
C Hermite spline, psi, supporting the derivative of
C the function.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      IF(X.GE.0.D0) GO TO 10
      A=X+1.D0
      PSI0=X*A*A
      GO TO 11
 10   A=X-1.D0
      PSI0=X*A*A
 11   RETURN
      END 
 
      DOUBLE PRECISION FUNCTION ETA0P(X)
C This function calculates the derivative of the local
C Hermite spline eta.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      IF(X.GE.0.D0) GO TO 10
      ETA0P=(-6.D0*X-6.D0)*X
      GO TO 11
 10   ETA0P=(6.D0*X-6.D0)*X
 11   RETURN
      END 
 
      DOUBLE PRECISION FUNCTION PSI0P(X)
C This function calculates the derivative of the local
C Hermite spline psi.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      IF(X.GE.0.D0) GO TO 10
      PSI0P=(3.D0*X+4.D0)*X+1.D0
      GO TO 11
 10   PSI0P=(3.D0*X-4.D0)*X+1.D0
 11   RETURN
      END

      DOUBLE PRECISION FUNCTION ZETA0(X)
C This function calculates the canonical quadratic, zeta,
C supporting the function.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      IF(X.GE.0.D0) GO TO 10
      A=X+1.D0
      B=-A-A-A+4.D0
      ZETA0=A*B
      GO TO 11
 10   A=X-1.D0
      ZETA0=A*A
 11   RETURN
      END

      DOUBLE PRECISION FUNCTION PHI0(X)
C This function calculates the canonical quadratic, phi,
C supporting the derivative of the function.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      IF(X.GE.0.D0) GO TO 10
      A=X+1.D0
      PHI0=A*X
      GO TO 11
 10   PHI0=0.D0
 11   RETURN
      END

      DOUBLE PRECISION FUNCTION ZETA0P(X)
C This function calculates the derivative of
C the canonical quadratic zeta. 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      IF(X.GE.0.D0) GO TO 10
      ZETA0P=-6.D0*X-2.D0
      GO TO 11
 10   ZETA0P=2.D0*(X-1.D0)
 11   RETURN
      END

      DOUBLE PRECISION FUNCTION PHI0P(X)
C This function caculates the derivative of
C the canonical quadratic phi.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      IF(X.GE.0.D0) GO TO 10
      PHI0P=2.D0*X+1.D0
      GO TO 11
 10   PHI0P=0.D0
 11   RETURN
      END 
C
      DOUBLE PRECISION FUNCTION PMN(Z,M,N)
C
C PMN computes the value of the Legendre function of degree N, azimuthal
C number M, for argument Z. The procedure begins with formula for
C the sectoral value for N=M, and then uses the recurrence relation
C across degree to arrive at the value for degree N.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C Test if input arguments are out of range.
      IF(M.LT.0.OR.DABS(Z).GT.1.D0) WRITE(6,10)
 10   FORMAT(1X,'ONE OR MORE INPUT PARAMETERS OUT OF RANGE')
C Set Legendre function to zero if M>N and return.
      PMN=0.D0
      IF(M.GT.N) GO TO 11
C Begin calculation of sectoral value PMM.
C Set initial value of PMM.
      PMM=1.D0
C Test if Legendre polynomial.
      IF(M.EQ.0) GO TO 12
C Find square root of 1-Z*Z.
      SRUMZ2=DSQRT((1.D0-Z)*(1.D0+Z))
C Complete calculation of sectoral value PMM.
      FACT=1.D0
      DO 13 I=1,M
        PMM=-PMM*FACT*SRUMZ2
        FACT=FACT+2.D0
 13   CONTINUE
C Test if sectoral Legendre function.
 12   IF(M.EQ.N) GO TO 14
C Convert M, N to floating point variables.
      AM=DFLOAT(M)
      AN=DFLOAT(N)
C Find next higher degree Legendre function.
      PMMP1=(2.D0*AM+1.D0)*Z*PMM
C Test if M+1=N.
      IF(M+1.EQ.N) GO TO 15
C Prepare to complete recursion.
      MP2=M+2
      P1=PMM
      P2=PMMP1
C Complete recursion.
      DO 16 I=MP2,N
        AI=DFLOAT(I)
        P3=((2.D0*AI-1.D0)*Z*P2-(AI+AM-1.D0)*P1)/(AI-AM)
        P1=P2
        P2=P3
 16   CONTINUE
      PMN=P3
      GO TO 11
 14   PMN=PMM
      GO TO 11
 15   PMN=PMMP1
 11   CONTINUE
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION FACTOR(N)
C Factorial function.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      FACTOR=1.D0
      IF(N.EQ.0) GO TO 2
      DO 1 I=1,N
      AI=I
    1 FACTOR=AI*FACTOR
    2 CONTINUE
      RETURN
      END  

      SUBROUTINE SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C
C SPMAT computes the N1 by N1 matrix C connecting
C second derivatives to function values at N1 nodal points R(I).
C B is the N3=N1-2 by N2=2*N1-2 augmented matrix used internally
C in the Gaussian elimination procedure.
C M1, M2, M3 are the respective maximum dimensions for N1, N2, N3
C specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),B(M3,M2)
      N=N3+1
C Construct augmented matrix A.
      DO 11 I=1,N3
        DO 10 J=1,N2
          B(I,J)=0.D0
  10    CONTINUE
        B(I,I)=2.D0*(R(I+2)-R(I))
        B(I,I+N3)=1.D0
        IF(I.EQ.1.OR.I.EQ.N3) GO TO 11
        B(I,I-1)=R(I+1)-R(I)
        B(I,I+1)=R(I+2)-R(I+1)
  11  CONTINUE
C Apply conditions that third derivatives are constant
C in first two and last two intervals.
      B(1,2)=R(3)-R(2)-((R(2)-R(1))**2.D0)/(R(3)-R(2))
      B(N3,N3-1)=R(N3+1)-R(N3)-((R(N1)-R(N1-1))**2.D0)/(R(N1-1)-R(N1-2))
      B(1,1)=B(1,1)+((R(3)-R(1))*(R(2)-R(1)))/(R(3)-R(2))
      B(N3,N3)=B(N3,N3)
     1 +((R(N1)-R(N1-1))*(R(N1)-R(N1-2)))/(R(N1-1)-R(N1-2))
      NM1=N3-1
C Eliminate sub-diagonal.
      DO 13 I=1,NM1
        DIAG=B(I,I)
        ELL=B(I+1,I)
        DO 12 J=1,N2
          B(I,J)=B(I,J)/DIAG
          B(I+1,J)=B(I+1,J)-B(I,J)*ELL
  12    CONTINUE
  13  CONTINUE
C Eliminate super-diagonal.
      DO 15 I=1,NM1
        NO=N-I
        DIAG=B(NO,NO)
        ELL=B(NO-1,NO)
        DO 14 J=1,N2
          B(NO,J)=B(NO,J)/DIAG
          B(NO-1,J)=B(NO-1,J)-B(NO,J)*ELL
  14    CONTINUE
  15  CONTINUE
C Move inverse to position formerly occupied by A.
      DO 17 I=1,N3
        DO 16 J=1,N3
          B(I,J)=B(I,J+N3)
  16    CONTINUE
  17  CONTINUE
C Construct matrix B in N-2 by N augmentation space. 
      DO 19 I=1,N3
        DO 18 J=1,N1
          B(I,J+N3)=0.0D0
  18    CONTINUE
        IN=I+N3
        FACTOR=6.D0/((R(I+2)-R(I+1))*(R(I+1)-R(I)))
        B(I,IN)=FACTOR*(R(I+2)-R(I+1))
        B(I,IN+1)=-FACTOR*(R(I+2)-R(I))
        B(I,IN+2)=FACTOR*(R(I+1)-R(I))
  19  CONTINUE
C Multiply inverse of A into B to get N-2 by N version of C.
      DO 22 I=1,N3
        DO 21 J=1,N1
          SUM=0.D0
          DO 20 K=1,N3
            SUM=SUM+B(I,K)*B(K,J+N3)
  20      CONTINUE
          C(I+1,J)=SUM
  21    CONTINUE
  22  CONTINUE
C Add first and last rows to C.
      DO 23 J=1,N1
        C(1,J)=C(2,J)*(R(3)-R(1))/(R(3)-R(2))
     1 -C(3,J)*(R(2)-R(1))/(R(3)-R(2))
        C(N1,J)=C(N1-1,J)*(R(N1)-R(N1-2))/(R(N1-1)-R(N1-2))
     1 -C(N1-2,J)*(R(N1)-R(N1-1))/(R(N1-1)-R(N1-2))
  23  CONTINUE
      RETURN
      END
C
      SUBROUTINE INTPL(XV,YV,N1,C,R,S,M1)
C
C INTPL interpolates to find YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is
C the coefficient matrix, from subroutine SPMAT, relating
C the second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second derivative SUM2 at end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE
C Find interpolate YV at XV.
      YV=S(J)+(S(J+1)-S(J))*(XV-R(J))/(R(J+1)-R(J))
     1  +(XV-R(J))*(XV-R(J+1))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     2  +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END
