C                  PROGRAMME MEMSPEC.FOR
C
C MEMSPEC.FOR finds the maximum entropy method (MEM) spectral density for
C the VLBI pole path, interpolated onto 3,513 equally spaced time points
C by natural spline interpolation, for a record segment of 26.25 years
C in length.  The length of the prediction error filter is entered as input
C on request.
C
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DIMENSION GT(4000),G(4000),PEF(4000),PER(4000),DFT(4000)
      DOUBLE PRECISION DAY(4000),AM,PEP,PI,OMEGA,T,DT,AIM1,FREQ(4000),
     1 AJ, ARG,SME(4000),DF
      OPEN(UNIT=1,FILE='ppintp.dat',STATUS='OLD')
      OPEN(UNIT=2,FILE='memspec.dat',STATUS='UNKNOWN')
C Set number of interpolated pole positions.
      M=3513
      AM=DFLOAT(M)
C Set values of pi and 2pi.
      PI=3.141592653589793D0
      OMEGA=2.D0*PI
C Set record length.
      T=26.25D0
C Find sample interval.
      DT=T/AM
C Set mean of pole path to zero.
      GTM=(0.D0,0.D0)
C Set prediction error power to zero.
      PEP=0.D0
C Read in interpolated pole path.
      DO 10 I=1,M
        READ(1,11)J,IYYY,IM,ID,DAY(I),GT(I)
 11     FORMAT(1X,4I5,F12.7,2F12.6)
C Accumulate sum of co-ordinates.
        GTM=GTM+GT(I)
 10   CONTINUE
C Find mean of pole positions.
      GTM=GTM/AM
C Reduce co-ordinates to zero mean.
      DO 12 I=1,M
        GT(I)=GT(I)-GTM
C Find sum of squares of pole positions.
        PEP=PEP+GT(I)*DCONJG(GT(I))
 12   CONTINUE
C Find initial value of prediction error power.
      PEP=PEP/AM
C Enter length of prediction error filter.
      WRITE(6,13)
 13   FORMAT(1X,'Enter length of prediction error filter.')
      READ(5,*)K
C Find prediction error filter.
      DO 14 NP1=1,K
        CALL BPEC(M,NP1,GT,G,PEF,PER)
C Update prediction error power.
        PEP=PEP*(1.D0-G(NP1)*DCONJG(G(NP1)))
 14   CONTINUE
C Find DFT of prediction error filter.
      DF=0.01D0
      DO 15 I=1,301
        IM1=I-1
        AIM1=DFLOAT(IM1)
        FREQ(I)=-1.5D0+AIM1*DF
C Initialize DFT.
        DFT(I)=(1.D0,0.D0)
        DO 16 J=1,K
          AJ=DFLOAT(J)
          ARG=-OMEGA*FREQ(I)
          ARG=ARG*AJ*DT
         DFT(I)=DFT(I)+G(J)*DCMPLX(DCOS(ARG),DSIN(ARG))
 16     CONTINUE
 15   CONTINUE
C Find and write out MEM spectrum.
      DO 17 I=1,301
        SME(I)=DT*PEP/(DFT(I)*DCONJG(DFT(I)))
        WRITE(2,18)FREQ(I),SME(I)
 18     FORMAT(1X,D15.5,D26.15)
 17   CONTINUE
      END
C
      SUBROUTINE BPEC(M,NP1,F,G,PEF,PER)
C
C Subroutine BPEC finds the prediction error filter coefficients G
C of length NP1 from those of length N for the time sequence F of length M.
C It also returns the forward and backward prediction error sequences.
C
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DIMENSION F(M),G(M),H(M),O(M),PEF(M),PER(M)
C Find previous recursion number.
      N=NP1-1
      IF(N.NE.0) GO TO 10
C If previous recursion number is zero, set forward and backward
C prediction error sequences to given data sequence.
      DO 11 J=1,M
        PEF(J)=F(J)
        PER(J)=F(J)
 11   CONTINUE
C Find next member of prediction error filter G(NP1).
 10   SN=(0.D0,0.D0)
      SD=(0.D0,0.D0)
C Find upper limit of sum.
      JJ=M-N-1
      DO 12 J=1,JJ
        SN=SN-2.D0*PEF(J+N+1)*DCONJG(PER(J))
        SD=SD+PEF(J+N+1)*DCONJG(PEF(J+N+1))+PER(J)*DCONJG(PER(J))
 12   CONTINUE
      G(NP1)=SN/SD
C If previous recursion number is zero, do not update prediction error
C filter coefficients.
      IF(N.EQ.0) GO TO 13
C Update prediction error filter coefficients and store them in vector H(J).
      DO 14 J=1,N
        K=N-J+1
        H(J)=G(J)+G(NP1)*DCONJG(G(K))
 14   CONTINUE
C Put updated prediction error coefficients in G(J).
      DO 15 J=1,N
        G(J)=H(J)
 15   CONTINUE
C Update forward prediction error sequence and store in H(K).
 13   DO 16 J=1,JJ
        K=J+NP1
        H(K)=PEF(K)+G(NP1)*PER(J)
 16   CONTINUE
C Update backward prediction error sequence and store in O(J).
      DO 17 J=1,JJ
        O(J)=PER(J)+DCONJG(G(NP1))*PEF(J+NP1)
 17   CONTINUE
C Put updated forward and backward prediction error sequences
C in PEF(K) AND PER(J), respectively.
      DO 18 J=1,JJ
        K=J+NP1
        PEF(K)=H(K)
        PER(J)=O(J)
 18   CONTINUE
      RETURN
      END
