C          PROGRAMME POINCARE.FOR
C       
C POINCARE.FOR is an interactive programme to calculate the
C eigenperiods of the Poincare inertial wave equation for an
C ellipsoidal container. The programme allows a range of values of
C the Coriolis frequency to be searched at specified intervals,
C or an iteration to be performed, starting with two initial
C trial periods.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C Specify value of pi.
      PI=3.141592653589793D0
C Set angular frequency of Earth's rotation.
      WE=7.292115D-5
C Calculate number of hours in a semi-sidereal day.
      HSSD=PI/(WE*3600.D0)
C Set flattening to hydrostatic value at the core-mantle boundary
C for Earth model Cal8.
      FLAT=1.D0/394.03D0
C Calculate square of the eccentricity from flattening.
      ESQD=1.D0-(1.D0-FLAT)*(1.D0-FLAT)
C Enter azimuthal number M and degree N.
 10   CONTINUE
      WRITE(6,11)
 11   FORMAT(1X,'Enter M and N')
      READ(5,*)M,N
C Choose between searching a range of Coriolis frequencies or
C iterating on two trial periods.
 12   CONTINUE
      WRITE(6,13)
 13   FORMAT(1X,'Enter -1 for search a range, 0 for iterate')
      READ(5,*)ISW
      IF(ISW.GE.0) GO TO 14
C Enter bottom and top of range of Coriolis frequencies
C and number of increments.
 21   CONTINUE
      WRITE(6,15)
 15   FORMAT(1X,'Enter bottom and top of Coriolis frequency range',1X,
     1 'and number of increments')
      READ(5,*)BOT,TOP,NT
C Find increment in period.
      DSIG=(TOP-BOT)/DFLOAT(NT)
      NTP1=NT+1
C Find residual ERR1 of secular equation at each of trial periods.
      DO 16 I=1,NTP1
C Find Coriolis frequency.
        SIG1=(BOT+DFLOAT(I-1)*DSIG)
        Z=SIG1*DSQRT((1.D0-ESQD)/(1.D0-ESQD*SIG1*SIG1))
C Find error in satisfying secular equation.
        ERR1=ERR(Z,M,N,SIG1)
        TS=12.D0/SIG1
        T=HSSD/SIG1
        WRITE(6,17)SIG1,TS,ERR1,T
 16   CONTINUE
 17   FORMAT(1X,F7.4,2X,'Sidereal period=',F8.4,2X,'Error=',E10.3,2X,
     1'Period',F7.3,1X,'hours')
C Decide whether to do new search or to iterate.
      GO TO 12
C Enter two trial periods for iteration.
 14   CONTINUE
      WRITE(6,18)
 18   FORMAT(1X,'Enter trial periods')
      READ(5,*)T1,T2
      SIG1=12.D0/T1
      Z=SIG1*DSQRT((1.D0-ESQD)/(1.D0-ESQD*SIG1*SIG1))
      ERR1=ERR(Z,M,N,SIG1)
      SIG2=12.D0/T2   
      Z=SIG2*DSQRT((1.D0-ESQD)/(1.D0-ESQD*SIG2*SIG2))
      ERR2=ERR(Z,M,N,SIG2)
C Find next Coriolis frequency.
 19   CONTINUE
      SIG=(SIG1*ERR2-SIG2*ERR1)/(ERR2-ERR1)
C Calculate new sidereal period.
      TS=12.D0/SIG
C Find new period.
      T=HSSD/SIG
      Z=SIG*DSQRT((1.D0-ESQD)/(1.D0-ESQD*SIG*SIG))
C Find error for new iterate.
      ERRI=ERR(Z,M,N,SIG)
      WRITE(6,17)SIG,TS,ERRI,T
C Decide whether to do new search, do another iterate, or end.
      WRITE(6,20)
 20   FORMAT(1X,'Enter -1 for new search, 0 for another iterate,',1X,
     1 'or 1 for new harmonic')
      READ(5,*)IBR
      IF(IBR.LT.0) GO TO 21
      IF(IBR.GT.0) GO TO 23
C Prepare for another iteration.
      SIG1=SIG2
      ERR1=ERR2
      SIG2=SIG
      ERR2=ERRI
      GO TO 19
C Decide to search new harmonic or go to end.
 23   CONTINUE
      WRITE(6,22)
 22   FORMAT(1X,'Enter -1 for new M and N, 0 for end')
      READ(5,*)ISW
      IF(ISW.LT.0) GO TO 10
      END
C
      DOUBLE PRECISION FUNCTION ERR(Z,M,N,SIG)
C Calculates root function for Poincare inertial wave equation
C in a spheroid.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      AM=DFLOAT(M)
      AN=DFLOAT(N)
      ERR=(AN+1.D0)*(AN+AM)*PMN(Z,M,N-1)-AN*(AN-AM+1.D0)*PMN(Z,M,N+1)
     1 -(2.D0*AN+1.D0)*AM*Z*PMN(Z,M,N)/SIG
      RETURN
      END
C
      DOUBLE PRECISION FUNCTION PMN(Z,M,N)
C
C PMN computes the value of the Legendre function of degree N, azimuthal
C number M, for argument Z. The procedure begins with formula for the
C sectoral value for N=M, and then uses the recurrence relation across
C degree to arrive at the value for degree N.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C Test if input arguments are out of range.
      IF(M.LT.0.OR.DABS(Z).GT.1.D0) WRITE(6,10)
 10   FORMAT(1X,'ONE OR MORE INPUT PARAMETERS OUT OF RANGE')
C Set Legendre function to zero if M>N and return.
      PMN=0.D0
      IF(M.GT.N) GO TO 11
C Begin calculation of sectoral value PMM.
C Set initial value of PMM.
      PMM=1.D0
C Test if Legendre polynomial.
      IF(M.EQ.0) GO TO 12
C Find square root of 1-Z*Z.
      SRUMZ2=DSQRT((1.D0-Z)*(1.D0+Z))
C Complete calculation of sectoral value PMM.
      FACT=1.D0
      DO 13 I=1,M
        PMM=-PMM*FACT*SRUMZ2
        FACT=FACT+2.D0
 13   CONTINUE
C Test if sectoral Legendre function.
 12   IF(M.EQ.N) GO TO 14
C Convert M, N to floating point variables.
      AM=DFLOAT(M)
      AN=DFLOAT(N)
C Find next higher degree Legendre function.
      PMMP1=(2.D0*AM+1.D0)*Z*PMM
C Test if M+1=N.
      IF(M+1.EQ.N) GO TO 15
C Prepare to complete recursion.
      MP2=M+2
      P1=PMM
      P2=PMMP1
C Complete recursion.
      DO 16 I=MP2,N
        AI=DFLOAT(I)
        P3=((2.D0*AI-1.D0)*Z*P2-(AI+AM-1.D0)*P1)/(AI-AM)
        P1=P2
        P2=P3
 16   CONTINUE
      PMN=P3
      GO TO 11
 14   PMN=PMM
      GO TO 11
 15   PMN=PMMP1
 11   CONTINUE
      RETURN
      END
