C                    PROGRAMME PPCY2009A.FOR
C
C PPCY2009A.FOR converts pole co-ordinates to centiseconds of arc subtended
C at the geocentre, reverses the sign of the x_2-co-ordinate to change
C to a right-handed co-ordinate system, changes the time base to years
C and converts Julian days to calendar days. Input is pp2009a.dat,
C extracted from the file 2009a.eops on the GSFC website with null values
C eliminated and time values differing by less than a day averaged.
C Output is ppy2009a.dat.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      OPEN(UNIT=1,FILE='pp2009a.dat',STATUS='OLD')
      OPEN(UNIT=2,FILE='ppy2009a.dat',STATUS='UNKNOWN')
C Read in VLBI pole co-ordinates with null values eliminated and values with
C identical time tags averaged.
      DO 10 I=1,3560
        READ(1,11)J,DAY,XWOB,EXWOB,YWOB,EYWOB
C Change Julian days from beginning of the record to years.
        TOLD=DAY/365.25636D0
 11     FORMAT(1X,I4,F16.6,4F10.6)
C Change sign of x2 co-ordinate, scale up to centiseconds of arc.
        XWOB=100.D0*XWOB
        EXWOB=100.D0*EXWOB
        YWOB=-100.D0*YWOB
        EYWOB=100.D0*EYWOB
C Change Julian days to calendar days as year, month, day.
C Add initial Julian day to Julian days from beginning of the record.
        DAY=DAY+44089.993750D0+2400000.D0
        IDAY=INT(DAY)
        CALL CALDAT(IDAY,MM,ID,IYYY)
C Write out converted times and co-ordinates.
        WRITE(2,12)J,IYYY,MM,ID,TOLD,XWOB,YWOB,EXWOB,EYWOB
 12     FORMAT(1X,4I5,F10.6,4F11.6)  
 10   CONTINUE
      END
C
      SUBROUTINE CALDAT(JULIAN,MM,ID,IYYY)
      PARAMETER (IGREG=2299161)
      IF(JULIAN.GE.IGREG)THEN
      JALPHA=INT(((JULIAN-1867216)-0.25)/36524.25)
      JA=JULIAN+1+JALPHA-INT(0.25*JALPHA)
      ELSE
      JA=JULIAN
      ENDIF
      JB=JA+1524
      JC=INT(6680.+((JB-2439870)-122.1)/365.25)
      JD=365*JC+INT(0.25*JC)
      JE=INT((JB-JD)/30.6001)
      ID=JB-JD-INT(30.6001*JE)
      MM=JE-1
      IF(MM.GT.12)MM=MM-12
      IYYY=JC-4715
      IF(MM.GT.2)IYYY=IYYY-1
      IF(IYYY.LE.0)IYYY=IYYY-1
      RETURN
      END
