C
C A programme to test the singular value decomposition subroutine
C SVD and the associated subroutines HHOLDER and BIDIAG.
C The 5x5 complex matrix considered in Section 2.3.6 of Earth Dynamics
C is used for illustration. U and V are unitary matrices and UH and VH
C are their Hermitian transposes. SVD factors the complex coeffcient
C matix C into C=U.W.VH, where W is a real diagonal matrix with the
C positive sngular values of C in decending order down the diagonal.
C W can recovered from C as the product W=UH.C.V.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE COMPLEX C(50,50),U(50,50),VH(50,50),CS(50,50),HOLD1,HOLD2,
     1CP(50,50),CSS(50,50)
      DIMENSION DCR(50,50),DCI(50,50),S(50)
      OPEN(UNIT=1,FILE='testsvd.dat',STATUS='OLD')
C Enter dimension of square matrix C.
      WRITE(6,10)
 10   FORMAT(1X,'Enter dimension of matrix C.')
      READ(5,*)M
C Read in and write out real and imaginary parts of matrix C.
      READ(1,11)((DCR(I,J),J=1,M),I=1,M)
      READ(1,11)((DCI(I,J),J=1,M),I=1,M)
 11   FORMAT(1X,5D15.5)
C Construct C from its real and imaginary parts.
      DO 12 I=1,M 
        DO 12 J=1,M
          C(I,J)=DCMPLX(DCR(I,J),DCI(I,J))
C Save original matrix C in CS.
          CS(I,J)=C(I,J)  
 12   CONTINUE
C Write out C.
      WRITE(6,13)
 13   FORMAT(1X,/,'Coefficient matrix C.')
      CALL OUTWRM(C,M,50)
      CALL SVD(C,U,S,VH,M,50)
C Reconstruct original matrix C from its SVD factors.
C Multiply U into S=W.
      DO 14 I=1,M
        DO 14 J=1,M
          CSS(I,J)=U(I,J)*S(J)
 14    CONTINUE
C Multiply result into VH.
      DO 15 I=1,M
        DO 15 J=1,M
          CP(I,J)=(0.D0,0.D0)
          DO 15 K=1,M
            CP(I,J)=CP(I,J)+CSS(I,K)*VH(K,J)
 15      CONTINUE
      WRITE(6,16)
 16   FORMAT(1X,/,'Reconstruction of matrix C=U.W.VH the product of its 
     1SVD factors.')
C Write out reconstructed matrix C.
      CALL OUTWRM(CP,M,50)
C Take Hermitian transposes of U and VH.
      DO 17 I=1,M
        DO 17 J=1,I
          HOLD1=U(I,J)
          HOLD2=VH(I,J)
          U(I,J)=DCONJG(U(J,I))
          VH(I,J)=DCONJG(VH(J,I))
          U(J,I)=DCONJG(HOLD1)
          VH(J,I)=DCONJG(HOLD2)
 17    CONTINUE
C Take product UH.C.V.
      CALL PRODM(U,C,VH,CS,M,50)
C Write out recovered matrix W.
      WRITE(6,18)
 18   FORMAT(1X,/,'Recovered matrix of singular values, W=UH.C.V.')
C Write out product UH.CS.V.
      CALL OUTWRM(C,M,50)
      END
C
      SUBROUTINE SVD(C,U,S,VH,M,MMAX)
C
C The subroutine SVD calls the subroutine BIDIAG to convert the complex
C matrix C to real, positive, upper bidiagonal form. The diagonal elements
C are stored in the vector S and its upper diagonal is stored in
C the vector UD. An implicit QR iteration with Wilkinson shift is then
C performed on the bidiagonal matrix through a series of Givens rotations
C to chase a rogue element, RE, from the matrix. After each QR iteration,
C the upper diagonal element in the leading row is tested for negligibility.
C The matrix is deflated when this element is found negligible, to
C computational accuracy, compared to its smallest diagonal neighbour,
C establishing a new leading row K. Similarly, a new trailing row, N, is
C established as the first row below the leading row with an upper diagonal
C element that is neglible compared to its smallest diagonal neighbour.
C The final iteration is on a 2 x 2 matrix. The singular values are on
C the diagonal of the final matrix. The singular values and their
C asociated eigenvectors are then sorted so that they are in descending
C order down the diagonal of the final matrix. The Hermitian transposes of
C the accumulated matrices U and VH are taken to convert them to
C the matrices appearing in the SVD factorization.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE COMPLEX C(MMAX,MMAX),U(MMAX,MMAX),VH(MMAX,MMAX),HOLD
      DIMENSION S(MMAX),UD(MMAX)
C Set maximum number of iterations.
      MAXI=100
      CALL BIDIAG(U,C,VH,M,MMAX)
C Put diagonal of real, positive bidiagonal matrix in S and its upper
C diagonal in UD.
      DO 10 I=1,M
        IP1=I+1 
        S(I)=DREAL(C(I,I))
        IF(IP1.GT.M) GO TO 10
        UD(I)=DREAL(C(I,IP1))
 10   CONTINUE    
C Begin diagonalization of matrix B.
C Set initial values of iteration number, ITER, starting row, K, and
C trailing row, N, of working matrix.
      ITER=0
      K=1
      N=M
C Continue reduction of working matrix.
 11   CONTINUE
C Advance iteration counter.
      ITER=ITER+1
      KP1=K+1
      NM1=N-1
C Find Wilkinson shift.
      P=S(N)
      Q=S(NM1)
      R=UD(NM1)
      H1=DSQRT((P+Q)*(P+Q)+R*R)
      H2=DSQRT((P-Q)*(P-Q)+R*R)
      SIG1=(H1-H2)/2.D0
C Find first Givens rotation.
      T11=(S(K)-SIG1)*(S(K)+SIG1)
      T12=S(K)*UD(K)
      CMAG=DSQRT(T11*T11+T12*T12)
      CO=T11/CMAG
      SI=-T12/CMAG
C Update unitary matrix VH.
      DO 12 I=1,M
        HOLD=VH(I,K)
        VH(I,K)=CO*HOLD-SI*VH(I,KP1)
        VH(I,KP1)=SI*HOLD+CO*VH(I,KP1)
 12   CONTINUE
C Post-multiply bidiagonal matrix B by rotation.
      H1=CO*S(K)-SI*UD(K)
      H2=CO*UD(K)+SI*S(K)
      S(K)=H1
      UD(K)=H2
C Find rogue element RE.
      RE=-SI*S(KP1)
      S(KP1)=CO*S(KP1)
C Find remaining Givens rotations.
      DO 13 I=K,NM1
      IP1=I+1
      IP2=I+2
      CMAG=DSQRT(S(I)*S(I)+RE*RE)
      CO=S(I)/CMAG
      SI=-RE/CMAG
C Update unitary matrix U.
      DO 14 J=1,M
        HOLD=U(I,J)
        U(I,J)=CO*HOLD-SI*U(IP1,J)
        U(IP1,J)=SI*HOLD+CO*U(IP1,J)
 14   CONTINUE
C Pre-multiply matrix B by rotation.
      H1=CMAG
      H2=CO*UD(I)-SI*S(IP1)
      H4=CO*S(IP1)+SI*UD(I)
      S(I)=H1
      UD(I)=H2
      S(IP1)=H4
      IF(IP2.GT.N) GO TO 13
      RE=-SI*UD(IP1)
      UD(IP1)=CO*UD(IP1)
C Find next Givens rotation.
      CMAG=DSQRT(UD(I)*UD(I)+RE*RE)
      CO=UD(I)/CMAG
      SI=-RE/CMAG
C  Update unitary matrix VH.
      DO 15 J=1,M
        HOLD=VH(J,IP1)
        VH(J,IP1)=CO*HOLD-SI*VH(J,IP2)
        VH(J,IP2)=SI*HOLD+CO*VH(J,IP2)
 15   CONTINUE
C Post-multiply matrix B by rotation.
      H1=CMAG
      H2=CO*S(IP2)
      H3=CO*S(IP1)-SI*UD(IP1)
      H4=CO*UD(IP1)+SI*S(IP1)
      UD(I)=H1
      S(IP1)=H3
      UD(IP1)=H4
      RE=-SI*S(IP2)
      S(IP2)=H2
 13   CONTINUE
C Find leading row of working matrix.
      J=K
      MM1=M-1
      DO 16 I=K,MM1
        IP1=I+1
C Find smallest neighbouring diagonal element.
        SMIN=DMIN1(DABS(S(I)),DABS(S(IP1)))
C Compare to see if upper diagonal element is negligible to machine precision.
        SIZE=SMIN+DABS(UD(I))
        IF(SIZE.NE.SMIN) GO TO 17
        J=J+1
 16   CONTINUE
 17   CONTINUE
C Set new leading row number.
      K=J
C Test to see if diagonalization is complete.
      IF(K.EQ.M) GO TO 20
C Find trailing row of working matrix.
      J=K+1
      KKP1=K+1
      DO 18 I=KKP1,MM1
        IP1=I+1
C Find smallest neighbouring diagonal element.
        SMIN=DMIN1(DABS(S(I)),DABS(S(IP1)))
C Compare to see if upper diagonal element is negligible.
        SIZE=SMIN+DABS(UD(I))
        IF(SIZE.EQ.SMIN) GO TO 19
        J=J+1
 18   CONTINUE
 19   CONTINUE
C Set new trailing row number.
      N=J
C Test for failure to converge.
      NNM1=N-1
      IF(KKP1.NE.KP1.OR.NNM1.NE.NM1) ITER=0
      IF(ITER.EQ.MAXI) PAUSE 'Maximum iterations reached.'
      GO TO 11
 20   CONTINUE
C Take Hermitian transpose of U and VH to get their final forms.
      DO 21 I=1,M
        DO 21 J=1,I
          HOLD=U(I,J)
          U(I,J)=DCONJG(U(J,I))
          U(J,I)=DCONJG(HOLD)
 21   CONTINUE
      DO 22 I=1,M
        DO 22 J=1,I
          HOLD=VH(I,J)
          VH(I,J)=DCONJG(VH(J,I))
          VH(J,I)=DCONJG(HOLD)
 22   CONTINUE
C Sort singular values into descending order, exchange columns of U
C and rows of VH.
      N=1
 23   CONTINUE
      IMAX=N
      DO 24 I=N,M
        IF(S(I).GE.S(IMAX)) IMAX=I
 24   CONTINUE
C Exchange singular values.
      HOLD=S(N)
      S(N)=S(IMAX)
      S(IMAX)=HOLD
C Exchange columns of U and rows of VH.
      DO 25 I=1,M
        HOLD=U(I,N)
        U(I,N)=U(I,IMAX)
        U(I,IMAX)=HOLD
        HOLD=VH(N,I)
        VH(N,I)=VH(IMAX,I)
        VH(IMAX,I)=HOLD
 25   CONTINUE  
      N=N+1
      IF(N.LE.M) GO TO 23    
      RETURN    
      END
C
      SUBROUTINE BIDIAG(U,C,VH,M,MMAX)
C
C The subroutine BIDIAG converts the complex M x M matrix C into upper
C bidiagonal form by a series of Householder unitary transformations,
C applied alternately, to columns by pre-multiplication and to rows by
C post-multiplication. Finally, rows are scaled by pre-multiplication
C with a unitary matrix and, alternately, columns are scaled by
C post-multiplication with a unitary matrix, to reduce the phase of diagonal
C elements to zero and, alternately, to reduce the phase of above diagonal
C elements to zero, producing an upper bidiagonal matrix with real and
C positive elements. Thus, the final, real and positive upper bidiagonal
C matrix B is found as B=U.C.VH, a unitary similarity transformation of C.
C B is returned in C. The unitary matrices U and VH are also returned by
C the subroutine.
C 
      DOUBLE COMPLEX C(MMAX,MMAX),V(MMAX),HU(MMAX),
     1U(MMAX,MMAX),VH(MMAX,MMAX),H0(MMAX),H1(MMAX),H2(MMAX),CONST
      DOUBLE PRECISION EMAG 
C Initialize unitary matrices U and VH.
      DO 10 I=1,M
        DO 11 J=1,M
          U(I,J)=(0.D0,0.D0)
          VH(I,J)=(0.D0,0.D0)
 11     CONTINUE
        U(I,I)=(1.D0,0.D0)
        VH(I,I)=(1.D0,0.D0)  
 10   CONTINUE
      write(6,100)
 100  format(1x,/,'Matrix C after operations on first column and row.')
C Apply Householder annihilation.
      MM1=M-1
      DO 12 N=1,MM1
C Put Nth column in V.
        DO 13 I=1,M
          V(I)=C(I,N)
 13     CONTINUE
C Annihilate Nth column.  
        CALL HHOLDER(V,HU,N,M,MMAX)
C Multiply Hermitian transpose of Householder vector HU
C into unitary matrix U and into transformed matrix B.
        DO 14 I=1,M
          H1(I)=(0.D0,0.D0)
          H2(I)=(0.D0,0.D0)
          DO 14 J=N,M
            H0(J)=DCONJG(HU(J))
            H1(I)=H1(I)+H0(J)*U(J,I)
            H2(I)=H2(I)+H0(J)*C(J,I)
 14     CONTINUE
C Update unitary matrix U and transformed matrix B.
        DO 15 I=N,M
          DO 15 J=1,M
            U(I,J)=U(I,J)-2.D0*HU(I)*H1(J)
            C(I,J)=C(I,J)-2.D0*HU(I)*H2(J)
 15     CONTINUE
C Overwrite annihilated Nth column.
        DO 16 I=1,M
          C(I,N)=V(I)
 16     CONTINUE  
        IF(N.EQ.MM1) GO TO 12       
C Put Nth row in V.
        DO 17 J=1,M
          V(J)=C(N,J)
 17     CONTINUE
        NP1=N+1
C Annihilate Nth row.
        CALL HHOLDER(V,HU,NP1,M,MMAX)
C Multiply unitary matrix VH and transformed matrix C
C into Householder vector HU.
        DO 18 I=1,M
          H1(I)=(0.D0,0.D0)
          H2(I)=(0.D0,0.D0)
          DO 18 J=NP1,M
            H0(J)=DCONJG(HU(J))
            H1(I)=H1(I)+VH(I,J)*H0(J)
            H2(I)=H2(I)+C(I,J)*H0(J)
 18     CONTINUE
C Update unitary matrix VH and transformed matrix B.
        DO 19 I=1,M
          DO 19 J=NP1,M
            VH(I,J)=VH(I,J)-2.D0*H1(I)*HU(J)
            C(I,J)=C(I,J)-2.D0*H2(I)*HU(J)
 19     CONTINUE    
C Overwrite annihlated Nth row.
        DO 20 J=1,M
          C(N,J)=V(J)
 20     CONTINUE
      if(n.eq.1) call outwrm(c,m,50)     
 12   CONTINUE
      write(6,101)
 101  format(1x,/,'Matrix C after reduction to complex upper bidiagonal 
     1form.')
      call outwrm(c,m,50)
C Convert upper diagonal matrix B to a real, positive matrix.
      N=1
      CONST=(1.D0,0.D0)
 21   CONTINUE
      NP1=N+1
C Find magnitudes of current row elements.
      EMAG=DREAL(C(N,N)*DCONJG(C(N,N)))
      EMAG=DSQRT(EMAG)
C Update unitary matrix U.
      CONST=DCONJG(CONST)*DCONJG(C(N,N))/EMAG
C Scale current row.
      DO 22 I=1,M
        U(N,I)=CONST*U(N,I)
 22   CONTINUE
C Replace diagonal element.  
      C(N,N)=DCMPLX(EMAG,0.D0)
      IF(N.EQ.M) GO TO 23
      EMAG=DREAL(C(N,NP1)*DCONJG(C(N,NP1)))
      EMAG=DSQRT(EMAG)
C Update unitary matrix VH.
      CONST=DCONJG(CONST)*DCONJG(C(N,NP1))/EMAG
C Scale next column.
      DO 24 I=1,M
         VH(I,NP1)=CONST*VH(I,NP1)
 24   CONTINUE
C Replace above diagonal element.   
      C(N,NP1)=DCMPLX(EMAG,0.D0)
      N=N+1
      GO TO 21
 23   CONTINUE
      write(6,102)
 102  format(1x,/,'Matrix C after conversion to real upper bidiagonal fo
     1rm.')
      call outwrm(c,m,50) 
      RETURN
      END
C
      SUBROUTINE HHOLDER(V,U,N,M,MMAX)
C
C The subroutine HHOLDER annihilates the components of the vector V(M)
C beyond the Nth component for N<M. The Householder vector U is calculated
C and the annihilated vector W replaces V on return.
C
      DOUBLE COMPLEX U(MMAX),V(MMAX),W(MMAX)
      DOUBLE PRECISION ETA,AMAG,VWMAG,VR,VI
C No modification for N=M.
      IF(N.EQ.M) GO TO 10
C Find value of ETA.
      ETA=0.D0
      DO 11 I=N,M
        ETA=ETA+DREAL(V(I)*DCONJG(V(I)))
 11   CONTINUE
      ETA=DSQRT(ETA)
C Find magnitude and real and imaginary parts of V(N).
      AMAG=CDABS(V(N))
      VR=DREAL(V(N))
      VI=DIMAG(V(N))
C Find magnitude of V-W.
      VWMAG=2.D0*ETA*(ETA+AMAG)
      VWMAG=DSQRT(VWMAG)
C Construct vector U.
C Set initial values of components of U to zero.
      DO 12 I=1,M
        U(I)=(0.D0,0.D0)
 12   CONTINUE
C Set value of U(N).
      U(N)=V(N)+ETA*DCMPLX(VR/AMAG,VI/AMAG)
C Set values of remaining components of U.
      NP1=N+1
      DO 13 I=NP1,M
        U(I)=V(I)
 13   CONTINUE
C Normalize U to unit vector.
      DO 14 I=1,M
        U(I)=U(I)/VWMAG
 14   CONTINUE
C Initialize vector W as vector V.
      DO 15 I=1,M
        W(I)=V(I)
 15   CONTINUE  
C Construct vector W.
C Set value of W(N).
      W(N)=W(N)-VWMAG*U(N)
C Zero remaining components of W.
      DO 16 I=NP1,M
        W(I)=(0.D0,0.D0)
 16   CONTINUE
C Replace V by W.
      DO 17 I=1,M
        V(I)=W(I)
 17   CONTINUE  
 10   CONTINUE
      RETURN
      END
C
      SUBROUTINE OUTWRM(A,M,MMAX)
C OUTWRM writes out the real and imaginary parts of the matrix A
C to the screen. 
      INTEGER M,MMAX
      DOUBLE COMPLEX A(MMAX,MMAX)
c      WRITE(6,10)
 10   FORMAT(1x,/)
      WRITE(6,11)
 11   FORMAT(1X,'real part of matrix is,')
      WRITE(6,12)((DREAL(A(I,J)),J=1,M),I=1,M)
 12   FORMAT(1X,5D15.8)
      WRITE(6,13)
 13   FORMAT(1X,'imaginary part of matrix is,')
      WRITE(6,12)((DIMAG(A(I,J)),J=1,M),I=1,M)
      RETURN
      END
C
      SUBROUTINE OUTWR(UD,S,M,MMAX)
C OUTWR writes out the upper diagonal UD and the diagonal S of the real
C bidiagonal matrix B.
      INTEGER M,MMAX 
      REAL*8 UD(MMAX),S(MMAX)
      write(6,10)
 10   format(1x,'The final form of the matrix B.')
      WRITE(6,11)
 11   FORMAT(1X,'upper diagonal of B is,')
      WRITE(6,12)(UD(I),I=1,M)
 12   FORMAT(1X,5D15.8)
      WRITE(6,13)
 13   FORMAT(1X,'diagonal of B is,')
      WRITE(6,12)(S(I),I=1,M)
      RETURN
      END
C
      SUBROUTINE PRODM(U,C,VH,CS,M,MMAX)
C PRODM finds the product C=U.CS.VH.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE COMPLEX C(MMAX,MMAX),CS(MMAX,MMAX),U(MMAX,MMAX),
     1VH(MMAX,MMAX)
      DO 10 I=1,M
        DO 10 J=1,M 
          C(I,J)=(0.D0,0.D0)
          DO 10 K=1,M
            DO 10 L=1,M
            C(I,J)=C(I,J)+U(I,K)*CS(K,L)*VH(L,J)
 10   CONTINUE
      RETURN
      END      
