C                                   PROGRAMME TORQUE.FOR
C
C TORQUE.FOR calculates the quantities needed to find the gravity
C restoring torque on a tilted Inner Core. These include
C the difference between the axial and equatorial moments of inertia
C and the radial derivative of the flattening.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION R(100),RHO(100),RHOB(100),F(100),RI(100),RHOI(100),
     1 RHOBI(100),FI(100),FPI(100),B(98,198),C(100,100),Q(100),CMA(100),
     2 ENAME(10),W(6),X(6)
      DOUBLE PRECISION MASS(100),INT(100,4),MASSIP,MASSC
C Open Inner Core data file.
      OPEN(UNIT=1,FILE='innerc.dat',STATUS='OLD')
C Open torque file.
      OPEN(UNIT=2,FILE='torque.dat',STATUS='UNKNOWN')
C Set value of pi.
      PI=3.141592653589793D0
C Set value of universal constant of gravitation (CODATA 2006).
      G=6.67428D-11
C Set angular frequency of Earth's rotation (WGS84).
      WE=7.292115D-5
C Set maximum dimensions for interpolation.
      M1=100
      M2=198
      M3=98
C Read in and write out Earth model name.
      READ(1,10)(ENAME(I),I=1,10)
 10   FORMAT(10A7)
      WRITE(2,11)(ENAME(I),I=1,10)
 11   FORMAT(11X,10A7)
C Read in number of Inner Core model points.
      READ(1,12)NM
 12   FORMAT(1X,I9)
C Read in density at the bottom of the Outer Core.
      READ(1,13)RHOBOC
 13   FORMAT(1X,D15.8)
C Read in Inner Core model data.
      READ(1,14)(R(I),RHO(I),RHOB(I),F(I),I=1,NM)
 14   FORMAT(1X,4D15.8)
C Set up interpolation for Inner Core.
      N1=NM
      N2=2*N1-2
      N3=N1-2
C Enter number points in model of the Inner Core.
      WRITE(6,15)
 15   FORMAT(/,/,'Select number of model points for Inner Core.')
      READ(5,*)NS
      CALL SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C Interpolate onto equally-spaced Inner Core model points.
      NSM1=NS-1
C Find space between model points.
      H=R(NM)/DFLOAT(NSM1)
C Set geocentre values.
      RI(1)=R(1)
      RHOI(1)=RHO(1)
      RHOBI(1)=RHOB(1)
      FI(1)=F(1)   
C Construct new equally-spaced model.
      DO 16 I=1,NSM1
         IP1=I+1
         RI(IP1)=H*DFLOAT(I)
         CALL INTPL(RI(IP1),RHOI(IP1),NM,C,R,RHO,M1)
         CALL INTPL(RI(IP1),RHOBI(IP1),NM,C,R,RHOB,M1)
         CALL INTPL(RI(IP1),FI(IP1),NM,C,R,F,M1)
         CALL DERIV(RI(IP1),FPI(IP1),NM,C,R,F,M1)
 16   CONTINUE
C Begin integration for the calculation of the difference between the
C axial and equatorial moments of inertia of the Inner Core.
C Set weights and abscissae for six point Guassian integration.
      W(1)=0.171324492379170D0
      W(2)=0.360761573048139D0
      W(3)=0.467913934572691D0
      W(4)=W(3)
      W(5)=W(2)
      W(6)=W(1)
      X(1)=-0.9324695142031521D0
      X(2)=-0.6612093864662649D0
      X(3)=-0.2386191860831970D0
      X(4)=-X(3)
      X(5)=-X(2)
      X(6)=-X(1)
C Set exponent in integrand.
      EX=4.D0
C Initialize starting value of integral INT.
      INT(1,1)=0.D0
C Form integrand function Q.
      DO 17 I=1,NS
         Q(I)=RHOI(I)*(5.D0*FI(I)+RI(I)*FPI(I))
 17   CONTINUE
C Perform quadrature.
      CALL QUAD(INT,EX,Q,1,W,X,NS,C,RI,1,M1)
C Calculate difference between axial and equatorial moments of inertia.
      DO 18 I=1,NS
         CMA(I)=8.D0*PI*INT(I,1)/15.D0
 18   CONTINUE
C Calculate enclosed mass as a function of radius.
      DO 19 I=1,NS
         MASS(I)=4.D0*PI*(RI(I)**3)*RHOBI(I)/3.D0
 19   CONTINUE
C Calculate mass of the Inner Core as though its density was that
C at the bottom of the Outer Core.
      MASSIP=4.D0*PI*(RI(NS)**3)*RHOBOC/3.D0
C Find coefficient of gravity restoring torque.
      GAMMA=2.d0*CMA(NS)*(G*MASS(NS)*(2.D0*FI(NS)+RI(NS)*
     1 FPI(NS))/(RI(NS)**3)+3.D0*G*MASSIP*FI(NS)/(RI(NS)**3)
     2 +5.D0*(OMEGA**2)/2.D0)/5.D0
C Write out radius, density, 1/f, f', C-A, and mass for Inner Core.
C Write out headings.
      WRITE(2,20)
      WRITE(6,20)
 20   FORMAT(/,/,10X,'Radius',8X,'Density',5X,'1/f',5X,'df/dr ',5X,
     1 'C-A',7X,'Mass')
      WRITE(2,21)
      WRITE(6,21)
 21   FORMAT(11X,'(km)',9X,'(gm/cc)',11X,'(10^-11',4X,
     1 '(10^30',3X,'(10^21 kg)')
      WRITE(2,22)
      WRITE(6,22)
 22   FORMAT(47X,'/m)',3X,'kg.m^2)')
C Scale and write out values.
      DO 23 I=1,NS
         RAD=RI(I)/1000.D0
         DEN=RHOI(I)/1000.D0
         RECIPF=1.D0/FI(I)
         FP=FPI(I)*1.D11
         CMAS=CMA(I)*1.D-30
         MASSC=MASS(I)*1.D-21
         WRITE(2,24)RAD,DEN,RECIPF,FP,CMAS,MASSC
         WRITE(6,24)RAD,DEN,RECIPF,FP,CMAS,MASSC
 24      FORMAT(1X,F15.1,F15.3,F10.2,F8.3,2F10.2)
 23   CONTINUE
C Write out gravity torque coefficient GAMMA.
      WRITE(2,25)GAMMA
      WRITE(6,25)GAMMA
 25   FORMAT(/,/,5X,'GAMMA=',1PD8.2,X,'N.m')
C Write out equivalent mass of the Inner Core as though density is that
C at bottom of the Outer Core.
      WRITE(2,26)MASSIP
      WRITE(6,26)MASSIP
 26   FORMAT(/,/,5X,'MASSIP=',1PD8.2,X,'kg')
C Write out density at the bottom of the Outer Core.
      RHOBOC=RHOBOC/1000.D0
      WRITE(2,27)RHOBOC
      WRITE(6,27)RHOBOC
 27   FORMAT(/,/,5X,'density at bottom of Outer Core=',2PD10.4,X,
     1 'gm/cc')
      END
C
      SUBROUTINE QUAD(INT,EX,Q,NIS,W,X,N1,C,R,M,M1)
C
C This subroutine performs the quadrature integral of a function, Q, of mean
C radius, from the centre of the Earth through the Inner Core for M=1, or the
C Outer Core for M=2, or the Mantle for M=3, or the Crust for M=4. The
C result is returned in INT. The integration uses six point Guassian integration
C which is eleventh order accurate (exact for polynomials up to and including
C degree eleven). Interpolation of Q to the Guassian integration points is
C performed by the spline interpolator subroutine INTPL. The number of
C integration steps between Earth model points is specified by NIS.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION Q(M1),R(M1),C(M1,M1),W(6),X(6),RI(6),
     1 QI(6)
      DOUBLE PRECISION INT(M1,4)
      N1M1=N1-1
      DO 10 I=1,N1M1
        IP1=I+1
C Find half increment in radius.
        H=(R(IP1)-R(I))/(2.D0*DFLOAT(NIS))
C Set sum to zero.
        SUM=0.D0
C Integrate over subinterval.
        DO 11 J=1,NIS
          JM1=J-1 
C Find mean radius.
          RM=R(I)+2.D0*DFLOAT(JM1)*H+H
C Find radii for Guassian integration.
          DO 12 K=1,6
            RI(K)=H*X(K)+RM
C Interpolate integrand at Guassian abscissae.
            CALL INTPL(RI(K),QI(K),N1,C,R,Q,M1)
C Accumulate sum.
            SUM=SUM+W(K)*QI(K)*((RI(K))**EX)
 12       CONTINUE
 11     CONTINUE
C Find contribution of current increment.
        INT(IP1,M)=INT(I,M)+H*SUM
 10   CONTINUE
      RETURN
      END
C
      SUBROUTINE SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C
C SPMAT computes the N1 by N1 matrix C connecting
C second derivatives to function values at N1 nodal points R(I).
C B is the N3=N1-2 by N2=2*N1-2 augmented matrix used internally
C in the Gaussian elimination procedure.
C M1, M2, M3 are the respective maximum dimensions for N1, N2, N3
C specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),B(M3,M2)
      N=N3+1
C Construct augmented matrix A.
      DO 11 I=1,N3
        DO 10 J=1,N2
          B(I,J)=0.D0
  10    CONTINUE
        B(I,I)=2.D0*(R(I+2)-R(I))
        B(I,I+N3)=1.D0
        IF(I.EQ.1.OR.I.EQ.N3) GO TO 11
        B(I,I-1)=R(I+1)-R(I)
        B(I,I+1)=R(I+2)-R(I+1)
  11  CONTINUE
C Apply conditions that third derivatives are constant
C in first two and last two intervals.
      B(1,2)=R(3)-R(2)-((R(2)-R(1))**2.D0)/(R(3)-R(2))
      B(N3,N3-1)=R(N3+1)-R(N3)-((R(N1)-R(N1-1))**2.D0)/(R(N1-1)-R(N1-2))
      B(1,1)=B(1,1)+((R(3)-R(1))*(R(2)-R(1)))/(R(3)-R(2))
      B(N3,N3)=B(N3,N3)
     1 +((R(N1)-R(N1-1))*(R(N1)-R(N1-2)))/(R(N1-1)-R(N1-2))
      NM1=N3-1
C Eliminate sub-diagonal.
      DO 13 I=1,NM1
        DIAG=B(I,I)
        ELL=B(I+1,I)
        DO 12 J=1,N2
          B(I,J)=B(I,J)/DIAG
          B(I+1,J)=B(I+1,J)-B(I,J)*ELL
  12    CONTINUE
  13  CONTINUE
C Eliminate super-diagonal.
      DO 15 I=1,NM1
        NO=N-I
        DIAG=B(NO,NO)
        ELL=B(NO-1,NO)
        DO 14 J=1,N2
          B(NO,J)=B(NO,J)/DIAG
          B(NO-1,J)=B(NO-1,J)-B(NO,J)*ELL
  14    CONTINUE
  15  CONTINUE
C Move inverse to position formerly occupied by A.
      DO 17 I=1,N3
        DO 16 J=1,N3
          B(I,J)=B(I,J+N3)
  16    CONTINUE
  17  CONTINUE
C Construct matrix B in N-2 by N augmentation space. 
      DO 19 I=1,N3
        DO 18 J=1,N1
          B(I,J+N3)=0.0D0
  18    CONTINUE
        IN=I+N3
        FACTOR=6.D0/((R(I+2)-R(I+1))*(R(I+1)-R(I)))
        B(I,IN)=FACTOR*(R(I+2)-R(I+1))
        B(I,IN+1)=-FACTOR*(R(I+2)-R(I))
        B(I,IN+2)=FACTOR*(R(I+1)-R(I))
  19  CONTINUE
C Multiply inverse of A into B to get N-2 by N version of C.
      DO 22 I=1,N3
        DO 21 J=1,N1
          SUM=0.D0
          DO 20 K=1,N3
            SUM=SUM+B(I,K)*B(K,J+N3)
  20      CONTINUE
          C(I+1,J)=SUM
  21    CONTINUE
  22  CONTINUE
C Add first and last rows to C.
      DO 23 J=1,N1
        C(1,J)=C(2,J)*(R(3)-R(1))/(R(3)-R(2))
     1 -C(3,J)*(R(2)-R(1))/(R(3)-R(2))
        C(N1,J)=C(N1-1,J)*(R(N1)-R(N1-2))/(R(N1-1)-R(N1-2))
     1 -C(N1-2,J)*(R(N1)-R(N1-1))/(R(N1-1)-R(N1-2))
  23  CONTINUE
      RETURN
      END
C
      SUBROUTINE INTPL(XV,YV,N1,C,R,S,M1)
C
C INTPL interpolates to find YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is
C the coefficient matrix, from subroutine SPMAT, relating
C the second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second deivative SUM2 at end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE
C Find interpolate YV at XV.
      YV=S(J)+(S(J+1)-S(J))*(XV-R(J))/(R(J+1)-R(J))
     1  +(XV-R(J))*(XV-R(J+1))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     2  +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END
C
      SUBROUTINE DERIV(XV,YV,N1,C,R,S,M1)
C
C DERIV calculates the derivative YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is
C the coefficient matrix, from subroutine SPMAT, relating
C the second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second derivative SUM2 at the end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE   
C Find derivative YV at XV.
      YV=(S(J+1)-S(J))/(R(J+1)-R(J))
     1  +(XV-R(J))*(XV-R(J+1))*(SUM2-SUM1)/(6.D0*(R(J+1)-R(J)))
     2  +(2.D0*XV-R(J+1)-R(J))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     3  +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END




 


